/*****************************************************************************/
/*                                                                           */
/*  HyperPanel OS                       File "stm32wl/src/board_stm32wl.c"   */
/*  IO container profile for stm32wl - STM32WL55JC - Nucleo-64.              */
/*                                                                           */
/*  Author : TE/FA                      Last modification : 08/13/2024       */
/*                                                                           */
/*****************************************************************************/

 
/*  Some explanations ========================================================



    1    Purpose of this module
    ___________________________


         This module contains the IO container data configuration, the 
parameters for the VMIO drivers for the STM32WL Nucleo-64 board. Data includes
const data (parameters) and RAM data (drivers context tables). The processor
is a STM32WL55JC. The file contains the following items:

#ARRAY ------------------------------------------------------------------------
         - Driver system parameters and RAM data

         - Character drivers parameters and RAM data

         - Index tables and data

         - A very limited number of board specific C procedures
#END --------------------------------------------------------------------------

    Reference documents for this board are:

#ARRAY ------------------------------------------------------------------------
        - RM0453 Rev 5 January 2023 STM32WL5x
          stm-rm0453-stm32wl5x.pdf

        - DSI13293 Rev5 December 2022 STM32WL55xx STM32WL54xx
          stm-datasheet-stm32wl5x.pdf
#END --------------------------------------------------------------------------

   The MCU is:

#ARRAY ------------------------------------------------------------------------
        - STM32WL55JC (DS13293 - page 2)

        - STM32 : Arm based 32-bit microcontroller
        - WL    : Wireless long range
        - 55    : Cortex-M4, Cortex-M0+, full set of modulations
        - J     : STM32 package pin count: 73 pins
        - C     : STM32 Flash memory size: 256Kbyte
#END --------------------------------------------------------------------------



    2    User manual
    ________________


    2.1  Contents of the present file
    2.2  Static const data overall schematic
    2.3  System drivers
    2.4  Character drivers
    2.5  Block drivers
    2.6  Volume drivers
    2.7  Protocols
    2.8  Services
    2.9  Interrupt handlers


    2.1  Contents of the present file

         Any board is described with a C structures tree, the root structure is
always of type "Board_link". A same file may contains a number of "Board_link"
structures, each corresponding to a different board, allowing a same binary OS
file to be used with variants of a board.


    2.2  Static const data overall schematic

         The diagram which follows is a copy of the schematic one will find at
"2.9.1  General Overview" of "drv_req1.c". So, for more information, one will
find it in "drv_req1.c".


#DIAGRAM ----------------------------------------------------------------------
===============================================================================
INT            Board_link *                           Board descriptions
num_board      boards[]                               See 2.9.2, drv_req1.c
+--------+     +--------+
|        | -+  |        |
+--------+  |  +--------+
            |  |        |      Board_link
            |  +--------+      +------------+
            +->| ADBOARD| -+-> | board_name |
               +--------+  |   | drv_link   |-----+
               |        |  |   +------------+     |
               +--------+  |   A Board_link       |
               |        |  |   structure          |
               +--------+  |                      |
               |        |  |                      |
               +--------+  |                      |
    Table of addresses of  |                      |
    Board_link structures  |                      |
                           |                      |
   Board_link* +--------+  |                      |
        board  | ADBOARD| -+                      |
               +--------+                         |
                                                  |
                                                  |
   +----------------------------------------------+
===|===========================================================================
   |   Drv_link * []                                 System driver description
   |   +-----------+                                       See 2.9.3 and 2.9.9
   +-> |           |     Sdrv_link          Sdrv_desc            of drv_req1.c
       +-----------+     +-----------+      +----------+
       |           | --> | sdrv_desc | ---> | drv_name |
       +-----------+     | dev_link  | -+   | typdrv   |
       |           |     | nb_dev    |  |   | init     | --> proc C
       +-----------+     +-----------+  |   | end      | --> proc C
       |           |      A Sdrv_link   |   +----------+
       +-----------+      structure     |
       |           |                    |   Dev_link[]
       +-----------+                    |   +----------+      +-----+
       |           |                    +-> | dev_name |      |     | -> proc C
       +-----------+                        | dev_desc | ---->|     | -> proc C
       |           |                        | dev_def  | ---+ |     | -> proc C
       +-----------+                        | drv_def  | -+ | |     | -> proc C
       |           |                        +----------+  | | |     | -> proc C
       +-----------+                        | dev_name +  | | |     | -> proc C
       |           |                        | dev_desc |  | | |     | -> proc C
       +-----------+                        | dev_def  |  | | +-----+
       |           |                        | drv_def  |  | |
       +-----------+                        +----------+  | |
       |           |                        | dev_name |  | | +-----+
       +-----------+                        | dev_desc |  | +>|     |
       |           |                        | dev_def  |  |   |     |
       +-----------+                        | drv_def  |  |   +-----+
       |           |                        +----------+  |
       +-----------+                                      |   +-----+
       |           |                                      +-->|     |
       +-----------+                                          |     |
       |           |                                          +-----+
====== +-----------+ ==========================================================
       |           |                                  Driver/protocol/service/
       +-----------+                                  volume driver description
       |           |                                  See 2.9.4 and 2.9.10    
       +-----------+
       |           |     Drv_link          Drv_desc
       +-----------+     +----------+      +----------+
       |           | --> | drv_desc | ---> | drv_name |
       +-----------+     | dev_link | -+   | typdrv   |
       |           |     | nb_dev   |  |   | initdone | --> Init. flag
       +-----------+     +----------+  |   | init     | --> C proc
       |           |     A Drv_link    |   | autnum   | --> VMIO automaton num.
       +-----------+     structure     |   | lw0      | --> REQ_ALLOC LW 
       |     0     |                   |   | nbvl     | --> Total nb of LW
       +-----------+                   |   | nbiod    | --> Nb of user LW
                                       |   | nbmodd   | --> Nb of inits.
  Table of addresses of                |   | tlcm     | --> Telecom/pipe Itf
  Drv_link structures                  |   | adr      | --> User address
                                       |   +----------+
                                       |
                                       |   Dev_link[]
                                       |   +----------+
                                       +-> | dev_name |      +-----+
                                           | dev_desc | ---->|     | -> proc C
                                           | dev_def  | --+  |     | -> proc C
                                           | drv_def  |   |  |     | -> proc C
                                           +----------+   |  |     | -> proc C
                                           | dev_name |   |  +-----+
                                           | dev_desc |   |
                                           | dev_def  |   |
                                           | drv_def  |   |  +-----+
                                           +----------+   +->|     |
                                           | dev_name |      |     |
                                           | dev_desc |      +-----+
                                           | dev_def  |
                                           | drv_def  |
                                           +----------+
#END --------------------------------------------------------------------------


    2.3  System drivers

         The VMIO driver table is a list, a one dimensionnal array of 
Sdr_linl/Drv_link structure addresses. With system drivers, the used structure
is always of type "Sdrv_link". For any board, the contents of both the 
Sdrv_link and Drv_link structures are depending from the hardware and this is
the reason why they are located in the present board configuration file. Each
"Sdrv_link" holds a pointer to one structure of type "Sdrv_desc", which is not
defined in the present file and also holds a pointer to an array of "Dev_link"
structures, which is defined in the presnet configuration file.

typedef struct                         /+ System driver description          +/
  BEGIN                                /+ ---------------------------------- +/
    const Sdrv_desc *sdrv_desc       ; /+ Driver interfaces address          +/
    const Dev_link  *dev_link        ; /+ Device interfaces address          +/
    UCHAR           nb_dev           ; /+ Number of devices in sdev_link[]   +/
  END               Sdrv_link        ; /+ ---------------------------------- +/

typedef struct                         /+ Character driver description       +/
  BEGIN                                /+ ---------------------------------- +/
    const Drv_desc  *drv_desc        ; /+ Driver interface address           +/
    const Dev_link  *dev_link        ; /+ Devices interfaces list addresses  +/
    UCHAR           nb_dev           ; /+ Number of devices in dev_link[]    +/
    const Pipe_desc *pipe            ; /+ Pipe interface address             +/
  END               Drv_link         ; /+ ---------------------------------- +/


         The "Sdrv_desc" structure is an interface that every system driver
must provide. This structure is made of 4 fields. The address of a character
string, which is the driver name, a "typdrv" integer which value is always 
DRV_SYS (0) for every system driver, the "init" address of a driver 
initialization procedure, and the "end" address of the driver termination
procedure. Each system driver source code file has one own "sdrv_desc"
structure.

typedef struct                         /+ VMIO system driver interface       +/
  BEGIN                                /+ ---------------------------------- +/
    const CHAR  *drv_name            ; /+ Driver name                        +/
    UCHAR       typdrv               ; /+ Always 0 with a system driver      +/
                                       /+ Interface procedures ------------- +/
    IWORD       (*init)()            ; /+ Initialization procedure           +/
    IWORD       (*end )()            ; /+ Termination procedure              +/
  END           Sdrv_desc            ; /+ ---------------------------------- +/


         The "Dev_link" structures are also board dependant and so are defined
within the prsent file. The "dev_desc" field of each "Dev_link" may point to
data structures which are not defined in the present file but, most of the time,
inside the source file of a low-level driver.

typedef struct                         /+ Device interface                   +/
  BEGIN                                /+ ---------------------------------- +/
    const CHAR    *dev_name          ; /+ System device name (unused)        +/
    const void    *dev_desc          ; /+ Low-level procedure addresses      +/
    const void    *dev_def           ; /+ Low-level parameters addresses     +/
    const void    *drv_def           ; /+ High-level parameters addresses    +/
  END             Dev_link           ; /+ ---------------------------------- +/


    2.4  Character drivers

         The VMIO driver table is a list of "Sdrv_link" and "Drv_link" 
addresses. With character drivers, "Drv_link" structures are used. The contents
of those are dependant from the board and so are defined in the present file.
Each "Drv_link" structure holds one pointer to a "Drv_desc" structure, which is
not defined in the present module but is the source file of the character
driver.

         Each character driver, but also each protocol or service has to 
provide, to export one "Drv_desc" structure, which is its interface. The two
first fields of the "Drv_desc" structure (see "board.h") are same as the 
"Sdrv_desc" ("drv_name" and "typdrv")) but the "Drv_desc" structure is made of
much more fields than "Sdrv_desc". Each C source file which is a character
driver contains and exports its own "Drv_desc" structure.

         The "Dev_link" structures are board dependant and are defined in the
present file. The "dev_desc" field of those is most of the time pointing to
a structure we will find in a low-level (hardware dependant code) module.


    2.5  Block drivers

         There is currently no block driver in the present configuration


    2.6  Volume drivers

         There is currently no volume driver, no file system, in the present
confiuration file.


    2.7  Protocols

         The VMIO driver table is a list of "Sdrv_link" and "Drv_link" 
addresses. With protocol drivers, "Drv_link" structures are used. The contents
of those are dependant from the board and so are defined in the present file.
Each "Drv_link" structure holds one pointer to a "Drv_desc" structure, which is
not defined in the present module but is the source file of the protocol
driver. There is currently no protocol in the present configuration file.


    2.8  Services

         There is currently no service in the present configuration file.


    2.9  Interrupt handlers

         One will find in "9.1.5  VMIT driver - vmit_sdrv_link" a table named
"ithand", which contains all the adresses of all the interrupt handlers. The
IRQ handler codes are located in low-level, hardware dependant modules. Within
the "ithand" table, the handler name is followed by two integer's which are the
two input parameters given to the handler procedure when called.



    3    Porting guide
    __________________

         The present file holds mainly data that indeed do not require anny
low-level code. The few procedure at the end of this file do not require any
specific dedicated low-level code. For example, "setLED" calls 
"set_output_stm32", which is indeed a low-level procedure, but this procedure 
is not a "request", a need  from the present module, but from "drv_gpio.c".


    4    Include files and external references
    __________________________________________
*/

#include <stdtyp.l>                    /* Definitions et constantes Cojyp.   */
#include <event.l>                     /* IND_DATA                           */
                                       /*                                    */
#include <aut.h>                       /* S_trans, S_evt                     */
#include <board.h>                     /* Def_link                           */
#include <drv.h>                       /* Sys_driver                         */
#include <drv_ram.h>                   /* Seg_def Part_def                   */
#include <vmit.h>                      /* Ithand_t                           */
#include <vmio.h>                      /* vmio()                             */
#include <vmk.h>                       /* set_idle                           */
#include <req_lib.h>                   /* Req_ctx                            */
#include <vmiodbg.h>                   /* Note_it                            */
                                       /*                                    */
#include <drv_gpio.h>                  /* Gpio_desc                          */
#include <drv_req.h>                   /* Itf_ctx                            */
#include <drv_dma.h>                   /* Dma_itproc                         */
#include <drv_i2c.h>                   /* I2c_desc                           */
#include <drv_asy.h>                   /* Asy_desc                           */
#include <drv_rad.h>                   /* Rad_desc                           */
                                       /*                                    */
#include <stm32.h>                     /* Low level structures               */
#include <boot_stm32.h>                /* PRO_POKE                           */
#include <signal_vmit.h>               /* GPIO_CPU                           */
#include <dma_stm32.h>                 /* Ref_dma32                          */
#include <sys_stm32.h>                 /* sdrv_syswl                         */
#include <gpio_stm32.h>                /* handlerGPIO_stm32                  */
#include <tim_stm32.h>                 /* handlerTIM_stm32                   */
#include <asy_stm32.h>                 /* handlerASY_stm32                   */
#include <i2c_stm32.h>                 /* handlerevI2C_stm32                 */
#include <rad_stm32.h>                 /* handlerRAD_stm32                   */
                                       /*                                    */
#define STM32WL55JC                    /* Needed by <defs_stm32.h>           */
#include <defs_stm32.h>                /* XYZ_IRD defines'                   */
                                       /*                                    */
#define BOARD_XYZ_DATA_C               /* Driver space/data                  */

/*
    5    Inner workings of the module
    _________________________________



    6    Exported (public) variables
    ________________________________

         6.1  Board_link structure - boards
         6.2  Preinit program for "boot_stm32.s" - preinit_stm32wl
         6.3  Timer numbers - sys_tim and flt_tim
         6.4  Timers reference table - ref_tim32
         6.5  DMA channels/streams reference table - ref_dma32
         6.6  Debug table for vmit.c - tnote_it[]
         6.7  Debug variable for GDB/IDLE mode - use_gdb


     Some drivers may need global variables we define in the present file, such
as the "led_tsubcnt[]" table for "drv_led.c". So strictly, those are exported 
(public) variables. However, we deliberately did not choose to put those here, 
we preferred to put them with the driver const data in section 9
   

    6.1  "Board_link" structure - boards
    ------------------------------------

    The "main" procedure of "vmk_emb.c" sets a first global variable named
    "num_board", in our case always to 0, and then sets another global variable
    named "board" to the address of "boards[num_board]", so in our case "board"
    is always set to the address of "board_wl55"
*/

extern Board_link board_wl55         ; /* Forward reference                  */

Board_link *boards[] =                 /* List of "Board_link" addresses     */
  BEGIN                                /* ------------ |MC  |CPU| RAM|Flash| */
    &board_wl55                      , /* 0 STM32WL55  |WL55|  0| 64K| 256K| */
  END                                ; /* ---------------------------------+ */


/*  6.2  preinit_stm32wl program for boot_stm32.s
    ---------------------------------------------

    The USR1 button is connected on PA0, there is no external pull up or pull 
    down. The pin has to be configured as an input with a weak pull-up. The 
    signal is high when the button is not pressed, the bit value read in the 
    input register is 1. When the button is pressed, the signal goes low and 
    we read a 0 in the data input register.

    The "preinit_stm32wl" program read the USR1 button and if the button is not 
    pressed (PA0 state is 1) a 0,5 millisecond positive pulse is send ON PA2
    and the program terminates. If the USR1 button is pressed then a positive 
    pulse of 1 millisecond on PA2 (This will be the TX signal for the USART1) 
    and then a stop on breakpoint is executed

    So if we press the USR1 button and then, without releasing the USR1 button,
    we press the RST button then the boot will break. This provides us 
    the opportunity to use the GDB CTRL-C command, set break-points, issue 
    a GDB "c" command and then continue.

    What this "preinit_stm32wl" does it as follows :

      - Enable the clock for GPIO bank A. The RCC base address is 0x5800 0000,
        GPIO bank A uses bit 0. We set bit 0 of register RCC_AHB2EN.

      - We activate PA0 pull-up. We have to set bits b0..1 of register
        GPIOA_PUPDR to 1.

      - Set the pin mode of PA0 to 00 Input, clear bits b0..1 of GPIOA_MODE.

      - Set the pin mode of PA2 to 01 General purpose output, set bits b4..5 
        of GPIOA_MODE to 01.

      - Then we wait for 10000 usec.

      - We read in the accumulator the USR1 button state, so PA0 state, bit 0
        of register GPIOA_ID.

      - If the read value is 0, button is pressed 
            . Set GPIO PA2 to 1/high. We set bit 0 of GPIOA_BSRR
            . Wait for 1 msec, so 100 counts of 10 usec
            . Set GPIO PA2 to 0/low. We set bit 16 of GPIOA_BSRR
            . Breakpoint stop
            . After beeing restarted with GDB (c), exit from preinit
        else
            . Set GPIO PA2 to 1/high. We set bit 0 of GPIOA_BSRR
            . Wait for 1 msec, so 100 counts of 10 usec
            . Set GPIO PA2 to 0/low. We set bit 16 of GPIOA_BSRR
            . Exit from preinit
       
    We do not mofify GPIOD_OTYP value since after reset it is push-pull.

*/

const UINT cnt10usec = 39     ; /* See WAIT10US in boot_stm32.c              */

#define RCC_AHB2EN  0x5800004C  /* Enable clock for AHB2 peripheral bus dev. */
#define GPIOA_MODE  0x48000000  /* GPIO A Mode register                      */
#define GPIOA_BSRR  0x48000018  /* GPIO A bit set/reset register             */
#define GPIOA_PUPDR 0x4800000C  /* GPIO A pull-up/pull-down register         */
#define GPIOA_ID    0x48000010  /* GPIO A Input data register                */

const UINT preinit_stm32wl[][5] =
  BEGIN
/*  Enable clock for the GPIO device A ......................................*/
/*  0 */   PRE_POKE      , RCC_AHB2EN , 0xFFFFFFFE , 0x00000001 , PEMPTY     ,
/*  1 */   PRE_WAIT10US  , 1          , PEMPTY     , PEMPTY     , PEMPTY     ,

/*  Enable the pull-up resistor for PA0 .....................................*/
/*  2 */   PRE_POKE      , GPIOA_PUPDR, 0xFFFFFFFC , 1 << 0     , PEMPTY     ,

/*  Select GPIO output for PA2 (ASY TX) and GPIO input for PA0 (USR1) .......*/
/*  3 */   PRE_POKE      , GPIOA_MODE , 0xFFFFFFCC , 1 << 4     , PEMPTY     ,

/*  Check if USR1 is pressed, if not (if state is 1) GOTO 12 ................*/
/*  4 */   PRE_WAIT10US  , 100        , PEMPTY     , PEMPTY     , PEMPTY     ,
/*  5 */   PRE_PEEK      , GPIOA_ID   , 0x00000001 , PEMPTY     , PEMPTY     ,
/*  6 */   PRE_BEQ       , 0x00000001 , 0x00000001 , 12         , PEMPTY     ,

/* USR1 button is pressed. Send a 1 msec pulse and stop on BKPT .............*/
/*  7 */   PRE_POKE      , GPIOA_BSRR , 0x00000004 , 1 << 2     , PEMPTY     ,
/*  8 */   PRE_WAIT10US  , 100        , PEMPTY     , PEMPTY     , PEMPTY     ,
/*  9 */   PRE_POKE      , GPIOA_BSRR , 0x00040000 , 1 << 18    , PEMPTY     ,
/* 10 */   PRE_BKPT      , 0x11111111 , PEMPTY     , PEMPTY     , PEMPTY     ,
/* 11 */   PRE_B         , 15         , PEMPTY     , PEMPTY     , PEMPTY     ,

/* USR1 button is not pressed. Send a 0,5 msec pulse an exit ................*/
/* 12 */   PRE_POKE      , GPIOA_BSRR , 0x00000004 , 1 << 2     , PEMPTY     ,
/* 13 */   PRE_WAIT10US  , 50         , PEMPTY     , PEMPTY     , PEMPTY     ,
/* 14 */   PRE_POKE      , GPIOA_BSRR , 0x00040000 , 1 << 18    , PEMPTY     ,

/* 15 */   PRE_ENDPGM    , PEMPTY     , PEMPTY     , PEMPTY     , PEMPTY     ,
  END;


/*  6.3  Timer numbers - sys_tim and flt_tim
    -----------------------------------------
*/

INT       sys_tim                    ; /* tim_arm.c                          */
INT       flt_tim                    ; /* tim_stm32.c                        */


/*  6.4  Timers reference table - ref_tim32
    ---------------------------------------

    The "ref_tim32[]" table is directly referenced by the "tim_stm32.c" module
    This table is used by the "get_ref_tim_stm32" function from "tim_stm32.c"
    that takes in input a "uname" string character and that returns the
    corresponding "ldev" and "chan" values. The STM32WL has 4 differents
    variants timers that are called "Advanced" and "General purpose".

    1    16 bits Advanced
    2    32 bits General purpose
    16   16 bits General purpose
    17   16 bits General purpose
*/

const Ref_tim32 ref_tim32[] =          /* Timers reference table             */
  BEGIN                                /* ---------------------------------- */
/*  uname        use     ldev    chan     Used by                            */
/*  -----------  ----    ----    ----     ---------------------------------- */
   "SYS_TIM"    ,USYS ,   2  ,  255  , /* tim_arm.c                          */
   "FLT_TIM"    ,USYS ,  16  ,  255  , /* tim_stm32, digital filter timer    */
    HNULL       ,USYS ,   0  ,    0  , /* End of table mark                  */
  END                                ; /* ---------------------------------- */


/*  6.5  DMA channels/streams reference table - ref_dma32
    -----------------------------------------------------

    The "ref_dma32[]" table is directly referenced by the "dma_stm32.c" module
    This table is used by the "get_ref_dma_stm32" function from "dma_stm32",
    which takes on input a "uname" string character and returns the
    corresponding "ldev", "chan" and "strm" values. The STM32WLx5x has 2 DMA
    devices respectively numbered 1 and 2. Each DMA comes with 8 streams
    numbered from 0 to 7. Each stream is associated to a specific hardware 
    using the "channel" value. But we cannot associate any device with any
    stream, one has te refer to Table 42 DMA1 request mapping on page 307 of
    ST RM0090 and table 43 DMA2 request mapping. For example, UART5_RX is
    DMA=1,stream=0,chan=4, there is no other possibility.
*/

/* DMA parameters for UART 1 ..................................................

   Here, all the numbers and names (streams and channels) are ST defined. Those
   values are coming from ST RM0434, pages xxx and xxx.

   ASY is USART1  is not using DMA

*/

const Ref_dma32 ref_dma32[] =          /* DMA streams reference table        */
  BEGIN                                /* ---------------------------------- */
/*  uname             ldev    strm         chan             Used by          */
/*  --------------   ------   ----------   ----------       ---------------- */
   "NO_DMA"        , 0xFF   , 0xFF       , 0xFF       ,  /* No DMA used      */
    HNULL          ,   0    ,    0       ,   0        ,  /* End of table mark*/
  END                                ; /* ---------------------------------- */


/*  6.6  Debug table for drv_int.c - tnote_it[]
    -------------------------------------------

       To help identify bugs that may occur after a long running time, a number
of debug tables are provided by the various VMIO modules. These tables can be
very large, and there are cases where we want to reduce them to a minimum
(loader) and others where we want to increase them to a maximum (debugging a
problem on a given platform). We therefore define these tables in the various
board files.

*/

const IWORD max_note_it =   1      ; /* Debug table size.                    */
Note_it     tnote_it      [ 1 ]    ; /* Debug circular buffer.               */


/* 6.7  Debug variable for GDB/IDLE mode - use_gdb
   -----------------------------------------------

        When using GDB/OpenOCD, we have a regular polling of the target by the
debugger or OpenOCD: If we slow down the JTAG clock at 100 or 10 KHz, we will
notice that the bicolor GREEN/RED led of Olimex probe is flashing Green/RED,
indicating activity, even if there is no reason to have any polling. When the
CPU becomes IDLE, t seems it does not answers any more to GDB/OpenOCD requests 
and we get on the GDB window:           *

         DAP transaction stalled (WAIT) - slowing down

and then using GDB becomes very difficult or impossible. What is sure, we never 
get this message if "set_idle(1)" is not called. Therefore, the "board_init"
procedure uses a "use_gdb" global variable which is set by "gdb.cfg" and 
"board_init" does not call "set_idle" if this global variable is TRUE

*/

extern UINT use_gdb               ; /* For "board_init"                     */


/*
    7    Internal constants of the module
    _____________________________________
*/

/*
    8    Internal data types of the module
    ______________________________________
*/

IWORD sdrv_empty(IWORD)              ; /* Termination empty procedure.       */


/*
    9    Internal (private) variables of the module
    _______________________________________________

    9.1  VMIO system drivers configuration data
         9.1.1  SYSTEM driver - sys_sdrv_link
         9.1.2  BOARD_STM32WB55_NUCLEO64 driver - board_sdrv_link
         9.1.3  RAM_HARD driver - ram_hard_sdrv_link
         9.1.4  RAM_SOFT driver - ram_soft_sdrv_link
         9.1.5  RESOLVE driver - rslv_sdrv_link
         9.1.6  VMIT driver - vmit_sdrv_link
         9.1.7  VMIO driver - vmio_sdrv_link
         9.1.8  TIME driver - tim_sdrv_link
         9.1.9  GPIO driver - gpio_sdrv_link
         9.1.10 ASY driver - asy_sdrv_link
         9.1.11 I2C driver - i2c_sdrv_link
         9.1.12 RAD driver - rad_sdrv_link

    9.2  VMIO character drivers configuration data
         9.2.1  GPIO driver
         9.2.2  ASY driver
         9.2.3  LED driver
         9.2.4  RAD driver

    9.3  Block drivers definitions
    9.4  Volume drivers definitions
    9.5  Protocols definitions
         9.5.1  LPWAN protocol

    9.6  Services definitions

    9.7  VMIO driver list and other global data structures
         9.7.1  VMIO driver list - drv_wl55
         9.7.2  Board_link structure - board_wl55
         9.7.3  Table of Drv_req_ctx contexts - drv_req_ctx
*/

/*  9.1  VMIO system drivers configuration data 
    ===========================================


    9.1.1  SYSTEM Driver - sys_sdrv_link
    ------------------------------------

    This driver configures CPU and bus clocks. The configuration data we
    provide here is made of:

    - "sys_sdrv_link" is the "Sdrv_link" structure, which is pointed to
      by "drv_wl55", the VMIO driver list. The "sdrv_desc" field is the
      address of the "sdrv_syswl" structure of "sys_stm32.c" and the "dev_link"
      fields is the address of the "dev_sys" structure below defined.

    - "dev_sys" is the "Dev_link" structure, which is pointed to by 
      "sys_sdr_link". There is no low-level procedures (dev_desc) but we have
      a set of high level parameters, "dev_def" points to "sysdef".

    - "sysdef" is an array of UINT's which are the CPUs and bus clock 
       parameters. Those parameters are used by the "init" procedure of the
       "sdrv_desc" (here sdrv_syswl) structure, in our case the "init" function
       is "init_syswb" (see "sys_stm32.c), which tunes SYSCLK, HCLK, HCLK3,
       PCLK1, PCLK2 and PCLK3 clocks, the internal flash latency, the HSE,
       prescalers for buses AHB1, AHB2, AHB3, AHB4, APB1 and APB2.
*/

static const UINT sysdef[] =           /* Parametres du systeme              */
  BEGIN                                /* ---------------------------------- */
     48000000                       ,  /* SYSCLK            clock frequency  */
     48000000                       ,  /* HCLK    AHB  bus  clock frequency  */
     0                              ,  /* HCLK2   CPU2      clock frequency  */
     48000000                       ,  /* HCLK3   APB3 bus  clock frequency  */
     48000000                       ,  /* PCLK1   APB1 buf  clock frequency  */
     48000000                       ,  /* PCLK2   APB2 bus  clock frequency  */
     1                              ,  /* LSE clock status                   */
  END                                ; /* ---------------------------------- */

static const Dev_link dev_sys =        /* SYS_WL55 device for SYSTEM         */
  BEGIN                                /* ---------------------------------- */
    "SYS_WL55"                      ,  /* dev_name  Device name              */
    HNULL                           ,  /* dev_desc  Low-level proc addresses */
    sysdef                          ,  /* dev_def   High level parameters    */
    HNULL                           ,  /* drv_def   Low level parameters     */
  END                                ; /* ---------------------------------- */

static const Sdrv_link sys_sdrv_link = /* SYSTEM driver                      */
  BEGIN                                /* ---------------------------------- */
    &sdrv_syswl                     ,  /* sdrv_desc  Driver interface        */
    &dev_sys                        ,  /* dev_link   List of devices         */
    1                               ,  /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.2  BOARD_STM32WL55_NUCLEO - board_sdrv_link
    -----------------------------------------------

    This driver executes board-dependant initializations, if any. Here, it
calls the "board_init" procedure, which code is located in the present file,
currently it only calls "set_idle(1)" in order to allow CPU idle mode. The
needed configuration data is :

    - "board_sdrv_link" is the "Sdrv_link" structure, which is pointed to
      by "drv_wl55", the VMIO driver list. The "sdrv_desc" field is the
      address of the "sdrv_board" structure and the "dev_link" field is HNULL.

    - "board_sdrv_desc" is the "Sdrv_desc" structure, which points the 
      "board_init" function though its "init" field. This "board_init" 
      procedure is therefore called by "init_hypsys".
*/

static IWORD board_init                    (IWORD);

static const Sdrv_desc board_sdrv_desc=/* MB1389E driver interface           */
  BEGIN                                /* ---------------------------------- */
    "MB1389E"                       ,  /* drv_name  Driver name              */
    DRV_SYS                         ,  /* typdrv    Always 0 for system sdrv */
                                       /* Interface procedures ------------- */
    board_init                      ,  /* init      Initialization procedure */
    sdrv_empty                      ,  /* end       Termination procedure    */
  END                                ; /* ---------------------------------- */

static const Sdrv_link board_sdrv_link=/* BOARD_STM32WL55 driver             */
  BEGIN                                /* ---------------------------------- */
    &board_sdrv_desc                ,  /* sdrv_desc  Driver interface        */
    HNULL                           ,  /* dev_link   List of devices         */
    0                               ,  /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.3  RAM_HARD driver - ram_hard_sdrv_link
    -------------------------------------------

    This driver discovers additionnal, external DRAM devices, if any. It resets
those if requested. With the STM32WL55-Nucleo board, there is no external DRAM
and no RAM to reset of start: This driver does nothing. However, in order to do
nothing, the RAM_HARD driver needs the following configuration data:

    - "ramh_sdrv_link" is the "Sdrv_link" structure, which is pointed to by
      "drv_wl55", the VMIO driver list. The "sdrv_desc" field value is the
      address of the "ram_hard_sdrv_desc" structure from "drv_ram.c". The
      "dev_link" field points to a list of "Seg_def" structures, here "seg_def"
      and "nb_dev" is the size of the "seg_def" list, here 1 because we only
      have one RAM segment.

    - "ramh_dev_link" is a list of one "Dev_link" structure, which is pointed 
      to by "ramh_sdrv_link", the RAM_HARD driver structure. The "drv_def" 
      field points to a list of "Seg_def" structures, here "seg_def". The count
      of structures is given by the "nb_dev" field of "ramh_sdrv_link" (1).

    - "seg_def" is a list of one "Seg_def" structure. This structure holds the
      "adr" address of the RAM segment, its "size" size in bytes, and a "flags"
      value which is currently unused.

    The values we use to initialize the "adr" and "size" fields of "seg_def"
may lool surprising, especially the "&lg_buf_part" value which is a size in
bytes. This requires a little explanation. In fact the two "ad_buf_part0" and
"lg_buf_part0" names are symbols generated by the linker when it reads the
"boards/stm32wb/exe/linker.cfg" where one will find:

 ad_buf_part0 = ORIGIN(RAM_MEM_PART)           ; /+ Part 0 low-address.      +/
 lg_buf_part0 = LENGTH(RAM_MEM_PART)           ; /+ Part 0 size in bytes     +/

    For the linker, a symbol is simply a character string which replaces a 32
bits integer value. There is no symbol data typing: both "ad_buf_part0" and 
"lg_buf_part0" are both 32 bits integer, no matter they are an address or a 
length. But for a C program, a symbol is implicitely a variable, so the sole 
name is "the value at address symbol" and "&symbol" is the address of the 
value, so the linker symbol value. So for example if LENGTH(RAM_MEM_PART) is 
512, then in C "lg_buf_part" is the value at address 512 and "&lg_buf_part" is 
equal to 512.

    The "init_hypsys" procedure will call the "ram_hard_init" procedure through
the "init" field of "ram_hard_sdrv_desc". This procedure will call 
"alloc_init_ram" and will copy "seg_def" to the "init RAM" area. Then the
RAM_SOFT driver will read the copy of "seg_def".

*/

extern UINT         ad_buf_part0     ; /* Linker defined symbol              */
extern UINT         lg_buf_part0     ; /* Linker defined symbol              */

static const Seg_def seg_def[1] =      /* List of RAM segment descriptors    */
  BEGIN                                /* ---------------------------------- */
    (UCHAR*)&  ad_buf_part0          , /* adr  Start of segment address      */
    (UINT  )&  lg_buf_part0          , /* size Segment size in bytes         */
    0                                , /* List of option flags               */
  END                                ; /* ---------------------------------- */

static const Dev_link ramh_dev_link[1]=/* List of device descriptors         */
  BEGIN                                /* ---------------------------------- */
    "PART0"                          , /* dev_name System device name        */
    HNULL                            , /* dec_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    seg_def                          , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link ramh_sdrv_link= /* RAM_HARD driver                    */
  BEGIN                                /* ---------------------------------- */
    &ram_hard_sdrv_desc             ,  /* sdrv_desc  Driver interface        */
    ramh_dev_link                   ,  /* dev_link   List of devices         */
    1                               ,  /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.4  RAM_SOFT driver - ram_soft_sdrv_link
    -------------------------------------------

    This driver is responsible for execution the first(s) call(s) to the
"init_mem" procedure, which creates one RAM partition at each call. With the
STM32WB55 - Nucleo board, we create partition number 0. To do that, the
RAM_SOFT (the "ram_soft_init" procedure) needs the following configuration
data:

    - "rams_sdrv_link" is the "Sdrv_link" structure, which is pointed to by
      "drv_wl55", the VMIO driver list. The "sdrv_desc" field value is the
      address of the "ram_soft_sdrv_desc" structure from "drv_ram.c". The
      "dev_link" field points to a "rams_dev_link" list one one "Devlink"
      structure. The "nb_dev" field holds the "dev_link"' list size, here 1.

    - "rams_dev_link" is a list of one "Dev_link" structure, we have one and
      one only "Dev_link" structure per RAM partition to be created. The 
      "dev_desc" and "dev_def" fields are not used and so HNULL. The "drv_def"
      field is pointing to a list of one "Part_def" structure.

    - "part_def[1]" is a list of one "Part_def" structure. For each memory
      partition, we may have a list of possible partition parameters, starting
      with biggest sizes and ending with smaller sizes. In our case, we do not
      have external DRAM devices with variable/unknown capacities, we have only
      one and one only RAM segment (see 9.1.3  RAM_HARD driver) and so we only
      have one "Part_def" structure, with only one RAM segment, with a 6KB 
      minimum requested size. The "part" partition number is 0, the selected
      slicing code is 0 and the "big blocks" code size is 7 (4576 bytes).

*/

static const Part_def part_def[2] =    /* List of RAM partition decriptors   */
  BEGIN                                /* ---------------------------------- */
    0                               ,  /* part        Mem partition # 0..3   */
    0                               ,  /* dec         Slicing code 0..3      */
    7                               ,  /* bigszcd     Big blocks size 1..15  */
    0      , 0xFF,0xFF,0xF          ,  /* segnum[4]   Indexes in "Seg_def[]" */
    1024*6 ,   -1,  -1, -1          ,  /* segminsz[4] Segments minimum sizes */
                                       /* ---------------------------------- */
    0xFF                               /* part        End of list marker     */
  END                                ; /* ---------------------------------- */

static const Dev_link rams_dev_link[1]=/* List of device descriptors         */
  BEGIN                                /* ---------------------------------- */
    "PART0"                          , /* dev_name System device name        */
    HNULL                            , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    part_def                         , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link rams_sdrv_link= /* RAM_SOFT driver                    */
  BEGIN                                /* ---------------------------------- */
    &ram_soft_sdrv_desc             ,  /* sdrv_desc  Driver interface        */
    rams_dev_link                   ,  /* dev_link   List of devices         */
    1                               ,  /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.5  RESOLVE driver - rslv_sdrv_link
    --------------------------------------

    This dystem driver does not drivers any hardware, be is resolving automaton
names to automaton numbers, which is needed to initialize cheracter drivers:
When a VMK entity needs to send events to VMIO entities, which may be located
in another computer, the distant VMIO number is not known to VMK. Therefore,
any VMIO includes a RESOLVE automaton which receives the REQ_INIT events, the
attached data buffer holds the name. The AUT_RESOLVE scans the VMIO list of
drivers, finds the corresponding driver and so the associated VMIO automaton
and send re-sends the REQ_INIT to that autoamton. The driver/protocol/servide
VMIO automaton will send the RESP_INIT directly to the original REQ_INIT sender
and the "saut" field of RESP_INIT holds the VMIO automaton number. This
"software only" driver needs no configuration data but a "Sdrv_link" structure
which points to "rslv_sdrv_desc", defined in "drv_rslv.c"

*/

static const Sdrv_link rslv_sdrv_link= /* RESOLVE driver                     */
  BEGIN                                /* ---------------------------------- */
    &rslv_sdrv_desc                 ,  /* sdrv_desc  Driver interface        */
    HNULL                           ,  /* dev_link   List of devices         */
    0                               ,  /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.6  VMIT driver - vmit_sdrv_link
    -----------------------------------

    This system driver is responsible to initailize the NVIC interrupt 
controller. One has to be aware that next drivers, such as the I2C, the ASY,..
may then in turn raise interrupts, but not before the NVIC has been initialized.
This INT system driver needs the following configuration data:

    - "vmit_sdrv_link" is the "Sdrv_link" structure, which is pointed to by
      "drv_wl55", the VMIO driver list. The "sdrv_desc" field value is the
      address of the "int_sdrv_desc" structure from "vmit.c". The "dev_link"
      field points to a list of one "Dev_link" structures, here "int_dev_link".
      The "nb_dev" field value is 1 because we only have one "Dev_link"
      structure, one IRQ controller (one NVIC).

    - "int_dev_link" is the list of one "Dev_link" structure, corresponding to
      the INT_NVIC device. The "dev_desc" field is pointing to "int_desc_nvic"
      which is defined in "vmit.c". The "drv_def" field points to "int_def",
      this driver parameter structure is defined below.

    - "vmit_def" structure of type "Vmit_def" is pointed by the "drv_def" field
      of "int_dev_link[0]" and holds the three "itidx/ithead.ithand" table
      addresses needed by "vmit.c".
      
    - "ithand" is the table of IRQ handler addresses. It is an array of 
      "Ithand_t" structures, each entry holds one handler C procedure address,
      the two parameters given to the C procedure and the link to the next
      handler associated with the same IRQ, if any. An end of link is marked by
      the 0xFFFFFFFF special value.

    - "ithead" is the table wich holds the first handler, the head of link, 
      for the chain of handlers for each IRQ links between handlers for one same
      IRQ. Within this table, heads of IRQ are grouped by same priorities,
      beginning by the lowest 0 priority.

    - "itidx" is the table of IRQ priorities, it is a list of indexes in the
      previous "ithead" table, it indicates the limits of the priority groups.

    With ARM CPU's, there is an "exception table" located at address 0x00000000.
Every IRQ is considered as an exception, meaning we have one vector (4 bytes)
per IRQ, when a specific IRQ is taken in account then the CPU fetches the 
vector (a code address) and then continues execution at this address. In our 
case, all the entries are holding one same address, which is the address of
assembly written "irq" procedure (see int_cortex.s). So whatever IRQ is raised,
"irq" is called. The "irq" procedure retreives the exception number from the
cortex IPSR register, substracts 15 in order to get 0 for Systick and "n+1"
for IRQ "n". Then, "irq" extracts for the "itdef" table, which has been built
onece "init_vector", a 4 byte integer (IRQ priority + "ithead" index value),
and calls "int_proc" (vmit.c) with the 4 byte integer has input parameter.
This parameter, and so all the UINT's of "itdef", are equal to 
"prio + index in ithead << 8). So, one can check that the "itdef" contents is
corresponding to "ithead" and then that the indexes in "ithand" from "ithead"
are correct.

    Devices interrupt numbers will be found on pages 500 to 505 of RM0453.

*/

/* Handler numbers ......................................................... */

#define H_VMIO                0        /* Handler number                     */
#define H_VMK_TIME            1        /* Handler number                     */

#define H_ASY                 2        /* Handler number                     */

#define H_GPIO00              3        /* Handler number                     */
#define H_GPIO01              4        /* Handler number                     */
#define H_GPIO02              5        /* Handler number                     */
#define H_GPIO03              6        /* Handler number                     */
#define H_GPIO04              7        /* Handler number                     */

#define H_IRQ22               8        /* Dispatcher number                  */
#define H_GPIO05              9        /* Handler number                     */
#define H_GPIO06             10        /* Handler number                     */
#define H_GPIO07             11        /* Handler number                     */
#define H_GPIO08             12        /* Handler number                     */
#define H_GPIO09             13        /* Handler number                     */

#define H_IRQ41              14        /* Dispatcher number                  */
#define H_GPIO10             15        /* Handler number                     */
#define H_GPIO11             16        /* Handler number                     */
#define H_GPIO12             17        /* Handler number                     */
#define H_GPIO13             18        /* Handler number                     */
#define H_GPIO14             19        /* Handler number                     */
#define H_GPIO15             20        /* Handler number                     */

#define H_RADIO              21        /* Handler number                     */

#define H_I2C_EV             22        /* Handler number                     */
#define H_I2C_ERR            23        /* Handler number                     */

#define H_TIME               24        /* Handler number                     */
#define H_TRIGTO             25        /* Handler number                     */

#define INT_NBIRQ            26        /* Needed by "drv_int.h"              */
#define NB_SOFTIRQ            2        /* Size of "ref_nvic[]"               */
#include <vmit.h>                      /* itdec[] itcnt[]                    */

/* Data needed by "int_arm.c" ................................................

   The software raised interrupts are always device IRQs. We have to select
   unused devices, such as UART2
*/

#define VMIO_IRQ     WWDG_IRQ           /* Equal to entry 0 -  0             */
#define VMK_TIME_IRQ UART2_IRQ          /* Equal to entry 1 - 37             */


const Ref_nvic ref_nvic[NB_SOFTIRQ] =  /* Names/numbers of software IRQ's    */
  BEGIN                                /* ---------------------------------- */
    "VMIO"  , VMIO_IRQ           , 1 , /* Window Watchdog                  0 */
    "LPTICK", VMK_TIME_IRQ       , 1 , /* USART2 global interrupt         37 */
  END                                ; /* ---------------------------------- */

/* Handler table ...........................................................

   We have some handlers which requires a two stage handling because the
   IRQ is raised at a high level but corresponding handler must be called
   at a lower level. In order to do that, the first stage at high priority
   calls "raise_vmit_signal" and second stage is the software raised IRQ handler

   In our case, two stage IRQ processings are:

   - H_TIME (Systick) has the highest priority 7. It triggers 
     "handlerTIM_armv7m" and then "vmio_trigto". The "vmio_trigto" function
     calls "raise_vmit_signal(1)". The handler is "vmk_tim_stm32" which calls
     "time_itmsec" and then "clear_vmit_signal(1)". So, "time_itmsec" is called
     at the same level as "vmio" (level 1).

   - EXTI 10..15 are sharing one same IRQ line wich is IRQ41. Handlers for
     EXTI10 to EXTI13 are "handlerGPIO_stm32" and must be executed at priority
     3 whilst EXTI14 and EXTI15 handlers are "handlerGPIO_hrt32" and must
     be executed at priority 7. So, we call "raise_vmit_signal" at level 7 for
     software raise IRQ's 2/3/4/5 and at priority 3 we call "handlerGPIO_stm32"
     for EXTI10/11/12/13.

 */

static const Ithand_t ithand[] =  /* Table of IRQ handlers                   */
  BEGIN                                                 
/*  Link b24..31    Handler address      Par1   Par 2       Index            */
/*  --------------  ------------------   -----  ----------  ---------------- */

/*  Priority 1 ..............................................................*/

    SINGLE        , vmio              ,  0    , 0        , /*  0 H_VMIO      */
    SINGLE        , vmk_tim_stm32     ,  1    , 0        , /*  1 H_VMK_TIME  */

/*  Priority 2 ..............................................................*/

    SINGLE        , handlerASY_stm32  ,  0    , 0        , /*  2 H_ASY       */

/*  Priority 3 ..............................................................*/

    SINGLE        , handlerGPIO_stm32 ,  0    , 0        , /*  3 H_GPIO00    */
    SINGLE        , handlerGPIO_stm32 ,  1    , 0        , /*  4 H_GPIO01    */
    SINGLE        , handlerGPIO_stm32 ,  2    , 0        , /*  5 H_GPIO02    */
    SINGLE        , handlerGPIO_stm32 ,  3    , 0        , /*  6 H_GPIO03    */
    SINGLE        , handlerGPIO_stm32 ,  4    , 0        , /*  7 H_GPIO04    */
                                                           /*                */
    DISPATCH(5)   , dispatchEXTI_IRQ  ,  5    , 9        , /*  8 H_IRQ22     */
    ENDLINK       , handlerGPIO_stm32 ,  5    , 0        , /*  9 H_GPIO05    */
    ENDLINK       , handlerGPIO_stm32 ,  6    , 1        , /* 10 H_GPIO06    */
    ENDLINK       , handlerGPIO_stm32 ,  7    , 0        , /* 11 H_GPIO07    */
    ENDLINK       , handlerGPIO_stm32 ,  8    , 0        , /* 12 H_GPIO08    */
    ENDLINK       , handlerGPIO_stm32 ,  9    , 0        , /* 13 H_GPIO09    */
                                                           /*                */
    DISPATCH(6)   , dispatchEXTI_IRQ  , 10    , 15       , /* 14 H_IRQ_41    */
    ENDLINK       , handlerGPIO_stm32 , 10    , 0        , /* 15 H_GPIO10    */
    ENDLINK       , handlerGPIO_stm32 , 11    , 0        , /* 16 H_GPIO11    */
    ENDLINK       , handlerGPIO_stm32 , 12    , 0        , /* 17 H_GPIO12    */
    ENDLINK       , handlerGPIO_stm32 , 13    , 0        , /* 18 H_GPIO13    */
    ENDLINK       , handlerGPIO_stm32 , 14    , 0        , /* 19 H_GPIO14    */
    ENDLINK       , handlerGPIO_stm32 , 15    , 0        , /* 20 H_GPIO15    */

/*  Priority 4 ..............................................................*/

/*  Priority 5 ..............................................................*/

    SINGLE        , handlerRAD_stm32  ,  0    , 0        , /* 21 H_RADIO     */

/*  Priority 6 ..............................................................*/

    SINGLE        , handlerevI2C_stm32,  0    , 0        , /* 22 H_I2C_EV    */
    SINGLE        , handlererrI2C_stm32, 0    , 0        , /* 23 H_I2C_ERR   */

/*  Priority 7 ..............................................................*/

    NEXT(H_TRIGTO), handlerTIM_armv7m ,  0    , 0        , /* 24 H_TIME      */
    ENDLINK       , vmio_trigto       ,  0    , 0        , /* 25 H_TRIGTO    */
    0                                                      /* END OF TABLE   */
  END                                                    ; /* -------------- */

/* Head of chains of handler for each IRQ .................................. */

static const Ithead_t ithead[] =       /* First handler for each IRQ         */
  BEGIN                                /* ---------------------------------- */

/* Priority 0 is used for currently unused devices (no activated IRQ) ...... */
#define L_ITPRI0  0

/* Priority 1 .............................................................. */
#define L_ITPRI1  L_ITPRI0 + 2

   VMIO_IRQ        , 1 , H_VMIO      , /* IRQ 0 , interruption logicielle    */
   VMK_TIME_IRQ    , 1 , H_VMK_TIME  , /* VMK time software raised IRQ       */

/* Priority 2 .............................................................. */
#define L_ITPRI2  L_ITPRI1 + 1

   UART1_IRQ       , 1 , H_ASY       , /* IRQ36 , USART1                     */

/* Priority 3 .............................................................. */
#define L_ITPRI3  L_ITPRI2 + 7

   EXTI0_IRQ       , 1 , H_GPIO00    , /* IRQ 6 , GPIO 0                     */
   EXTI1_IRQ       , 1 , H_GPIO01    , /* IRQ 7 , GPIO 1                     */
   EXTI2_IRQ       , 1 , H_GPIO02    , /* IRQ 8 , GPIO 2                     */
   EXTI3_IRQ       , 1 , H_GPIO03    , /* IRQ 9 , GPIO 3                     */
   EXTI4_IRQ       , 1 , H_GPIO04    , /* IRQ10 , GPIO 4                     */
   EXTI9_5_IRQ     , 1 , H_IRQ22     , /* IRQ22 , GPIO 5 to GPIO 9           */
   EXTI15_10_IRQ   , 1 , H_IRQ41     , /* IRQ41 , GPIO 10 to GPIO 15         */

/* Priority 4 .............................................................. */
#define L_ITPRI4  L_ITPRI3 + 0

/* Priority 5 .............................................................. */
#define L_ITPRI5  L_ITPRI4 + 1

   RADIO_IRQ       , 1 , H_RADIO     , /* IRQ50 , Radio IRQs                 */

/* Priority 6 .............................................................. */
#define L_ITPRI6  L_ITPRI5 + 2

   I2C2_EV_IRQ     , 1 , H_I2C_EV    , /* IRQ32 , I2C2 event                 */
   I2C2_ER_IRQ     , 1 , H_I2C_ERR   , /* IRQ33 , I2C2 error                 */

/* Priority 7 .............................................................. */
#define L_ITPRI7  L_ITPRI6 + 1

   -1              , 1 , H_TIME      , /* SysTick, decompteur systeme.       */

  END                                ; /* ---------------------------------- */


/* Index in "ithead", priority per priority .................................*/

static const IWORD itidx[] =           /* Priority index in "ithead[]"       */
  BEGIN                                /* ---------------------------------- */
    0        , L_ITPRI0 , L_ITPRI1 ,   /* Priority limits (indexes) in the   */
    L_ITPRI2 , L_ITPRI3 , L_ITPRI4 ,   /* "ithead[]" table                   */
    L_ITPRI5 , L_ITPRI6 , L_ITPRI7 ,   /*                                    */
  END                                ; /* ---------------------------------- */

/* Standard configuration data structures for the VMIT driver .............. */

static const Vmit_def vmit_def =       /* "drv_def" structure for "vmit.c"   */
  BEGIN                                /* ---------------------------------- */
    itidx                            , /* idx  Index for priority groups     */
    ithead                           , /* head Head of links for each IRQ    */
    ithand                           , /* hand IRQ handler addresses         */
  END                                ; /* ---------------------------------- */

static const Dev_link vmit_dev_link[1]=/* List of device descriptors         */
  BEGIN                                /* ---------------------------------- */
    "INT_NVIC"                       , /* dev_name System device name        */
    &vmit_desc_nvic                  , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    &vmit_def                        , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */


static const Sdrv_link vmit_sdrv_link =/* VMIT driver                        */
  BEGIN                                /* ---------------------------------- */
    &vmit_sdrv_desc                  , /* sdrv_desc  Driver interface        */
    vmit_dev_link                    , /* dev_link   List of devices         */
    1                                , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.7  VMIO driver - vmio_sdrv_link
    -----------------------------------

    The VMIO handles the software raised low priority IRQ. The reason we have a
VMIO system driver is to allow VMIO initialization using "init_hypsys". The
VMIO initialization procedure is "init_vmio" and the address of this procedure
is held in the "init" field of the "vmio_sdrv_link" structure, which is defined
in "vmio.c". Therefore, "init_hypsys" will call "init_vmio". In the same way, 
the VMIO has a "end_vmio" termination procedure, its address is kept in the 
"end" field of "vmio_sdrv_desc" and so "end_hypys" will do an indirect call to
"end_vmio".

*/

static const Sdrv_link vmio_sdrv_link= /* VMIO system driver                 */
  BEGIN                                /* ---------------------------------- */
    &vmio_sdrv_desc                  , /* sdrv_desc  Driver interface        */
    HNULL                            , /* dev_link   List of devices         */
    0                                , /* nb_dev     Count of devices        */
 END                                 ; /* ---------------------------------- */


/*  9.1.8  TIME driver - tim_sdrv_link
    ----------------------------------

    The TIME driver maintains the one millisecond tick and increments all the
time related global variable. It also allows for usec measures. In our case,
we use the "hal/armcom/tim_arm.c" module as TIME driver. This driver needs the
following configuration data structures:

    - "tim_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      VMIO table of drivers, "drv_wl55" in our case. The "sdrv_desc" field
      points to "sdrv_tim_stm32" which is found in "hal/armcom/tim_arm.c", the
      "init" field of this structure is pointing to "init_timarmv7m", which is
      therefore the procedure called by "init_hypsys". The "dev_link" field is
      pointing to a one item list of "Dev_link" structure, here "tim_dev_link"
      and the "nb_dev" count of "Dev_link" structures is 1.

    - "tim_dev_link(1)" is the list of one "Dev_link" structure, all its field
      are empty because "init_timarmv7m" does not needs any configuration date.

*/

static const Dev_link tim_dev_link[1]= /* List of device descriptors         */
  BEGIN                                /* ---------------------------------- */
    "TIM_WL55"                       , /* dev_name System device name        */
    HNULL                            , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    HNULL                            , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link tim_sdrv_link = /* Driver TIME                        */
  BEGIN                                /* ---------------------------------- */
    &sdrv_tim_stm32                  , /* sdrv_desc  Driver interface        */
    tim_dev_link                     , /* dev_link   List of devices         */
    1                                , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.9  GPIO driver - gpio_sdrv_link
    -----------------------------------

    The GPIO system driver handles all the MCU GPIO's, which are grouped by
"banks" of 8 GPIOs each, each bank beeing named by ST with one letter ranging 
from A to H. But in our case, our GPIO drivers provides a model with which there
is only ONE GPIO device, with ALL the GPIO's beeing sub-channels on this single
device. The GPIO system driver needs the following configuration data:

    - "gpio_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      VMIO table of drivers, here "drv_wl55". The "sdrv_desc" field points to 
      the "gpio_sdrv_desc" structure which is found in "drv_gpio.c", the "init"
      field of this structure is pointing to "gpio_init", which is therefore the
      procedure called by "init_hypsys". The "dev_link" field is pointing to a 
      one item list of "Dev_link" structure, "gpio_dev_link" and the "nb_dev" 
      count of "Dev_link" structures is 1.

    - "gpio_dev_link(1)" is the list of one "Dev_link" structure, the "dev_desc"
      address of low-level procedure is pointing to "gpio_desc_stm32", which is
      defined in "gpio_stm32.c". The "drv_def" field is pointing to 
      "gpio_pindef", a list of "Gpio_pindef" structures defined below.

    - "gpio_pindef" is an array, a list of "Gpio_pindef" structures.
      are empty because "init_timarmv7m" does not needs any configuration date.
      It has to be noted this array has far less entries than the number of
      MCU's GPIO. This is because by default, all GPIOs are initialized as
      inputs with no pull-up. So, this table only contains the GPIO's for which
      this default configuration is not the one we need.
*/

/* Table of initial configurations that are the "drv_gpio" default one ......

Group     paint       Pin#  ST name  Signal      Alternate func  ST device
-------  --------     ----  -------  ----------  --------------  ---------
JTAG      JTAG1       B8    PA13     JTMS-SWDIO       AFIO0      DBG
          JTAG1       A4    PA14     JTCK-SWCLK       AFIO0
          JTAG1       57    PA15     JTDI             AFIO0
          JTAG1       C1    PB3      JTDO             AFIO0
          JTAG1       64    PB4      NJTRST           AFIO0

ASY       ASY_RX      E1    PB6      USART1_TX        AFIO7      CN9_RX
          ASY_TX      C3    PB7      USART1_RX        AFIO7      CN9_TX

I2C2      I2C2_SCL    A9    PA12     I2C2_SCL         AFIO4      CN5_SCL
          I2C2_SDA    B9    PA11     I2C2_SDA         AFIO4      CN5_SDA

RFSWCTL   RFSWCTL1    F3    PC4      SW_CTL1          GPIO       RF Switch Ctl1
          RFSWCTL2    E4    PC5      SW_CTL2          GPIO       RF Switch Ctl2
          RFSWCTL3    D5    PC3      SW_CTL3          GPIO       RF Switch Ctl3

LED       LED1        B4    PB15     GPIO out         GPIO       LED Blue
          LED2        C4    PB9      GPIO out         GPIO       LED Green
          LED3        D5    PC3      GPIO out         GPIO       LED Red

BUTTON    USR1        D6    PA0      GPIO in          GPIO       Button B1


Notes: - When an alternate function is used, so if "func" is not an empty 
         string, field "outcfg" is not used, we put a 0.

       - The ST RM0453 is unclear about an important detail regarding the
         alternate function configuration (see page 277 of RM0090). More in
         details:
         . The pull-up/pull-down setting are always applicable, whatever the
           pin is GPIO, alternate or analog

         . When the alternate function is an output, such as an UART TX or
           I2C SDA, then Push-pull/Open drain and speed are applicable

         . When the alternate is an input, such as an UART RX, it SEEMS that
           the hardware disables the Output control block (Figure 30 on page
           277). As a consequence, we can put any value for PP/OD, outcfg and
           speed for alternate pins which are in fact used as inputs.
*/

const Gpio_pindef gpio_pindef[] =      /* Specific init.                     */
  BEGIN                                /* ---------------------------------- */
/*sname   uname   hidden   func     PU/PD  PP/OD  outcfg  speed              */

/* JTAG signals .............................................................*/

"PA13", "JTMS"    ,USYS, "AFIO0", GP_PU,GP_PP,GP_LOW, DEF, /* JTAG_JTMS      */
"PA14", "JTCK"    ,USYS, "AFIO0", GP_PD,GP_PP,GP_LOW, DEF, /* JTAG_JTCK      */
"PA15", "JTDI"    ,USYS, "AFIO0", GP_PU,GP_PP,GP_LOW, DEF, /* JTAG_JTDI      */
"PB3" , "JTDO"    ,USYS, "AFIO0", GP_NP,GP_PP,GP_LOW, DEF, /* JTAG_JTDO      */
"PB4" , "NJTRST"  ,USYS, "AFIO0", GP_PU,GP_PP,GP_LOW, DEF, /* NJTRST         */

/* ASY is USART1 ............................................................*/

"PB6" , "ASY_TX" ,USYS, "AFIO7", GP_PU,GP_PP,GP_LOW, 0   , /* UART1_TX       */
"PB7" , "ASY_RX" ,USYS, "AFIO7", GP_PU,GP_HZ,GP_IN , 0   , /* UART1_RX       */

/* I2C is I2C2 ..............................................................*/

"PA11" ,"I2C2_SDA" ,USYS, "AFIO4", GP_NP,GP_OD,GP_LOW, 0 , /* I2C2_SDA       */
"PA12" ,"I2C2_SCL" ,USYS, "AFIO4", GP_NP,GP_PP,GP_LOW, 0 , /* I2C28SCL       */

/* RF Switch control pins are PC4/PC5/PC3 ...................................*/

"PC4" , "RFSWCTL1",USYS, "GPIO" , GP_NP,GP_PP,GP_LOW, 3  , /* SW_CTL1        */
"PC5" , "RFSWCTL2",USYS, "GPIO" , GP_NP,GP_PP,GP_LOW, 3  , /* SW_CTL2        */
"PC3" , "RFSWCTL3",USYS, "GPIO" , GP_NP,GP_PP,GP_LOW, 3  , /* SW_CTL3        */

/* LEDs are PB15/PB9/PB11 ...................................................*/

"PB15", "LEDB"    ,USYS, "GPIO" , GP_NP,GP_PP,GP_LOW, 0  , /* LED1_BLUE      */
"PB9" , "LEDG"    ,USYS, "GPIO" , GP_NP,GP_PP,GP_LOW, 0  , /* LED2_GREEN     */
"PB11", "LEDR"    ,USYS, "GPIO" , GP_NP,GP_PP,GP_LOW, 0  , /* LED3_RED       */

/* BUTTON is PA0 ............................................................*/

"PA0" , "USR1"    ,UUSR, "GPIO" , GP_NP,GP_PP,GP_HZ , 0  , /* BUTTON B1      */

 { 0 }                               , /* END OF TABLE                       */
  END                                ; /* ---------------------------------- */

/* GPIO contexts sizes ......................................................*/

#define GPIO_NBIOD             1       /* Must be LE than GPIO_NBCTL         */
#define GPIO_NBCTL             1       /* Only one device.                   */
#define GPIO_NBPIN        (8*16)       /* Number of pins.                    */
#define GPIO_NBVL     (1+1+8*16)       /* Total number of logical channels.  */

#define GPIO_PKG_NUM           2       /* UFBGA73 is 2 (const_stm32_inc.c)   */
#include <drv_gpio.h>                  /* led_nbiod, led_nbvl, led_etat      */

static const Dev_link gpio_dev_link[1]=/* GPIO device                        */
  BEGIN                                /* ---------------------------------- */
    "GPIO"                           , /* dev_name System device name        */
    &gpio_desc_stm32                 , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    &gpio_pindef                     , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link gpio_sdrv_link= /* GPIO driver                        */
  BEGIN                                /* ---------------------------------- */
    &gpio_sdrv_desc                  , /* sdrv_desc  Driver interface        */
    gpio_dev_link                    , /* dev_link   List of devices         */
    GPIO_NBCTL                       , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.10 ASY driver - asy_sdrv_link
    ---------------------------------

    The ASY driver handles UARTs (Universal Asynchronous Receiver Transmitter).
It has no be noted that ST MCU's are coming with UART's but also USART's which
are Unisversal Asynchronous and Synchronous Receiver Transmitter, which means
those are UART able. The HyperPanel OS ASY driver only manages UART's and not
the Synchronous mode, USART ST device may only be used in asynchronous mode. 
The HyperPanel ASY system driver needs the following configuration structures:

    - "asy_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      "drv_wl55" VMIO table of drivers. The "sdrv_desc" field points to 
      "asy_sdrv_desc" which is found in "drv_asy.c", the "init" field of this 
      structure is pointing to "asy_init", which is so the procedure called by 
      "init_hypsys". Here, we use 1 ASY controllers, the "dev_link" field is 
      pointing to a one item list of "Dev_link" structures, here "
      asy_dev_link[1]" and the "nb_dev" count of "Dev_link" structures is 1.

    - "asy_dev_link[1]" is the list of three "Dev_link" structure. The   
      "dev_desc" field is pointing to the "asy_desc_stm32" structure from
      "asy_stm32.c", which holds the low-procedure addresses. The "dev_def"
      field is pointing to "asy_devdef[0]". The "drv_def" field is pointing to
      "asy_drvdef[0]".

    - Each ASY controller needs one structure of low-level (hardware dependant)
      in our case one "Asy_def_stm32" structure (see asy_stm32.h). We have
      ony one entry and we use ST devices 1 (USART1) for the trace on the
      console (asy_write()).

    - Each ASY controller needs one structure of high-level driver parameters,
      a "Asy_drvdef" structure for "drv_asy.c". We store it in "asy_drvdef[1]".
      It's an array but we use just one entry.

*/

#define ASY_NBCTX             1        /* Number of used UARTs               */
#include <drv_asy.h>                   /* asy_nbiod asy_nbvl asy_state       */
                                       /*                                    */
#define ASY_STM32_NBCTX       1        /* Number of used STM32 UASRTs        */
#include <asy_stm32.h>                 /* asy_ctx_stm32[]                    */

static const Asy_def_stm32             /*                                    */
               asy_devdef[ASY_NBCTX] = /* Low level device definition        */
  BEGIN                                /* ---------------------------------- */
    1                                , /* USART1                             */
    HNULL                            , /* Timer number                       */
    "NO_DMA"                         , /* DMA TX channel name       FF FF FF */
    "NO_DMA"                         , /* DMA RX channel name       FF FF FF */
                                       /* Description des GPIO des signaux   */
                                       /* port, pin, logique; inutilise = -1 */
    HNULL          , 0               , /* CTS                                */
    HNULL          , 0               , /* DSR                                */
    HNULL          , 0               , /* RI                                 */
    HNULL          , 0               , /* DCD                                */
    HNULL          , 0               , /* DTR                                */
    HNULL          , 0               , /* RTS                                */
    "ASY_RX"       , 0               , /* RXB  USART1 RX (PA3)               */
    "ASY_TX"       , 0               , /* TXB  USART1 TX (PA2)               */
  END                                ; /* ---------------------------------- */

static const
    Asy_drvdef asy_drvdef[ASY_NBCTX] = /* Device definition                  */
  BEGIN                                /* ---------------------------------- */
                                       /* ST LPUART1/USART1                  */
    0                                , /* Numero de contexte de bas niveau   */
    BR_ALL & ~((1 << BR_50) |          /* Vitesses supportees : vitesses >   */
      (1 << BR_75)  | (1 << BR_110) |  /* 4000000 / 16 / 2^12 / 0,2 (+/-10%  */
      (1 << BR_134) | (1 << BR_150) |  /* d'erreur max) # 305 bauds          */
      (1 << BR_200))                 , /*                                    */
    ST_2 | ST_1                      , /* Only 1 or 2 stop bits              */
    0                                , /* Mode 5 bits non supporte           */
    0                                , /* Mode 6 bits non supporte           */
    PAR_ODD | PAR_EVEN               , /* Mode 7 bits avec parite uniquement */
    PAR_ALL                          , /* Mode 8 bits complet                */
    PAR_NONE                         , /* Mode 9 bits sans parite uniquement */
    0x00                             , /* Rien n'est supporte                */
  END                                ; /* ---------------------------------- */

static const Dev_link asy_dev_link[] = /* ASY_USART1                         */
  BEGIN                                /* ---------------------------------- */
    "ASY USART1"                     , /* dev_name System device name        */
    &asy_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    &asy_devdef[0]                   , /* dev_def  Low-level parameters addr */
    &asy_drvdef[0]                   , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link asy_sdrv_link=  /* ASY system driver                  */
  BEGIN                                /* ---------------------------------- */
    &asy_sdrv_desc                   , /* sdrv_desc  Driver interface        */
    asy_dev_link                     , /* dev_link   List of devices         */
    ASY_NBCTX                        , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.11 I2C driver - i2c_sdrv_link
    ---------------------------------

    The I2C driver handles all the I2C bus controllers of the MCU, it allows
to send and receive complex I2C frames. The "drv_i2c.c" driver needs the
following configuration data structures:

    - "i2C_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      VMIO table of drivers, "drv_wl55" in our case. The "sdrv_desc" field
      points to "i2c_sdrv_desc" which is found in "drv_i2c.c", the "init" field 
      of this structure is pointing to "i2c_init", which is therefore the 
      procedure called by "init_hypsys". Here, we use only one I2C controller,
      the "dev_link" field is pointing to a one item list of "Dev_link" 
      structure, here "i2c_dev_link[1]" and the "nb_dev" count of "Dev_link" 
      structures is set to 1.

    - "i2c_dev_link[1]" is the list of one "Dev_link" structure. The "dev_desc"
      field points to the "i2c_desc_stm32" from "i2c_stm32m7.c", which holds
      the low-level procedure addresses. The "dev_def" field points to the
      low-level parameter structure, here "i2c_def". And finally, "drv_def"
      points to the high-level driver parameter structures, here "i2c_drvdef".

    - The "i2c_def" structure type is "I2c_def_stm32", it helds the low-level
      parameters. For "i2c_stm32m7.c", the "I2c_def_stm32" structure holds the
      "i2c_num" ST defined controller number and the "fbus" frequency of SCL.

    - The "i2c_drvdef" structure type is "I2c_drvdef", it contains the high-
      level driver parameters. There is only one parameter, which is the delay
      in millisecond between two consecutive bus accesses.
*/

#define I2C_NBCTX            1         /* I2C controllers contexts           */
#define I2C_NBSCTX           1         /* I2C slave devices contexts         */
#define I2C_BCNT             1         /* I2C buffers number                 */
#define I2C_NBNODE           4         /* I2C nodes                          */
#define I2C_SBCNT            1         /* ?                                  */
#define I2C_BBSIZE           1         /* ?                                  */
#define I2C_BBCNT            1         /* ?                                  */
#define NUM_I2C              1         /* Number of used I2Cs                */
#include <drv_i2c.h>                   /* i2c_ctx i2c_etat                   */
                                       /*                                    */
#define STM32_I2C            1         /* Number of used STM32 I2Cs          */
#include <i2c_stm32.h>                 /* i2c_ctxstm32                       */

static const
    I2c_def_stm32 i2c_devdef[NUM_I2C]= /* Low level device parameters        */
  BEGIN                                /* ---------------------------------- */
    2                                , /* i2c_num  ST defined controller #   */
    100000                           , /* fbus     I2C bus speed (SCL freq)  */
    0x40912732                       , /* TIMINGR register content, width:   */
                                       /* - PRESC=0x4  (Timing prescaler)    */
                                       /* - SCLDEL=0x9 (Data setup time)     */
                                       /* - SDADEL=0x1 (Data hold time)      */
                                       /* - SCLH=0x27  (SCL high period)     */
                                       /* - SCLL=0x32  (SCL low period)      */

#ifdef _WORK_
    0x40D32A31                       , /* TIMINGR register content, width:   */
                                       /* - PRESC=0x4  (Timing prescaler)    */
                                       /* - SCLDEL=0xD (Data setup time)     */
                                       /* - SDADEL=0x3 (Data hold time)      */
                                       /* - SCLH=0x2A  (SCL high period)     */
                                       /* - SCLL=0x31  (SCL low period)      */
#endif

  END                                ; /* ---------------------------------- */

static const
    I2c_drvdef i2c_drvdef[NUM_I2C]   = /* High level driver parameters       */
  BEGIN                                /* ---------------------------------- */
    "I2C"                            , /* Device name                        */
    0                                , /* Low-level driver context number    */
    0                                , /* Count of slave devices             */
    MODE_MASTER                      , /* OFF MASTER SLAVE TUNMASTER TUNSLAVE*/
    0                                , /* SPI bus speed in KHz               */
    0                                , /* Wait duration between 2 requests   */
    0                                , /* Tunnel polling period in msec      */
    HNULL                            , /* List of slave parameters           */
  END                                ; /* ---------------------------------- */

static const Dev_link i2c_dev_link[] = /* I2C1 device                        */
  BEGIN                                /* ---------------------------------- */
    "I2C"                            , /* dev_name System device name        */
    &i2c_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    i2c_devdef                       , /* dev_def  Low-level parameters addr */
    i2c_drvdef                       , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link i2c_sdrv_link = /* I2C system driver                  */
  BEGIN                                /* ---------------------------------- */
    &i2c_sdrv_desc                   , /* sdrv_desc  Driver interface        */
    i2c_dev_link                     , /* dev_link   List of devices         */
    NUM_I2C                          , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.1.14 RAD driver - rad_sdrv_link
    ---------------------------------

    The RAD driver handles radio modem receiver transmitter. The HyperPanel
OS RAD driver only manages LoRa modem receiver transmitter. The HyperPanel
RAD system driver needs the following configuration structures:

    - "rad_sdrv_link" is the "Sdrv_link" structure which is pointed to by the
      "drv_wl55" VMIO table of drivers. The "sdrv_desc" field points to 
      "rad_sdrv_desc" which is found in "drv_rad.c", the "init" field of this 
      structure is pointing to "rad_init", which is so the procedure called by 
      "init_hypsys". Here, we use 1 RAD controller, the "dev_link" field is 
      pointing to a one item list of "Dev_link" structures, here
      "rad_dev_link[1]" and the "nb_dev" count of "Dev_link" structures is 1.

    - "rad_dev_link[1]" is the list of one "Dev_link" structure. The 3 
      "dev_desc" fields are pointing to the same "rad_desc_stm32" structure from
      "rad_stm32.c", which holds the low-procedure addresses. The "dev_def"
      fields are respectively pointing to "rad_def[0]". The "drv_def" fields
      are respectively pointing to "rad_drvdef[0]".

    - Each RAD controller needs one structure of low-level (hardware dependant)
      in our case one "Rad_def_stm32" structure (see rad_stm32.h). Those
      structures are independants but however there is no issue is we choose
      to put the 3 structures in one array, here "rad_def[1]". The first field
      of the structure is the ST assigned SubGHz number, in our case we use ST
      device 1.

    - Each RAD controller needs one structure of high-level driver parameters,
      a "Rad_drvdef" structure for "drv_rad.c". The 3 structures are 
      independant each other, but we can however choose to store those into one
      same storage array, her the "rad_drvdef[1]" array.

    - The Sub-GHz Radio device supports two forms of voltage regulation present
      on the board:
      - DC-DC buck converter (i.e SMPS converter) is used for the best power
        performance.
      - Linear LDO regulator is used for the lowest cost. Note that all Sub-GHz
        Radio device MUST support the Linear LDO regulator.
      Then, we MUST choose the type of the voltage regulation: LDO:0; SMPS:1.

    - The HSE32 reference clock should selects an external crystal (XTAL) or a
      temperature compensated crystal oscillator (TCXO). When the XTAL is
      selected, we should define the load capacitor trimming value. Range values
      are [11.3pF, 33.4pF] and corresponding to range value [0x00, 0x2F] with
      load capacitor trimming step size ~0.47 pf.

    - RF Switch device ...

*/

#define RAD_NBCTX             1        /* Number of used RADIO               */
#include <drv_rad.h>                   /* rad_nbiod rad_nbvl rad_state       */

#define RAD_STM32_NBCTX       1        /* Number of used STM32 SubGHz        */
#include <rad_stm32.h>                 /* rad_ctx_stm32[]                    */

static const Rad_def_stm32             /*                                    */
                  rad_def[RAD_NBCTX] = /* Low level device definition        */
  BEGIN                                /* ---------------------------------- */
    1                                , /* SUBGHZ --------------------------- */
    RFO_LP                           , /* Power Amplifier mode supported:    */
                                       /*   - RFO_LP_HP Low/High-power       */
                                       /*   - RFO_LP    Low-power  only      */
                                       /*   - RFO_HP    High-power only      */
    15                               , /* Max TX power in LP mode = 15 dBm   */
    22                               , /* Max TX power in HP mode = 22 dBm   */
     1                               , /* HSE32 clock source: XTAL:0; TCXO:1 */
     1                               , /* Regulator mode: LDO:0; SMPS:1      */
    0x20                             , /* XTAL load capacitor trimming       */
                                       /* RF Switch Control signals GPIO's:  */
                                       /* name, invSignal, ctrl_OFF,         */
                                       /* ctl_hp_TX, ctl_lp_TX, ctl_RX.      */
    "RFSWCTL1"  , 0 , 0 , 0 , 1 , 1  , /*   RF   ctrl switch 1 signal (PC4)  */
    "RFSWCTL2"  , 0 , 0 , 1 , 1 , 0  , /*   RF   ctrl switch 2 signal (PC5)  */
    "RFSWCTL3"  , 0 , 0 , 1 , 1 , 1  , /*   RF   ctrl switch 3 signal (PC3)  */
                                       /*                                    */
  END                                ; /* ---------------------------------- */

static const
    Rad_drvdef rad_drvdef[RAD_NBCTX] = /* RAD device definition              */
  BEGIN                                /* ---------------------------------- */
                                       /* ST SUBGHZ device *******************/
    0                                , /* Number of the low-level context    */
  END                                ; /* ---------------------------------- */

static const Dev_link rad_dev_link[] = /* RADIO                              */
  BEGIN                                /* ---------------------------------- */
    "sub-GHz"                        , /* dev_name System device name        */
    &rad_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    &rad_def[0]                      , /* dev_def  Low-level parameters addr */
    &rad_drvdef[0]                   , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Sdrv_link rad_sdrv_link=  /* RAD system driver                  */
  BEGIN                                /* ---------------------------------- */
    &rad_sdrv_desc                   , /* sdrv_desc  Driver interface        */
    rad_dev_link                     , /* dev_link   List of devices         */
    RAD_NBCTX                        , /* nb_dev     Count of devices        */
  END                                ; /* ---------------------------------- */


/*  9.2  VMIO character drivers configuration data
    ==============================================

    9.2.1 GPIO driver
    -----------------

    The GPIO driver has two software interfaces, its system driver interface
(see "GPIO driver - gpio_sdrv_link") and the present character driver
interface. The GPIO character interface only needs one "Drv_link" structure,
here "gpio_cdrv_link", which is pointed to by the "drv_wl55" VMIO list of
driver. So, "drv_wle5" points to "gpio_sdrv_link" and also to "gpio_cdrv_link".

    The "drv_desc" field of "gpio_cdrv_link" points to the "gpio_drv_desc"
structure from "drv_gpio.c". The "dev_link" field points to the "gpio_dev_link"
structure which can be found at "GPIO driver - gpio_sdrv_link". In fact,
initialization of the unique GPIO controller we use here is done by the GPIO
system driver.

    Besides this configuration data, the "drv_gpio.c" needs a const table which
name must be "gpio_tsubcnt". This table indicates the number of GPIO pins for
each device, for each controller, in our case we have only one controller, which
managed all the GPIO_NBPIN pins.

*/

static const Drv_link gpio_cdrv_link = /* GPIO character driver              */
  BEGIN                                /* ---------------------------------- */
    &gpio_drv_desc                  ,  /* drv_desc   Driver interface        */
    gpio_dev_link                   ,  /* dev_link   List of devices         */
    GPIO_NBIOD                      ,  /* nb_dev     Count of devices        */
    HNULL                              /* pipe       Pipe interface address  */
  END                                ; /* ---------------------------------- */

/* Needed by "drv_gpio.c" ...................................................*/

USHORT gpio_tsubcnt [GPIO_NBIOD] = { GPIO_NBPIN } ;


/*  9.2.2  ASY driver
    -----------------

    The ASY driver handles Universal Asynchronous Receiver Transmitter devices 
(UART). Despite the fact that some ST devices are USART able (Universal
Asynchronous and Synchronous Receiver Transceiver), the ASY driver does not
handles synchronous modes. One has to note that the "drv_asy.c" module handles
TWO distinct drivers : A system driver (see 9.1.10 ASY driver - asy_sdrv_link)
and a the present character driver. The character driver needs the following
configuration data, in top of the system driver configuration data:

    - One "Drv_link" structure, here "asy_cdrv_link", which is pointed to by
      the "drv_wl55" VMIO driver table. So, the "drv_wl55" has two entries
      corresponding to "drv_asy.c". One one side, "drv_wl55" is pointing to 
      "asy_sdrv_link", the system ASY driver. On the other side, to the second
      structure "asy_cdrv_link". The "drv_desc" field is pointing to 
      "asy_drv_desc", which is located in "drv_asy.c". The "dev_link" field is
      pointing to an array of 3 "Dev_link" structures, here "cdev_asy[3]".

    - The "cdev_asy[]" array contents are the same ones as "asy_dev_link[]",
      the array of "Dev_link" for the system driver. Only the 3 devices names
      are different. One have to note that the system driver is ALWAYS 
      initialized before the character driver and that an UART device which has
      been initialized by the system driver does not need to be initialized
      again by the character driver. So, the "cdev_asy" is useless. However, if 
      a device is removed from "asy_dev_link" (system driver) then it MUST be 
      present in "cdev_asy[]". This is the reason why we fill "cdev_asy", even
      if its data may be of no use.
*/


static const Dev_link  cdev_asy[] =    /* Device ASY                         */
  BEGIN                                /* ---------------------------------- */
    "ASY"                            , /* dev_name character device name     */
    &asy_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    &asy_devdef[0]                   , /* dev_def  Low-level parameters addr */
    &asy_drvdef[0]                   , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Drv_link asy_cdrv_link =  /* Driver ASY                         */
  BEGIN                                /* ---------------------------------- */
    &asy_drv_desc                    , /* drv_desc   Driver interface        */
    cdev_asy                         , /* dev_link   List of devices         */
    ASY_NBCTX                        , /* nb_dev     Count of devices        */
    HNULL                              /* pipe       Pipe interface address  */
  END                                ; /* ---------------------------------- */


/*  9.2.2  LED driver
    -----------------

    The LED driver handles blinking patterns for a number of LED's. Whatever
the hardware of LED's (of GPIO's), the LED driver provides up to 4 controller,
but in our case we only use one, and all the LED's are sub-channels of this 
unique controller. The LED driver is a character driver, which needs the 
following configuration data:

    - A "Drv_link" structure, here "led_cdrv_link", which is pointed to by the
      "drv_wl55" VMIO list of drivers. The "drv_desc" field of this structure
      is pointing to the "led_drv_desc" structure of "drv_led.c". The "dev_link"
      field points to the one item list of "Dev_link" structure, which is the
      "cdev_led" structure defined just below. The "nb_dev" count of devices is
      here equal to 1.

    - The "cdev_led" structure is a "Dev_link" structure for the LED controller,
      its "dev_desc" field is pointing to the "led_desc" structure, defined
      here below.

    - The "led_desc" structure is of type "Led_desc", which is defined in the
      "drv_led.h" include file. This structure holds two fields "info" and "set"
      which are low-level procedure addresses. In our case, the fields are
      respectively set to "infoLED" and "setLED", which are near the end of the
      present file.

    Besides the configuration data, other const data are needed, namely:

    - The "drv_led.c" driver needs a const table named "led_tsubcnt[]" which
      provides, for each controller, the cound of attached LED. Here, we have
      only one device, one LED controller and so the size of this array is 1.

    - The "infoLED" procedure, which is defined near the end of this file, needs
      3 const data tables named "lednames[]", "ledcol[]" and "ledsta". The
      "infoLED" procedure will in fact return the 3 table addresses to its
      caller, which is always "drv_led.c"

    - The "setLED" procedure, also defined near the end of the present file,
      needs one const table names "leppio[]". Each entry of this table is made
      of 2 UCHAR's, which are respectively the GPIO bank number and the GPIO
      number.
*/

/* LED controller and LED contexts ......................................... */

#define LEDCTRL_NB   1                 /* Count of LED controllers           */

#define LED_NB0      3                 /* Controller 0  LED number           */
#define LED_NB1      0                 /* Controller 1  LED number           */
#define LED_NB2      0                 /* Controller 2  LED number           */
#define LED_NB3      0                 /* Controller 3  LED number           */

#include <drv_led.h>                   /* ledctrl_ctx[]  led_ctx[] led_etat[]*/

/* LEDs are PB15/PB9/PB11 ...................................................*/

#define LEDB_PIO           2,   15    /* PIO number for LEDB BLUE       PB15 */
#define LEDG_PIO           2,   9     /* PIO number for LEDG GREEN      PB9  */
#define LEDR_PIO           2,   11    /* PIO number for LEDR RED        PB11 */

static void  infoLED                   (IWORD,const CHAR**,IWORD*,
                                        const CHAR***,
                                        const UCHAR**,const UCHAR**          );
static void  setLED                    (IWORD,IWORD,IWORD                    );

/* Configuration data for "drv_led.c" .......................................*/

static const Led_desc led_desc =       /* Low-level procedure addresses      */
  BEGIN                                /* ---------------------------------- */
    infoLED                          , /* info  Retreive LED controller info */
    setLED                           , /* set   Set one LED state            */
  END                                ; /* ---------------------------------- */

static const Dev_link cdev_led =       /* Device LED controller              */
  BEGIN                                /* ---------------------------------- */
    "LED"                            , /* dev_name character device name     */
    &led_desc                        , /* dev_desc Low-level procedure addr  */
    HNULL                            , /* dev_def  Low-level parameters addr */
    HNULL                            , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Drv_link led_cdrv_link =  /* LED character driver               */
  BEGIN                                /* ---------------------------------- */
    &led_drv_desc                    , /* drv_desc   Driver interface        */
    &cdev_led                        , /* dev_link   List of devices         */
    LEDCTRL_NB                       , /* nb_dev     Count of devices        */
    HNULL                              /* pipe       Pipe interface address  */
  END                                ; /* ---------------------------------- */

/* Needed by "drv_led.c" ................................................... */

const USHORT led_tsubcnt [LEDCTRL_NB] = { LED_NB0 } ;

/* Needed by "infoLED" ......................................................*/

static const CHAR *lednames[LED_NBTOT]=/* Noms des LEDs                      */
  BEGIN                                /* ---------------------------------- */
    "BLUE", "GREEN", "RED"           , /* Noms des LEDs par couleurs         */
  END                                ; /* ---------------------------------- */

static const UCHAR ledcol[LED_NBTOT] = /* Couleurs des LEDs                  */
  BEGIN                                /* ---------------------------------- */
    C_BLUE, C_GREEN , C_RED          , /* LEDs tri-colores.                  */
  END                                ; /* ---------------------------------- */

static const UCHAR ledsta[LED_NBTOT] = /* Etats des LEDs                     */
  BEGIN                                /* ---------------------------------- */
    ST_OFF, ST_OFF, ST_OFF           , /* Toutes les LEDs sont eteintes.     */
  END                                ; /* ---------------------------------- */

static const CHAR *ledpionames[LED_NBTOT]=/* GPIO user names for the LEDs    */
  BEGIN                                /* ---------------------------------- */
    "LEDB", "LEDG", "LEDR"           , /* List of GPIO user names            */
  END                                ; /* ---------------------------------- */

/* Needed by "setLED" .......................................................*/

static UCHAR ledpio[LED_NBTOT][2]    ; /* For each LED, bank and pin numbers */


/*  9.2.5  RAD driver
    -----------------

    The RAD driver handles Radio Receiver Transmitter devicesr. One has to note
that the "drv_rad.c" module handles TWO distinct drivers : A system driver (see
9.1.12 RAD driver - rad_sdrv_link) and a the present character driver. The
character driver needs the following configuration data, in top of the system
driver configuration data:

    - One "Drv_link" structure, here "rad_cdrv_link", which is pointed to by
      the "drv_wl55" VMIO driver table. So, the "drv_wl55" has two entries
      corresponding to "drv_rad.c". One one side, "drv_wl55" is pointing to 
      "rad_sdrv_link", the system RAD driver. On the other side, to the second
      structure "rad_cdrv_link". The "drv_desc" field is pointing to 
      "rad_drv_desc", which is located in "rad_asy.c". The "dev_link" field is
      pointing to an array of 1 "Dev_link" structures, here "cdev_rad[1]".

    - The "cdev_rad[]" array contents are the same ones as "rad_dev_link[]",
      the array of "Dev_link" for the system driver. Only the 1 devices names
      are different. One have to note that the system driver is ALWAYS 
      initialized before the character driver and that a RAD device which has
      been initialized by the system driver does not need to be initialized
      again by the character driver. So, the "cdev_rad" is useless. However, if 
      a device is removed from "rad_dev_link" (system driver) then it MUST be 
      present in "cdev_rad[]". This is the reason why we fill "cdev_rad", even
      if its data may be of no use.
*/

static const Dev_link  cdev_rad[] =    /* Device RAD                         */
  BEGIN                                /* ---------------------------------- */
    "RAD1"                           , /* dev_name character device name     */
    &rad_desc_stm32                  , /* dev_desc Low-level procedure addr  */
    &rad_def[0]                      , /* dev_def  Low-level parameters addr */
    &rad_drvdef[0]                   , /* drv_def  High-level parameters addr*/
  END                                ; /* ---------------------------------- */

static const Drv_link rad_cdrv_link =  /* Driver RAD                         */
  BEGIN                                /* ---------------------------------- */
    &rad_drv_desc                    , /* drv_desc   Driver interface        */
    cdev_rad                         , /* dev_link   List of devices         */
    RAD_NBCTX                        , /* nb_dev     Count of devices        */
    HNULL                              /* pipe       Pipe interface address  */
  END                                ; /* ---------------------------------- */


/*
    9.3  Block drivers definitions

      There is no block driver in this board configuration.

    9.4  Volume drivers definitions

      There is no volume driver and no file system in this board configuration.

    9.5 Protocols definitions

    9.5.1  LPWAN protocol
    ---------------------

    The LPWAN protocol driver handles LoRaWAN-1.0.4 protocol implementation.
This driver needs the following configuration data:

    - "lpwan_drv_link" is the "Drv_link" structure which is pointed to by the
      "drv_wl55" VMIO table of drivers. The "drv_desc" field points to
      "lpwan_drv_desc" which is found in "pro_lpwan.c". With protocol drivers,
      the "dev_link" field is HNULL and the "nb_dev" count of "Dev_link"
      structures is 0.

    - "lpwan_drv_link" is not defined in the present module but in the
      "pro_lpwan.c" source file of the protocol driver.

*/

#define LPWAN_NBCTX          1         /* Number of LoRaWAN context          */
#include <pro_lpwan.h>                 /* lpwan_nbiod lpwan_nbvl lpwan_state */

extern const Drv_link  lpwan_drv_link; /* IP protocol character driver desc. */

/*
    9.6 Services definitions

      There is no services in this board configuration.
*/


/*  9.7  VMIO driver list and other global data structures
    ======================================================

          9.7.1  VMIO driver list - drv_wl55
          9.7.2  Board_link structure - board_wl55
          9.7.3  Table of Drv_req_ctx contexts - drv_req_ctx


    9.7.1  VMIO driver list - drv_wl55
           ---------------------------
*/

static const Drv_link* const drv_wl55[]= /* VMIO list of drivers             */
  BEGIN                                  /* -------------------------------- */

/* System drivers .......................................................... */

    (const Drv_link *)&sys_sdrv_link    , /*  0 SYSTEM                       */
    (const Drv_link *)&board_sdrv_link  , /*  1 BOARD                        */
    (const Drv_link *)&ramh_sdrv_link   , /*  2 RAM_HARD                     */
    (const Drv_link *)&rams_sdrv_link   , /*  3 RAM_SOFT                     */
    (const Drv_link *)&rslv_sdrv_link   , /*  4 RESOLVE (avant carac)        */
    (const Drv_link *)&vmit_sdrv_link   , /*  5 VMIT                         */
    (const Drv_link *)&vmio_sdrv_link   , /*  6 VMIO                         */
    (const Drv_link *)&tim_sdrv_link    , /*  7 TIME                         */
    (const Drv_link *)&gpio_sdrv_link   , /*  8 GPIO                         */
    (const Drv_link *)&asy_sdrv_link    , /*  9 ASY                          */
    (const Drv_link *)&i2c_sdrv_link    , /* 10 I2C                          */
    (const Drv_link *)&rad_sdrv_link    , /* 11 RAD                          */

/* Characters drivers ...................................................... */

    (const Drv_link *)&gpio_cdrv_link   , /* 12 GPIO                         */
    (const Drv_link *)&asy_cdrv_link    , /* 13 ASY                          */
    (const Drv_link *)&led_cdrv_link    , /* 14 LED                          */
    (const Drv_link *)&rad_cdrv_link    , /* 15 RAD                          */

/* Block drivers ........................................................... */

/* Volume drivers .......................................................... */

/* Protocols ............................................................... */

    (const Drv_link *)&lpwan_drv_link   , /* 16 LPWAN                        */

/* Services ................................................................ */

/* HNULL end of list marker ................................................ */

    HNULL                            , /* END OF LIST marker                 */
  END                                ; /* ---------------------------------- */


/*  9.7.2  Board_link structure - board_wl55
    ----------------------------------------
*/

Board_link board_wl55 =                /* Board Nucleo kit STM32WL55JC       */
  BEGIN                                /* ---------------------------------- */
    "NUCLEO-WL55JC1"                 , /* Nom du board                       */
    drv_wl55                         , /* Liste des drivers                  */
  END                                ; /* ---------------------------------- */


/*  9.7.3  Table of Drv_req_ctx contexts - drv_req_ctx
    --------------------------------------------------

    WARNING: This table uses as an index the major number, which is intended
    to be used as the "vmos_driver[]" index ONLY. So one has to take care to
    always simultaneoulsy update both tables accordingly.
*/

Drv_req_ctx drv_req_ctx[] =
  BEGIN
/*   itfctx          sub_lw        nbiod        nbmodd   sessid_cond_act[]   */
/*   -------------   -----------   ----------   ------   -----------------   */
  {  HNULL         , HNULL       , 0          , 0      , {0}              } ,
  {  HNULL         , HNULL       , 0          , 0      , {0}              } ,

  {  gpio_itfctx   , gpio_sub_lw , GPIO_NBCTL , 1      , {0}              } ,
  {  led_itfctx    , led_sub_lw  , LEDCTRL_NB , 1      , {0}              } ,
  {  rad_itfctx    , HNULL       , RAD_NBCTX  , 1      , {0}              } ,

  {  lpwan_itfctx  , HNULL       , LPWAN_NBCTX, 1      , {0}              } ,
  END                                ;

/*
    10   Beginning of the code
    __________________________

board_init       Board dependant initialization procedure
vm_sleep         Enter CPU sleep mode, here WFI
infoLED          Low-level code for "drv_led.c" - info
setLED           Low-level code for "drv_led.c" - set
init_aut_script  Empty procedure, called by "vmk1.c"
init_aut_panel   Empty procedure, called by "vmk1.c"
sdrv_empty       Empty procedure for system drivers init/end
*/

/*  Procedure board_init ----------------------------------------------------*/
/*
    Purpose : Initialization procedure of the BOARD system driver. In fact,
              the purpose of the BOARD system driver is to allow "init_hypsys"
              to call "board_init" and nothing more. We remind that "init_board"
              is the second procedure to be called by "init_hypsys", just after
              the initialization procedure of the SYSTEM driver.

    Author : JYA                                 Last modification : 02/01/2022

...............................................................................

    Calling example : board_init(drv)

    Input     : drv      IWORD      Index in the VMIO driver table

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

static IWORD  PROCEDURE board_init(IWORD drv)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          ldev              ; /* SET timer number 1..14             */
    IWORD          chan              ; /* Optional TIM channel number        */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("board_init")            ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Activate the use of CPU sleep, whenever "vm_sleep" is called    *
 * -------   We simply call "set_idle(1)"                                    *
 *           --------------------------------------------------------------- *
 *           VERY IMPORTANT NOTICE : When using GDB/OpenOCD, we have a       *
 *           regular polling of the target by the debugger or OpenOCD. But   *
 *           when CPU is put in IDLE, it seems it does not answers any more  *
 *           to GDB/OpenOCD requests and we get on the GDB window:           *
 *           DAP transaction stalled (WAIT) - slowing down                   *
 *           and then using GDB becomes very difficult or impossible. What   *
 *           is sure, we never get this message if "set_idle(1)" is not      *
 *           called. Therefore, we have a "use_gdb" global variable which    *
 *           is set by "gdb.cfg" to 0x12345678 and we call "set_idle(0)" if  *
 *           variable is 0x12345678 (do not use IDLE mode) else with any     *
 *           other value we call "set_idle(1)"                               *
 *           --------------------------------------------------------------- *
 *           The "use_gdb" address is defined by "linker.cfg" as a linker    *
 *           symbol, so an address. If we want the get the value which is    *
 *           located at tha adress, we have to use "*use_gdb"                *
 *****************************************************************************/

                                       /* If GDB has written at address      */
   set_idle(use_gdb EQ 0x12345678  ?   /* "used_gdb" the 0x12345678 value    */
            0                      :   /* then we call "set_idle(0)" else    */
            1                      ) ; /* we call "set_idle(1)"              */


/*****************************************************************************
 * Step  2 : Initialize global variable holding timer numbers :              *
 * --------  - The "init_timarmv7m" procedure form "tim_arm.c" needs the     *
 *             "sys_tim" global variable as input, it should hold the ST     *
 *             defined TIM number we use a the system "tick". Here, we scan  *
 *             our "ref_tim32[]" table, looking for the "SYS_TIM" name, and  *
 *             we get "sys_tim"                                              *
 *           - The "filter_stm32" from "tim_stm32.c" needs a timer and       *
 *             assumes that the "flt_tim" global variable holds the ST TIM   *
 *             number. In our case, the FLT_TIM entry of "ref_tim32[]" holds *
 *             that value and we use the "get_ref_tim_stm32" procedure in    *
 *             order to retreive it.                                         *
 *****************************************************************************/

    get_ref_tim_stm32("SYS_TIM"  ,    /* Timer name for this configuration   */
                       &ldev     ,    /* Corresponding ST TIM number 1..14   */
                       &chan      ) ; /* Optional channel number (not used)  */
                                      /*                                     */
    sys_tim = ldev                  ; /* Keep in the "sys_tim" gloaal var    */
                                      /*                                     */
    get_ref_tim_stm32("FLT_TIM"  ,    /* Timer name for this configuration   */
                       &ldev     ,    /* Corresponding ST TIM number 1..14   */
                       &chan      ) ; /* Optional channel number (not used)  */
                                      /*                                     */
    flt_tim = ldev                  ; /* Keep in the "sys_tim" gloaal var    */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("board_init")            ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure vm_sleep ------------------------------------------------------*/
/*
    Purpose : Allow the CPU to enter it sleep mode, if any

    Author : JYA                                 Last modification : 01/13/2022

...............................................................................

    Calling example : vm_sleep( par )

    Input     : par      IWORD      0:Do not enter sleep mode 1:Enter sleep

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE vm_sleep(IWORD par)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("vm_sleep")              ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : If "par" is set, we execute the Wait For Irq ARM machine        *
 * -------   instruction: The CPU enters low power mode until and IRQ is     *
 *           raised.                                                         *
 *****************************************************************************/

    if (par) asm("WFI")              ; /* Si par = 1, on utilise WFI.        */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("vm_sleep")              ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure infoLED -------------------------------------------------------*/
/*
    Purpose : Low-level procedure for "drv_led.c". The procedure return
              addresses of const arrays, each array providing one information
              for every LED (name, color and initial state).

    Author : JYA                                 Last modification : 01/13/2022

...............................................................................

    Calling example : infoLED(dev, name, nbled, color, names, colors, states )

    Input     : dev      IWORD        Device/LED controller number

    Output    : name     const CHAR**  LED controller name
                nbleds   IWORD         Count of LEDs
                names    const CHAR*** List of addresses of LED names
                colors   const UCHAR*  List of color codes (UCHAR)
                states   const UCHAR   List of initial states

    Notes :  - This procedure is called once by "drv_led.c" and it is
               guaranted that this call always occurs before any call to
               the "setLED" procedure. So, we can here safely initialize the 
               "ledpio[][]" global variable which is needed by "setLED".

...............................................................................
*/

static void PROCEDURE infoLED(IWORD dev, const CHAR **name, IWORD *nbleds,
                              const CHAR ***names, const UCHAR **colors,
                              const UCHAR **states                         )

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD                 i          ; /* Loop counter on leds               */
    INT                   gdev       ; /* GPIO driver device number          */
    INT                   bank       ; /* GPIO bank number 1..9              */
    INT                   pin        ; /* GPIO pin number 1..15              */
    const Gpio_pindef    *pdef       ; /* Gpio_pindef structure address      */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("infoLED")               ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Return all the returned values addresses, all the pointer data  *
 * -------   items are const:                                                *
 *           - The LED controller name                                       *
 *           - The count of attached LED's to the controller                 *
 *           - The table of LED name addresses                               *
 *           - The table of LED's color codes                                *
 *           - The table of LED's initial states                             *
 *****************************************************************************/

    *name   = "Nucleo board"         ; /* Controller name                    */
    *nbleds = LED_NBTOT              ; /* Count of LEDs                      */
    *names  = lednames               ; /* Names table                        */
    *colors = ledcol                 ; /* Colors table      .                */
    *states = ledsta                 ; /* Initial states table               */


/*****************************************************************************
 * Step  2 : Initialize the contents of the "ledpio[][]" array, which is     *
 * -------   needed by the "setLED" procedure that follows. Each pair of     *
 *           entry holds the GPIO bank number (1..9) and the pin number      *
 *           (0..15), the bank number is stored in "ledpio[i][0]" and the    *
 *           pin number is put in "ledpio[i][1]"                             *
 *****************************************************************************/

    for (i = 0; i LT LED_NBTOT; i++)   /* Loop with the LED_NBTOT LED's      */
      BEGIN                            /*                                    */
                                       /* ---------                          */
        gpio_search_bank(                         /*                         */
                (CHAR*)ledpionames[i]        ,    /* GPIO user name          */
                GPIO_USER                    ,    /* Indicate user name      */
                (Gpio_desc*)&gpio_desc_stm32 ,    /* Tag for the GPIO drv    */
                &gdev                        ,    /* VMIO GPIO driver number */
                &bank                        ,    /* GPIO bank number        */
                &pin                         ,    /* GPIO pin number         */
                &pdef                         ) ; /* Gpio_pindef struct addr */
                                                  /*                         */
                                       /* ----------                         */
        ledpio[i][0] = bank          ; /* Keep the GPIO bank number          */
        ledpio[i][1] = pin           ; /* Keep the GPIO pin  number          */
      END_FOR

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("infoLED")               ; /* Message                            */
#endif                                 /* End   conditional compilation      */

  END_PROC                             /* End of the procedure               */

/*  Procedure setLED --------------------------------------------------------*/
/*
    Purpose : Set the ON/OFF state of one LED

    Author : JYA                                 Last modification : 01/13/2022

...............................................................................

    Calling example : setLED( dev, sc, state )

    Input     : dev      IWORD      device/LED controller number
                sc       IWORD      Sub-channel number
                state    IWORD      New state 0:OFF 1:ON

    Output    : 

    Notes : - The "dev" parameter is here not used, because in our case we
              only have one LED controller.

...............................................................................
*/

static void   PROCEDURE setLED(IWORD dev, IWORD sc, IWORD state)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("setLED")                ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    set_output_stm32(ledpio[sc][0],
                     ledpio[sc][1],
                     state         ) ;

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("setLED")                ; /* Message                            */
#endif                                 /* End   conditional compilation      */

  END_PROC                             /* End of the procedure               */


IWORD PROCTEST init_aut_script(UINT *id)
  BEGIN
    *id = 0xFFFFFFFF                  ;
    return ( 0 )                      ;
  END_PROC

IWORD PROCTEST init_aut_panel(UINT *id)
  BEGIN
    *id = 0xFFFFFFFF                  ;
    return ( 0 )                      ;
  END_PROC

IWORD PROCEDURE sdrv_empty(IWORD drv) 
  BEGIN
    return ( 0 )                      ;
  END_PROC
