/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/aut.h"                 */
/*  Constants and definitions for FSM's.                                     */
/*                                                                           */
/*  Author : JYA                        Last modification : 03/06/2022       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  S_trans structure field values

         2  Structures and typedefs
            2.1  S_evt event structure, used by VMK and VMIO
            2.2  S_trans transition structure, used by VMK and VMIO
            2.3  Aut_ctx Automaton context structure, used by VMK
            2.4  Aut_desc automaton description structure, used by VMIO
            2.5  D_trans transition structure for dialog/protocol FSM's
            2.6  S_putevt_err put event error structure

         3  Public variables

         4  Prototypes of public procedures

*/


#ifndef FSM_H                          /* If this file is not already        */
#define FSM_H                          /* included, include it.              */


/*
    1    Defines
    ____________

    1.1  S_trans structure field values
*/

/* Values for the "code" field ................................................

   The VMK FSM engine may generate two conditions named ENTER and EXIT, which
   are needed to "enter" and "exit" from a transition table, whenever
   hierarchical FSM mechanism is used. So, ENTER and EXIT may be used as
   the first "code" field value of a transition. For now, those are not used
   by VMIO transition tables.
*/

#define ENTER       0                  /* O_CALL self-generated condition    */
#define EXIT        1                  /* O_RET  self-generated condition    */


/* Values for bits b28..31 of field "op" ......................................

    Bits b28..31 of the "op" fields are used by the VMK FSM engine as the
    flow control operation code values. For now, those are not used by VMIO
    transition tables.
*/

#define O_GOTO(p)   (0x00000000 | (p)) /* Goto state "nstate"                */
#define O_CALL(p)   (0x10000000 | (p)) /* Push state var and goto "nstate"   */
#define O_RET(p)    (0x20000000 | (p)) /* Pop state vari (RETURN)            */
#define O_EXIT(p)   (0x30000000 | (p)) /* Subroutine exit to error handler   */
#define O_CND(p)    (0x40000000 | (p)) /* Goto then generate a condition     */


/* Values for the "nstate" field ..............................................

   The "nstate" field is the next state number. Two special degenerate values
   are avialable. STERR is used when an event is received in a state where it
   must never be received, it this happens we have a transition table design
   mistake. STNEVER is used when the treatment for the transition must always
   raise a condition, menaning that the "nstate" field of the transition will
   always be ignored.
*/

#define STNEVER     0x0FFE             /* Next step = never used             */
#define STERR       0x0FFF             /* Next step = err, impossible trans  */


/*
    2    Structures and typedefs
    ____________________________


   2.1  S_evt event structure, used by VMK and VMIO
   ------------------------------------------------

   This structure is the most important one of the OS, meaning that 
   modifications can only be made by the OS architect. It is used by both
   the VMK FSM engine (application container) and the VMIO FSM engine (IO
   container). All the App container queues are arrays or linked lists of
   S_evt structures. The three input queuesof the VMIO are linked lists of
   this same structure.

*/

typedef struct                         /* One event                          */
  BEGIN                                /* ---------------------------------- */
    USHORT      aut                  ; /* Target automaton number            */
    USHORT      lw                   ; /* Target logical way number          */
    USHORT      node                 ; /* Target node number (0 local)       */
                                       /*                                    */
    USHORT      code                 ; /* Event code                         */
    UINT        reserve              ; /* IOD value                          */
    UCHAR      *address              ; /* Data buffer address                */
    INT         length               ; /* Data buffer length                 */
    UINT        flags                ; /* Interface procedure flags parameter*/
    UINT        res2                 ; /* Additionnal reserve field          */
                                       /*                                    */
    USHORT      reqaut               ; /* Request originator automaton       */
    USHORT      reqlw                ; /* Request originator logical way     */
    USHORT      reqnode              ; /* Request originator node            */
    USHORT      reqid                ; /* User request identifier            */
    USHORT      sessid               ; /* Session identifier                 */
    UINT        rteid                ; /* Route identifier                   */
                                       /*                                    */
    USHORT      saut                 ; /* Event sender  automaton            */
    USHORT      slw                  ; /* Event sender  logical way          */
    USHORT      ownidx               ; /* Event owner                        */
    UINT        cptevt               ; /* Event unique identifier/counter    */
    UINT        dbg                  ; /* Data for performance checking      */
    USHORT      next                 ; /* Link to the next queued event      */
  END           S_evt                ; /* ---------------------------------- */

typedef struct                         /* Evt RESP_* informations            */
  BEGIN                                /* ---------------------------------- */
    USHORT      reqaut               ; /* reqaut  of the answer              */
    USHORT      reqlw                ; /* reqlw   of the answer              */
    USHORT      reqnode              ; /* reqnode of the answer              */
    USHORT      reqid                ; /* reqid   of the answer              */
    USHORT      sessid               ; /* sessid  of the answer              */
  END           Resp_info            ; /* ---------------------------------- */

#define SZEVT   sizeof(S_evt)          /* S_evt structure size in bytes      */

/*
   2.2  S_trans transition structure, used by VMK and VMIO
   ---------------------------------------------------

   This structure is needed by VMK and VMIO transition tables. Each transition
   table is a list of "S_trans" structures, each "S_trans" structure describes
   one and one only transition.
*/


typedef struct                         /* Automaton transition               */
  BEGIN                                /* ---------------------------------- */
    USHORT      code                 ; /* Incoming event code                */
    IWORD       (*proc)(INT)         ; /* Treatment procedure address        */
    UINT        op                   ; /* Flow control + 28 bits parameter   */
    USHORT      nstate               ; /* Automaton next state number        */
  END           S_trans              ; /* ---------------------------------- */

/*
   2.3  Aut_ctx Automaton structure, used by VMK
   ---------------------------------------------

*/

typedef struct                         /* One automaton data                 */
  BEGIN                                /* ---------------------------------- */
    CHAR           name[8]           ; /* FSM unique name                    */
    USHORT         stacksz           ; /* Depth of state stacks              */
                                       /* ---------------------------------- */
    const S_trans  (*tran[])[]       ; /* Array of S_trans Array addresses   */

  END              S_fsm             ; /* ---------------------------------- */

typedef struct                         /* A set of FSM's                     */
  BEGIN                                /* ---------------------------------- */
    USHORT         ownidx            ; /* Creator owner number               */
    CHAR           name[8]           ; /* FSM SET unique name                */
    const S_fsm  *const((*fsmlist)[]); /* Addr of an array of struct address */
    const USHORT   *alwnb            ; /* Counf of logical ways for every FSM*/
    USHORT         nbfsm             ; /* Count of FSM's                     */
                                       /* RAM storage addresses ------------ */
    USHORT         *astate           ; /* State variable stacks storage addr */
    UCHAR          *apstack          ; /* Stack pointers storage addr        */
    S_evt          *aevt             ; /* Copy of incoming event address     */
                                       /* Contextual values ---------------- */
    USHORT         lwcnt             ; /* Total count of logical ways        */
    USHORT         first             ; /* "push_evt_aut" queue head          */
    USHORT         last              ; /* "push_evt_aut" queue end           */
    USHORT         flags             ; /* List of option bits                */
  END              Aut_ctx           ; /* ---------------------------------- */


/*  2.5  D_trans transition structure for dialog/protocol FSM's
    -----------------------------------------------------------

   This structure is needed by VMK and VMIO dialog/protocol transition tables.
   Each transition table is a list of "D_trans" structures, each "D_trans" 
   structure describes one and one only transition. The dialog transition
   tables are "full" 2d transition tables, each state has one transition for
   each possible event, so there is no "code" value in the transition
*/

typedef struct                         /* One transition for protocol FSM    */
  BEGIN                                /* ---------------------------------- */
    IWORD          (*proc)()         ; /* Treatment procedure address        */
    UINT           next              ; /* Next state                         */
    USHORT         par               ; /* Flag "event to be sent"            */
  END              D_trans           ; /* ---------------------------------- */


/*  2.6  S_putevt_err put event error structure
    -------------------------------------------

    This structure is used by both VMK and VMIO, by the "set_putevt_vmk_err"
    and "set_putevt_vmio_err".
*/

typedef struct                         /* One transition for protocol FSM    */
  BEGIN                                /* ---------------------------------- */
    void           (*cb)()           ; /* User call-back procedure address   */
    INT            cbpar             ; /* Parameter to be given to "cb"      */
    CHAR           halt              ; /* 0:Do not call "vhalt" 1:Call vhalt */
  END              S_putevt_err      ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


#endif                                 /* End of FSM_H                       */
