/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/aut.l"                 */
/*  Constants and definitions for automatons handling.                       */
/*                                                                           */
/*  Author : JYA/FA/TE                  Last modification : 03/02/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Explanations
            1.2  VMK automatons numbers
            1.3  VMIO automatons numbers
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures

*/


#ifndef AUT_L                          /* If this file is not already        */
#define AUT_L                          /* included, include it.              */


/*
    1    Defines
    ____________
  

    1.1  Explanations
    -----------------

   This file contains the definitions of the VMK and VMIO automaton numbers,
   as well as the event numbers. There are currently two ranges of numbering
   for the automatons:

   . Numbers 0 to 127 are those of the VMK automatons, i.e. automatons executed
     by "vmk1.c".   

   . Numbers 128 to 255 are those of the VMIO automatons, i.e. automatons
     executed by "vmio.c". The value 128 is ALWAYS and ETERNALLY reserved for
     AUT_RESOLVE automaton.

   VMK automatons numbers are used as index in the five tables "anomvmk[]",
   "enom[]", "nnom[]", "fcode[]" and "b32fsm[]" located in "os/vmk/sys/vmk2.c".
   Each time we add, remove an automaton, or change the values of the numbers,
   it is necessary to modify these five tables. If we don't do it, not only the
   engine traces are false, but we can generate memory overwrites. The ranges
   of event numbers are as follows:

   . The events from 0 to 399 are codes that can be used in the interpreters,
     they have have menemonics. These codes are defined in "event.l". We have
     here the "general" codes of the VMK, as well as the "general" codes of the
     VMIO.

   . The events from 500 to 499 are the "general" codes of the VMIO. They are
     alsoe defined in "event.l". They have mnemonics, in case some of them
     "go back" to the VMK. But most of these mnemonics will never be used.

   . The events from 1024 on are the "private" event codes of VMK automatons.
     These codes are defined in the present file, which means that they are
     not mnemonics, like the "event.l" codes are. By convention, we reserve
     for each automaton of the VMK a range of 32 consecutive numbers. The first
     range thus reserved is: 1024 to 1024+32 = 1056, and is attributed to 
     the automaton "script" AUT_SCRIPT.

   . The events of code greater than or equal to 10000 are reserved for 
     applications. The event codes being USHORT, the largest value is 32767.
     Negative codes are not allowed.


    1.2  VMK automatons numbers
    ---------------------------

   These automaton numbers are between 0 and 127. For each automaton number, we
   find the codes of its private events. The first of these codes is always
   equal to 1024 + multiple of 32. Therefore, we always have at least 32 codes
   per automaton. This does not prevent some automatons from using two
   consecutive ranges of 32. 

   The first two automatons do not need specific codes, but however we reserve
   32 codes (1024 to 1055) for these two automatons, and therefore the first
   code of automaton 2 (AUT_SCRIPT) is 1024+32 = 1056. 

   VMK automatons list. This list correspond to the definition of the table
   "anomvmk[]" defined in "vmk2.c". The declarations of events codes must
    correspond to the tables "nom*[]" defined in "vmk2.c".

*/

/*****************************************************************************
 * To allow the TOs to be purged without risk of error, a particular value   *
 * of the "emet" fields of the events is reserved for the VMK TOs. It is the *
 * value 0.                                                                  *
 *****************************************************************************/

#define TIMERTO           0        /* vmk1.c, timerto                        */


/* Automaton MEMBUG - Allocation telecom buffers ............................*/


/* Automaton TASK - Task manager ........................................... */

                                   /* Private events range: 1024+2*32=1088   */
#define TASK_RUN       1088        /* Evt - Execute a task                   */
#define TASK_KILL      1089        /* Evt - Kill a task                      */
#define TASK_CLOSE     1090        /* Evt - End aof a task                   */
#define TASK_TIMER     1091        /* Evt - Task timer time-out              */
#define TASK_SIGNAL    1092        /* Evt - Task semaphore available         */
#define TASK_EVT0      1093        /* Evt - 1st  inter-task event            */
#define TASK_EVT1      1094        /* Evt - 2nd  inter-task event            */
#define TASK_EVT2      1095        /* Evt - 3rd  inter-task event            */
#define TASK_EVT3      1096        /* Evt - 4th  inter-task event            */
#define TASK_EVT4      1097        /* Evt - 5th  inter-task event            */
#define TASK_EVT5      1098        /* Evt - 6th  inter-task event            */
#define TASK_EVT6      1099        /* Evt - 7th  inter-task event            */
#define TASK_EVT7      1100        /* Evt - 8th  inter-task event            */
#define TASK_EVT8      1101        /* Evt - 9th  inter-task event            */
#define TASK_EVT9      1102        /* Evt - 10th inter-task event            */
#define TASK_EVT10     1103        /* Evt - 11th inter-task event            */
#define TASK_EVT11     1104        /* Evt - 12th inter-task event            */
#define TASK_EVT12     1105        /* Evt - 13th inter-task event            */
#define TASK_EVT13     1106        /* Evt - 14th inter-task event            */
#define TASK_EVT14     1107        /* Evt - 15th inter-task event            */
#define TASK_EVT15     1108        /* Evt - 16th inter-task event            */


/* Automaton SCREXE - Script interpretor ....................................*/

                                   /* Private events range: 1024+3*32=1120   */
#define SCREXE_CONT    1120        /* Evt - Execute a script                 */
#define SCREXE_BREAK   1121        /* Evt - Interrupt a script               */
#define SCREXE_LOAD    1122        /* Evt - Load a script                    */
#define SCREXE_SCHED   1123        /* Evt - Schedule a script                */


/* Automaton PAN - Panem manager ............................................*/

#define AUT_PAN           4        /* reserved                               */
                                   /* Private events range: 1024+4*32=1152   */


/* Automaton TKBD - Keyboard temporisation ................................. */

#define AUT_TKBD          5        /* kbd_fsm.c                              */
                                   /* Private events range: 1024+5*32=1184   */
#define TKBD_TEMPO     1184        /* Evt - Temporisation                    */


/* Automaton KBD - Keyboard ................................................ */

                                   /* Private events range: 1024+6*32=1216   */
#define KBD_RELEASE_R  1216        /* Evt - Key release                      */


/* Automaton MOUSE - Mouse ..................................................*/

#define AUT_MOUSE         7        /* mse_fsm.c                              */
                                   /* Private events range: 1024+7*32=1248   */
#define MOUSE_TEMPO    1248        /* Evt - End of temporisation             */
#define MOUSE_INIT     1249        /* Evt - First click on mouse button      */
#define MOUSE_STOP     1250        /* Evt - Return to state 0                */
#define MOUSE_BREAK    1251        /* Evt - Wait for move_cursor() end       */
#define MOUSE_RETURN   1252        /* Evt - End of move_cursor()             */
 

/* Automaton TIME - Timers ..................................................*/

#define AUT_TIME          8        /* tim_fsm.c                              */
                                   /* Private events range: 1024+8*32=1280   */
#define TIME_WATCHDOG  1280        /* Evt - Watchdog                         */


/* Automaton DF - File system ...............................................*/

#define AUT_DF            9        /* df.c                                   */
                                   /* Private events range: 1024+9*32=1312   */


/* Total number of automatons ...............................................*/

#define AUT_VMK_LAST     10        /* End of VMK automation list             */


/*
    1.3  VMIO automatons numbers
    ----------------------------

  These automaton numbers are between 128 and 255. For each automaton number,
  we find the codes of its private events. The first of these codes is always
  equal to 1024 + 32 * number of the automaton. Therefore, we have 32 codes for
  each automaton.

   VMIO automatons list:

   AUT_MEM_VMIO         129           Telecom buffers allocations
   AUT_SPI              133           STM32 SPI driver
   AUT_ETH              143           Ethernet driver
   AUT_ARP              144           ARP protocol
   AUT_IP               145           IP protocol

*/

#define AUT_VMIO_FIRST  128        /* First VMIO automaton number            */


/* Automaton RESOLVE - Driver names resolution ..............................*/


#define AUT_RESOLVE     128        /* drv_rslv.c                             */
                                   /* Private events range: 1024+32*128      */

/* Automaton MEM_VMIO - Telecom buffers allocations .........................*/


#define AUT_MEM_VMIO    129        /* mem_fsm.c                              */
                                   /* Private events range: 1024+32*129      */

/* Automaton GPIO - GPIO driver .............................................*/


                                   /* Private events range: 1024+32*131      */


/* Automaton ASY - Asynchronous driver ......................................*/


                                   /* Private events range: 1024+32*131      */
#define REQ_IO         5216        /* Evt - req_lib.c relfsm request         */
#define IT_SIG         5217        /* Evt - jnctin signal state update       */
#define TO_SIG0        5218        /* Internal - Write queue temporisation   */
#define TO_SIG1        5219        /* Internal - Read queue  temporisation   */
#define TO_SIG2        5220        /* Internal - Other queue temporisation   */
#define TO_MSEC0       5221        /* Internal - Write queue temporisation   */
#define TO_MSEC1       5222        /* Internal - Read queue  temporisation   */
#define TO_MSEC2       5223        /* Internal - Other queue temporisation   */


/* Automaton I2C - I2C protocol .............................................*/

                                   /* Private events range: 1024+32*132      */

/* Automaton SPI - SPI protocol .............................................*/

                                   /* Private events range: 1024+32*133      */

/* Automaton CAN - CAN protocols ............................................*/

                                   /* Private events range: 1024+32*134      */

/* Automaton DAC - Digital to analog controller driver ......................*/

                                   /* Private events range: 1024+32*136      */

/* Automaton ADC - Analog to digital controller driver ......................*/

                                   /* Private events range: 1024+32*137      */

/* Automaton LED - LED driver ...............................................*/

                                   /* Private events range: 1024+32*138      */

/* Automaton KBD - Keyboard / Remote unit interface .........................*/


                                   /* Private events range: 1024+32*139      */

/* Automaton MSE - Mouse interface ..........................................*/

                                   /* Private events range: 1024+32*140      */

/* Automaton RCU - Remote Control Unit infra-red protocol ...................*/

                                   /* Private events range: 1024+32*141      */
#define C_RCV          5536        /* Cnd - On demand receive mode           */
#define C_GET          5537        /* Cnd - Continuous receive mode          */
#define C_SND          5538        /* Cnd - On demande send mode             */


/* Automaton RAD - Radio driver .............................................*/

                                   /* Private events range: 1024+32*142      */

/* Automaton ETH - Ethernet driver ..........................................*/


                                   /* Private events range: 1024+32*143      */
#define IT_DMACPY      5600        /* Evt - End of DMA RAM2RAM copy          */
#define IT_STA         5601        /* Evt - State change PHY status          */
#define TO_STA         5602        /* Evt - Polling timeout for PHY status   */
#define TO_NEG         5603        /* Evt - End of self-negociation timeout  */


/* Automaton ARP - ARP protocol .............................................*/


                                   /* Private events range: 1024+32*144      */
#define TO_ARP         5632        /* Evt - Checking / resolution timout     */


/* Automaton IP - IP protocol ...............................................*/


                                   /* Private events range: 1024+32*145      */
#define TO_GETIP       5664        /* Cnd - Open in IP mode                  */
#define TO_RENEW       5665        /* DHCP Renewal   (T1) timeout event      */
#define TO_REBIND      5666        /* DHCP Rebinding (T2) timeout event      */
#define TO_TCP         5667        /* TCP timeout event                      */
#define TO_IGMP        5668        /* IGMP timeout event                     */
#define RESP_DMACPY    5669        /* End of DMA copy event                  */
#define RESP_DMACPY1   5670        /* End of DMA copy event                  */
#define RESP_DMACPY2   5671        /* End of DMA copy event                  */
#define BUF_RCV        5672        /* Buffer received                        */
#define BUF_SNT        5673        /* Buffer sent                            */
#define RST_TCP        5674        /* Reset the TCP connection               */


/* Automaton DNS - DNS protocol .............................................*/


                                   /* Private events range: 1024+32*146      */
#define TO_DNS         5696        /* Evt - DNS  Timer timout                */
#define TO_MDNS        5697        /* Evt - mDNS Timer timout                */


/* Automaton FTP - FTP protocol .............................................*/


                                   /* Private events range: 1024+32*147      */
#define SND_RESP_CONN  5728        /* Evt - Send RESP_CONNECT    request     */
#define SND_RESP_DISC  5729        /* Evt - Send RESP_CONNECT    request     */
#define SND_RESP_ABRT  5730        /* Evt - Send RESP_CONNECT    request     */



/* Automaton LPWAN - LoRaWAN protocol .......................................*/


                                   /* Private events range: 1024+32*148      */
#define TO_LPWAN       5760        /* Join-Accept timeout                    */
#define REQ_START      5761        /* Start an uplink frame                  */



/*
    2    Structures and typedefs
    ____________________________
*/

/*
    3    Public variables
    _____________________
*/

/*
    4    Prototypes of public procedures
    ____________________________________
*/

#endif                                 /* End of AUT_L                       */
