/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv.h"                 */
/*  Constants and definitions for drivers interface.                         */
/*                                                                           */
/*  Author : JYA                        Last modification : 04/04/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1 Driver type definition constants
            1.2 Flags masks and values
            1.3 Bits mask constants of the field "length" of REQ_OPEN

         2  Structures and typedefs
            2.1 Tag description (ROM) - Ptag
            2.2 Pointing on tags infos (ROM) - Drv_tag
            2.3 Tag to register number conversion (ROM) - Tag2reg
            2.4 Description of a VMOS system driver (ROM) - Sys_driver
            2.5 Description of a VMOS driver - Dps_driver
            2.6 Iteration structure for register list manipulation (ROM) - Rlit

         3  Public variables

         4  Prototypes of public procedures

*/


#ifndef DRV_H                          /* If this file is not already        */
#define DRV_H                          /* included, include it.              */


/*
    1    Defines
    ____________


    1.1 Driver type definition constants
    ------------------------------------

   These constants are used for the "typdrv" fields of VMOS structures
   "Sys_driver", "Dps_driver" and "Df_driver". They are also used for the
   "typdrv" field of the VMIO structure "Drv_desc".

*/

#define DRV_SYS     0                  /* Driver type - System driver        */
#define DRV_CAR     1                  /* Driver type - Character driver     */
#define PRO_CON     2                  /* Driver type - Connected Protocol   */
#define PRO_DTG     3                  /* Driver type - Datagram Protocol    */
#define SER_CON     4                  /* Driver type - Connected Service    */
#define SER_DTG     5                  /* Driver type - Datagram Service     */
#define DRV_BLK     6                  /* Driver type - Block driver         */
#define SER_VOL     7                  /* Driver type - Volume driver        */

#define BSYS        (1<<DRV_SYS)       /* Bit mask - System driver           */
#define BCAR        (1<<DRV_CAR)       /* Bit mask - Character driver        */
#define BCON        (1<<PRO_CON)       /* Bit mask - Connected Protocol      */
#define BDTG        (1<<PRO_DTG)       /* Bit mask - Datagram Protocol       */
#define BSCN        (1<<SER_CON)       /* Bit mask - Datagram Service        */
#define BSDT        (1<<SER_DTG)       /* Bit mask - Connected Service       */
#define BBLK        (1<<DRV_BLK)       /* Bit mask - Block driver            */
#define BVOL        (1<<SER_VOL)       /* Bit mask - Volume driver           */

#define BCHA        (1<<13)            /* Bit mask - Channel                 */
#define BSUB        (1<<14)            /* Bit mask - Sub-channel             */
#define BMODD       (1<<15)            /* Bit mask - Module descriptor       */

                                       /* Some usefull bit masks             */

#define BPRO ( BCON | BDTG )
#define BSER ( BSCN | BSDT )
#define BCPS ( BCAR | BPRO | BSER )
#define BDPS ( BCAR | BPRO | BSER | BBLK )
#define BTMD ( BSYS | BCAR | BPRO | BSER | BBLK | BVOL )
#define BTMA ( BTMD | BMODD )

/*
    1.2 Flags masks and values
    --------------------------
*/

#define POOL0         (INT)0x00000000  /* Alloc dans partition 0             */
#define POOL1         (INT)0x00001000  /* Alloc dans partition 1             */
#define POOL2         (INT)0x00002000  /* Alloc dans partition 2             */
#define POOL3         (INT)0x00003000  /* Alloc dans partition 3             */
                                       /*                                    */
#define NBLOCKING_CON (INT)0x00010000  /* Non-blocking Connect/disconnect 16 */
#define NBLOCKING_IO  (INT)0x00020000  /* Non-blocking Read/write         17 */
#define NBLOCKING_VAL (INT)0x00040000  /* Non-blocking Setval/getval      18 */
#define NBLOCKING_EXE (INT)0x00080000  /* Non-blocking Execute/RPC        19 */
#define NBLOCKING_MSK (INT)0x000F0000  /* OR of the 4 previous bits          */
                                       /*                                    */
#define ROUTE_IND     (INT)0x00100000  /* Demande de routage des IND_xxx  20 */
#define ROUTE_IND2    (INT)0x00200000  /* Demande de routage des IND_xxx  21 */

#define NOMOREDATA    (INT)0x00000000  /* Trame complete recue            22 */
#define MOREDATA      (INT)0x00400000  /* More data to send               22 */
#define OOB           (INT)0x00800000  /* Data is "out of band"           23 */

#define CALL          (INT)0x01000000  /* Outgoing connect context        24 */
#define LISTEN        (INT)0x02000000  /* Incoming connect context        25 */
#define BRUTAL        (INT)0x04000000  /* Brutal disconnect               26 */
#define SOURCE        (INT)0x08000000  /* "drv_start"                     27 */
                                       /*                                    */
#define ALLOC_MODE    (INT)0x10000000  /* 0:VMK allocate 1:User provided  28 */
#define  VMK_ALLOC    (INT)0x00000000  /* "drv_read" must allocate buffer    */
#define  USER_ALLOC   (INT)0x10000000  /* "drv_reqd" receives a buffer       */

#define RCV_MODE      (INT)0x20000000  /* Receive mode, PDU or socket     29 */
#define   PDU_RCV     (INT)0x00000000  /*   Remontee des donnees PDU par PDU.*/
#define   SOCKET_RCV  (INT)0x20000000  /*   Remontee des donnees par troncons*/
                                       /*                                    */
#define NET_ADR       (INT)0x40000000  /* user provided network address   30 */
#define PRIVATE       (INT)0x80000000  /* user provided private parameter 31 */


/*
    1.3 Bits mask constants of the field "length" of REQ_OPEN
    ---------------------------------------------------------
*/

#define NO_MSKR     (UCHAR)255         /* No mask register                   */


/*
    2    Structures and typedefs
    ____________________________


    2.1 Tag description (ROM) - Ptag
    --------------------------------

   The "ptag" field of the "Dps_driver" structure points to an array of "Ptag"
   structures. This table contains a structure of type "Ptag" by TAG. Each
   "Ptag" structure describes the properties of a single tag. This structure
   does NOT contain the name of the tag. The "Dps_driver" structure contains
   a field named "tag", which points to an array of string addresses, these
   strings are the names of the tags. In the same way, when a tag is an ENUM,
   the list of possible values is NOT in the "Ptag" structure, but is pointed
   by the "val" field of the "Dps_driver" structure.      

*/

typedef struct                         /* One tag properties                 */
  BEGIN                                /* ---------------------------------- */
    UCHAR       type                 ; /* Tag type                           */
    UINT        acces                ; /* Rag accessibility                  */
    USHORT      valr                 ; /* First register index of this tag   */
    USHORT      nbr                  ; /* Number of registers for this tag   */
    UCHAR       shift                ; /* Left shift to apply                */
    UINT        mask                 ; /* Mask to apply on rescue value      */
    UINT        sessid_mod           ; /* New sessid notification            */
  END           Ptag                 ; /* ---------------------------------- */


/*
    2.2 Pointing on tags infos (ROM) - Drv_tag
    ------------------------------------------
*/

typedef struct                         /* Pointers on tags infos             */
  BEGIN                                /* ---------------------------------- */
    INT         ntag                 ; /* Number of tags                     */
    const CHAR *const *tag           ; /* Tags names addresses table         */
    const CHAR *const *val           ; /* ENUM tags value table              */
    const Ptag *ptag                 ; /* Tags properties table              */
  END           Drv_tag              ; /* ---------------------------------- */


/*
    2.3 Tag to register number conversion (ROM) - Tag2reg
    -----------------------------------------------------
*/

typedef struct                         /* Tag value -> register value        */
  BEGIN                                /* ---------------------------------- */
    UINT        reg                  ; /* Indice of first register           */
    UINT        msk                  ; /* Mask registier value for this tag .*/
  END           Tag2reg              ; /* ---------------------------------- */


/*
    2.4 Description of a VMOS system driver (ROM) - Sys_driver
    -----------------------------------------------------------

   A VMOS has an array named "vmos_driver[]", which allows it to find its
   drivers, protocol stacks and services. The "vmos_driver" array is an array
   of addresses of "Sys_driver", "Dps_driver" and "Df_driver" structures. Each
   C module "system driver" contains a structure "Sys_driver" whose address
   will be one of the elements of "vmos_driver". The index in "vmos_driver[]"
   is called "major number"

*/

typedef struct  sys_driver             /* System driver                      */
  BEGIN                                /* ---------------------------------- */
    CHAR        name[STDNAME]        ; /* System driver name                 */
    CHAR        typdrv               ; /* Always equel to zero               */
    IWORD       (*drv_evt   )()      ; /* Adress of procedure xxx_yyy_evt    */
  END           Sys_driver           ; /* ---------------------------------- */


/*
    2.5 Description of a VMOS driver - Dps_driver
    ----------------------------------------------

   Each C module of type driver, protocol stack or service contains the
   definition of a structure "Dps_driver", whose address will be one of the
   elements of "vmos_driver". The index in "vmos_driver[]" is called the
   "major number".    

*/

typedef struct  dps_driver             /* One VMOS driver/protocole/service  */
  BEGIN                                /* ---------------------------------- */
    CHAR        name[STDNAME]        ; /* driver/protocole/service name      */
    CHAR        typdrv               ; /* 001:dcar 010:proto 011:ser 100:dblk*/

                                       /* Interface procedures ------------- */
    IWORD      (*drv_evt         )() ; /* xyz_evt          procedure address */
    IWORD      (*drv_info        )() ; /* xyz_info         procedure address */
    IWORD      (*drv_init        )() ; /* xyz_init         procedure address */
    IWORD      (*drv_end         )() ; /* xyz_end          procedure address */
    IWORD      (*drv_alloc       )() ; /* xyz_alloc        procedure address */
    IWORD      (*drv_free        )() ; /* xyz_free         procedure address */
    IWORD      (*drv_open_device )() ; /* xyz_open_device  procedure address */
    IWORD      (*drv_close_device)() ; /* xyz_close_device procedure address */
    IWORD      (*drv_alloc_sub   )() ; /* xyz_alloc_sub    procedure address */
    IWORD      (*drv_free_sub    )() ; /* xyz_free_sub     procedure address */
    IWORD      (*drv_connect     )() ; /* xyz_connect      procedure address */
    IWORD      (*drv_listen      )() ; /* xyz_listen       procedure address */
    IWORD      (*drv_disconnect  )() ; /* xyz_disconnect   procedure address */
    IWORD      (*drv_abort       )() ; /* xyz_abort        procedure address */
    IWORD      (*drv_get_bufsize )() ; /* xyz_get_bufsize  procedure address */
    IWORD      (*drv_write       )() ; /* xyz_write        procedure address */
    IWORD      (*drv_read        )() ; /* xyz_read         procedure address */
    IWORD      (*drv_signal      )() ; /* xyz_signal       procedure address */
    IWORD      (*drv_put         )() ; /* xyz_put          procedure address */
    IWORD      (*drv_get         )() ; /* xyz_get          procedure address */
    IWORD      (*drv_setval      )() ; /* xyz_setval       procedure address */
    IWORD      (*drv_getval      )() ; /* xyz_getval       procedure address */
    IWORD      (*drv_rpc         )() ; /* xyz_rpc          procedure address */

                                       /* Tags ----------------------------- */
    INT         ntag                 ; /* Count of channel tags              */
    INT         ntag_sub             ; /* Count of sub-channels tags         */
    const CHAR *const *tag           ; /* Tag names addresses array address  */
    const CHAR *const *val           ; /* ENUM tags values array address     */
    const Ptag *ptag                 ; /* Tags properties array address      */

                                       /* Device number -------------------- */
    const UINT  nreg_idev            ; /* Register number                    */
    const UINT  sreg_idev            ; /* Right shift and then               */
    const UINT  mreg_idev            ; /* mask value                         */

                                       /* Flags ---------------------------- */
    const UCHAR sc_size              ; /* Sub-channels number of bits        */
    UINT        alloc_flags_0        ; /* Allocation flags that must be 0    */
    UINT        alloc_flags_1        ; /* Allocation flags that must be 1    */
  END           Dps_driver           ; /* ---------------------------------- */


/*
    2.6 Iteration structure for register list manipulation (ROM) - Rlit
    -------------------------------------------------------------------

    Refers to drv_req1.c section 2.13.
*/

typedef struct _s_rl_it                /* Iteration for register list        */
  BEGIN                                /* ---------------------------------- */
    INT         nr                   ; /* Register value                     */
    INT         nbb                  ; /* Number of blocks                   */
    INT         fmt                  ; /* Data format                        */
    INT         nbr                  ; /* Number of register in the block    */
    UCHAR      *lr                   ; /* Current block pointer              */
    UCHAR      *ls                   ; /* Next block pointer                 */
    UCHAR       o[4]                 ; /* Endian 0123: Big, 3210: Little     */
    UCHAR      *p                    ; /* Register pointer in the list       */
  END           Rlit                 ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

/*
    4    Prototypes of public procedures
    ____________________________________
*/


IWORD drv_init            (void                                              );
IWORD drv_list_modules    (CHAR*,INT,INT*,INT                                );
IWORD drv_list_names      (CHAR*,CHAR*,INT*,INT                              );
IWORD drv_list_tags       (CHAR*,CHAR*,INT*                                  );

IWORD drv_list_enum       (CHAR*,CHAR*,CHAR*, INT*                           );
IWORD drv_tag_name        (CHAR*,INT,CHAR*                                   );
IWORD drv_tag_number      (CHAR*,CHAR*,INT*,INT*                             );
IWORD drv_enum_name       (CHAR*,CHAR*,INT,CHAR*                             );
IWORD drv_enum_number     (CHAR*,CHAR*,CHAR*,INT*                            );

IWORD drv_init_driver     (CHAR*,INT,INT,INT*                                );
IWORD drv_init_protocol   (CHAR*,INT,INT,INT*                                );
IWORD drv_init_service    (CHAR*,INT,INT,INT*                                );
IWORD drv_end             (INT,INT                                           );

IWORD drv_alloc_device    (INT,CHAR*,CHAR*,CHAR*,INT,INT,INT,INT,INT*        );
IWORD drv_alloc_protocol  (INT,CHAR*,CHAR*,INT,INT,INT,INT,INT*              );
IWORD drv_alloc_service   (INT,CHAR*,CHAR*,INT,INT,INT,INT,INT*              );
IWORD drv_free            (INT,INT                                           );

IWORD drv_open_device     (INT,CHAR*,INT                                     );
IWORD drv_close_device    (INT,INT                                           );
IWORD drv_alloc_subchan   (INT,CHAR*,CHAR*,INT,INT,INT,INT,INT*              );
IWORD drv_free_subchan    (INT,INT                                           );

IWORD drv_connect         (INT,CHAR*,INT,INT                                 );
IWORD drv_listen          (INT,CHAR*,INT,INT                                 );
IWORD drv_disconnect      (INT,INT,INT                                       );
IWORD drv_abort           (INT,INT                                           );

IWORD drv_alloc_subctx    (INT,CHAR*,CHAR*,INT,INT,INT,INT,INT*              );
IWORD drv_free_subctx     (INT,INT                                           );

IWORD drv_get_bufsize     (INT,INT,INT,INT*,INT*,INT*,INT*                   );
IWORD drv_write           (INT,UCHAR*,INT,INT,INT,INT                        );
IWORD drv_read            (INT,INT,INT,UCHAR**,INT*,INT*,INT*,INT            );
IWORD drv_signal          (INT,INT,INT,INT                                   );
IWORD drv_put             (INT,UCHAR*,UINT,UINT,INT,INT,INT                  );
IWORD drv_get             (INT,UINT,UINT,INT,INT,UCHAR**,INT                 );
IWORD drv_setval          (INT,CHAR*,INT                                     );
IWORD drv_getval          (INT,CHAR*,INT,INT                                 );
IWORD drv_execute         (INT,CHAR*,INT                                     );

IWORD drv_rpc             (INT,CHAR*,CHAR*,CHAR*,INT,INT                     );
IWORD drv_name2major      (CHAR*,INT*,INT*                                   );
IWORD drv_major2name      (INT,CHAR*,INT,INT*                                );
IWORD drv_name2minor      (CHAR*,INT*,INT*,INT*                              );
IWORD drv_chk_iod         (UINT                                              );
IWORD drv_chk_ioddev      (UINT,INT*,INT*,INT*                               );
IWORD drv_set_state       (S_evt*                                            );

IWORD tag2reg             (const Dps_driver*                                  ,
                           INT,INT,CHAR*[4],USHORT[4],UINT*,INT,UINT*,INT*   );
IWORD tag2cmd             (const Dps_driver*,INT,INT,CHAR*,UINT*,UINT*       );
IWORD reg2tag             (const Dps_driver*,INT,UINT*,UINT,CHAR*,INT,INT*   );

IWORD unschedule          (INT,INT,IWORD (*)(),INT,S_evt**                   );

#endif                                 /* End of DRV_H                       */
