/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_i2c.h"             */
/*  Constants and definitions for I2C driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 08/07/2024       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Register numbers
            1.2  Values of the "mode" parameter for "resetI2C_xyz" 
            1.3  Write flags bit values 
            1.4  Read flags bit values 
            1.5  I2C buffer header defines
            1.6  I2C protocol code values 
            1.7  Bits b0..3 of field "status" of "I2c_node" 
            1.8  Bits b4..7 of field "status" of "I2c_node"
            1.9  Values of field "type" of "I2c_node"
            1.10 I2C event protocol command codes

         2  Structures and typedefs
            2.1  I2C low-level - I2c_desc/I2c_subdef/I2c_drvdef
            2.2  I2C software low-level using GPIO's parameters
            2.3  I2C bus context - I2c_ctx/I2c_slactx
            2.4  "i2c_io" parameter - I2c_io
            2.5  I2C tunel - I2c_node

         3  Public variables
         4  Prototypes of public procedures
         5  RAM data to be included in board_xyz.c


*/


#ifndef DRV_I2C_H                      /* If this file is not already        */
#define DRV_I2C_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/

/*  1.1  Register numbers ...................................................*/

#define NR_I2C_TYPE          0         /* string  TYPE                     1 */
#define NR_I2C_NAME          1         /* string  NAME                     3 */
#define NR_I2C_NUMDEV        4         /* integer NUM                      1 */

#define NR_I2C_MODE          5         /* integer MODE                     1 */
#define NR_I2C_SPEED         6         /* integer SPEED                    1 */
#define NR_I2C_WAIT          7         /* integer WAIT                     1 */

#define NR_I2C_SLNAME        8         /* string  SLNAME                   3 */
#define NR_I2C_SLNUMDEV      11        /* integer SLNUM                    1 */
#define NR_I2C_SLPROTO       12        /* integer PROTO                    1 */
#define NR_I2C_SLADDR        13        /* integer SLADDR                   1 */
#define NR_I2C_SLSUBAD       14        /* integer SLSUBAD                  1 */
#define NR_I2C_SLFLAGS       15        /* integer SLFLAGS                  1 */
#define NR_I2C_EEPSUBSZ      16        /* integer EEPSUDSZ                 1 */
#define NR_I2C_EEPROWSZ      17        /* integer EEPROWSZ                 1 */

/* 1.2  Values of the "mode" parameter for "resetI2C_xyz" ...................*/

#define MODE_OFF             0         /* Controller is disabled             */
#define MODE_MASTER          1         /* Controller is the bus master       */
#define MODE_SLAVE           2         /* Controller is a slave              */
#define MODE_TUNMASTER       3         /* Controller is a tunnel master      */
#define MODE_TUNSLAVE        4         /* Controller is a tunnel slave       */

/* 1.3  Write flags bit values ..............................................

   Flags FLG_START/FLG_STOP/FLG_NO_ACK are also ise with the "putI2C_xyz"
   low-level "flags" parameter.
*/

#define FLG_START       (1<<0)         /* Generate "start"                   */
#define FLG_STOP        (1<<1)         /* Generate "stop"                    */
#define FLG_NO_ACK      (1<<2)         /* No acknowledgement expected        */
#define FLG_NO_WAIT     (1<<3)         /* Suppress wait between bytes 1 & 2  */
#define FLG_XSUBAD      (1<<4)         /* Sub-address is 2 bytes long        */
                                       /*                                    */
#define FLG_MULTI       (1<<5)         /* Sent data is multi-block format    */
#define FLG_STOPLAST    (1<<6)         /* Send a STOP after the last block   */
#define FLG_XBLEN       (1<<7)         /* Block lengthes are 2 bytes long    */

/* 1.4  Read flags bit values ...............................................
*/

#define FLG_NOSEL       (1<<5)         /* Do not select before read          */

/* 1.5  I2C buffer header defines ...........................................*/

#define HDR_I2C_SZ           8         /* Buffer header size in bytes        */
#define HDR_I2C_PRO          0         /* I2C protocol                   1   */
#define HDR_I2C_ADDR         1         /* I2C address offset             1   */
#define HDR_I2C_SUBAD        2         /* I2C sub-address offset         2   */
#define HDR_I2C_FLAGS        4         /* I2C write/read flags offset    1   */
#define HDR_I2C_EEPSUBSZ     5         /* EEPROM subad size in bits      1   */
#define HDR_I2C_EEPROWSZ     6         /* EEPROM row/page size in bytes  1   */
                                       /* Unused                         1   */

/* 1.6  I2C protocol code values ............................................*/

#define PRO_I2C_DEV          0
#define PRO_I2C_EEP          1
#define PRO_I2C_EVT          2
#define PRO_I2C_MAX          2

/* 1.7  Bits b0..3 of field "status" of "I2c_node" ...........................
   Those bits are the slave send status
*/

#define I2C_SNONE            0         /* No event to be received            */
#define I2C_SSTART           1         /* One event to be received           */
#define I2C_SWJOB            2         /* Waiting for a Job structure        */
#define I2C_SWSMALL          3         /* Waiting for a small buffer         */
#define I2C_SWTBUF           4         /* Waiting for RESP_ALLOC_BUF         */
#define I2C_SWAIT            5         /* Waiting for start sending          */
#define I2C_SND              6         /* Sending event via I2C is ongoing   */
#define I2C_SUNKWN          15         /* Polling failed, value unknown      */

/* 1.8  Bits b4..7 of field "status" of "I2c_node .............................
   Those bist are the slave receive status
*/
#define I2C_RNONE         0x00         /* Not ready to receive anything      */
#define I2C_REVT          0x10         /* Ready to receive a single event    */
#define I2C_REBUF         0x20         /* Ready to receive event + buffer    */
#define I2C_RCV           0x30         /* Receiving event via I2C is ongoing */
#define I2C_RUNKWN        0xF0         /* Polling failed, value unknown      */

/* 1.9  Values of field "type" of "I2c_node" ..................................
*/
#define TYPE_FREE            0         /* Entry is unused                    */
#define TYPE_SLAVE           1         /* Entry describes a bus slave        */
#define TYPE_MASTER          2         /* Entry describes a bus master       */

/* 1.10  I2C event protocol command codes .....................................
*/
#define CMD_POLL             0         /* Poll slave  - TR_POLL              */
#define CMD_EVT_WRITE        1         /* Event write - TR_WREVT             */
#define CMD_EVT_READ         2         /* Event read  - TR_RDEVT             */

/*
   2  Structures and typedefs
   __________________________

   2.1  I2C low-level - I2c_desc/I2c_subdef/I2c_drvdef
   ---------------------------------------------------
*/

typedef struct                         /* Low-level driver procedures addr   */
  BEGIN                                /* ---------------------------------- */
    void            (*reset)   ()    ; /* Initialization                     */
    void            (*put)     ()    ; /* Master - Write to the bus          */
    void            (*get)     ()    ; /* Master - Read from the bus         */
    void            (*waitadr) ()    ; /* Slave - wait for an I2C addr byte  */
    void            (*write)   ()    ; /* Slave - Write to the bus           */
    void            (*read)    ()    ; /* Slave - Read from the bus          */
  END               I2c_desc         ; /* ---------------------------------- */

typedef struct                         /* Generic I2C slave device parameters*/
  BEGIN                                /* ---------------------------------- */
    CHAR            *name            ; /* Slave name                         */
    UCHAR           sladdr           ; /* Slave I2C address                  */
    UCHAR           proto            ; /* Slave protocol                     */
    UCHAR           eepsubsz         ; /* EEPROM subad size in bits          */
    UCHAR           eeprowsz         ; /* EEPROM row/page size in bytes      */
    UCHAR           node             ; /* TUNNEL node number                 */
  END               I2c_subdef       ; /* ---------------------------------- */

typedef struct                         /* Generic I2C driver parameters      */
  BEGIN                                /* ---------------------------------- */
    CHAR           *name             ; /* Device name                        */
    UCHAR           lctx             ; /* Low-level driver context number    */
    UCHAR           nbslave          ; /* Count of slave devices             */
    UCHAR           mode             ; /* OFF MASTER SLAVE TUNMASTER TUNSLAVE*/
    UINT            speed            ; /* I2C bus speed in Hz                */
    INT             wait             ; /* Wait duration between 2 requests   */
    INT             tpoll            ; /* Tunnel polling period in msec      */
    I2c_subdef const *tsubdef        ; /* List of slave parameters           */
  END               I2c_drvdef       ; /* ---------------------------------- */


/*
   2.2  I2C software low-level using GPIO's parameters
   ----------------------------------------------------
*/
typedef struct                         /* I2C software low-level parameters  */
  BEGIN                                /* ---------------------------------- */
    UCHAR           pio_port         ; /* Numero du port PIO des fils I2C    */
    UCHAR           sda_pio          ; /* Numero du PIO du fil SDA           */
    UCHAR           scl_pio          ; /* Numero du PIO du fil SCL           */
    USHORT          hold             ; /* Temps de maintien en us            */
    USHORT          setup            ; /* Temps de chargement en us          */
  END               I2c_softdef      ; /* ---------------------------------- */


/*
   2.3  I2C bus context - I2c_ctx
   ------------------------------
*/

typedef struct                         /* I2C slave context                  */
  BEGIN                                /* ---------------------------------- */
    Req_subctx      rsctx            ; /* Req_lib.h header structure         */
    const I2c_subdef *subdef         ; /* I2C slave device parameters        */
    UCHAR           ictx             ; /* Index in "i2c_ctx[]"               */
    UCHAR           proto            ; /* Slave protocol                     */
    UCHAR           sladdr           ; /* Slave address                      */
    UCHAR           flags            ; /* I2C specific write/read flags      */
    USHORT          subad            ; /* Device/EEPROM sub-address          */
    UCHAR           eepsubsz         ; /* EEPROM subad size in bits          */
    UCHAR           eeprowsz         ; /* EEPROM row/page size in bytes      */
  END               I2c_slactx       ; /* ---------------------------------- */

typedef struct                         /* Control loop structure             */
  BEGIN                                /* ---------------------------------- */
    USHORT          cnt              ; /* Loop counter                       */
    UCHAR           begloop          ; /* Start of loop index                */
  END               I2c_loop         ; /* ---------------------------------- */

#define MLOOP      3
#define MSG_SZ     130

typedef struct                         /* I2C context                        */
  BEGIN                                /* ---------------------------------- */
    Req_ctx         rctx             ; /* The "Job" queues                   */
    Key_lnk         key              ; /* A single key by device             */
    const I2c_drvdef *drvdef         ; /* Generic controller parameters addr */
    I2c_slactx      *tslactx         ; /* First slave sub-context address    */
    USHORT          subflw           ; /* First slave Logical way number     */
                                       /*                                    */
    void            (*reset)    ()   ; /* Controller reset    procedure      */
    void            (*put)      ()   ; /* Master - Send one or N bytes       */
    void            (*get)      ()   ; /* Master - Bus bytes read            */
    void            (*waitadr)  ()   ; /* Slave - Wait for I2C addr byte     */
    void            (*write)    ()   ; /* Slave - Send one or N bytes        */
    void            (*read)     ()   ; /* Slave - Bus bytes read             */
                                       /*                                    */
    UCHAR           init_done        ; /* TRUE if initialized once           */
    UCHAR           sreset           ; /* RESET steps counter                */
    UCHAR           mode             ; /* mode, 0 if disabled                */
    UINT            speed            ; /* I2C bus speed in KHz               */
    INT             wait             ; /* Wait duration between 2 requests   */
                                       /*                                    */
    Job             *job             ; /* Current job address                */
    UCHAR           *frame           ; /* Data buffer address (read/write)   */
    I2c_slactx      *slactx          ; /* Current sub-channel context addr   */
    INT             lg               ; /* Count of bytes to be read          */
    UCHAR           trcode           ; /* Transaction code                   */
    UCHAR           ad               ; /* I2C address or -1                  */
    INT             subad            ; /* Register selection sub-address     */
    UCHAR           flags            ; /* Write flags                        */
    UCHAR           maxtry           ; /* Maximum tries count                */
    UCHAR           eepsubsz         ; /* EEPROM subad size in bits          */
    UCHAR           rowsize          ; /* EEPROM row/page size in bytes      */
    UCHAR           wasize           ; /* EEPROM Word Address size in byte   */
                                       /*                                    */
    UCHAR           *fracur          ; /* Read/write pointer                 */
    UCHAR           try              ; /* Tries counter                      */
    UCHAR           tryadr           ; /* Tries counter for the I2C address  */
    UCHAR           pollrnk          ; /* Current polled slave rank          */
    UCHAR           nodeidx          ; /* Index in "i2c_node[]" of master    */
    UINT            idto             ; /* Current running TO_COM identifier  */
    USHORT          et               ; /* "ti2c" automaton state             */
    UCHAR           ev               ; /* "ti2c" automaton event             */
                                       /*                                    */
    I2c_loop        loop[MLOOP]      ; /* Stack for intricated loops         */
    UCHAR           ploop            ; /* Stack pointer for "loop[]"         */
    UCHAR           ioidx            ; /* Index in the I2c_io array/list     */
                                       /*                                    */
    UCHAR           msg[MSG_SZ]      ; /* Register selection message + row   */
  END               I2c_ctx          ; /* ---------------------------------- */

/*
   2.4  "i2c_io" parameter - I2c_io
   --------------------------------
*/

typedef struct                         /* "i2c_io" list item                 */
  BEGIN                                /* ---------------------------------- */
    UCHAR          io                ; /* I2C address or 0xFF (wait)         */
    UCHAR          flags             ; /* Write flags                        */
    UCHAR          *adr              ; /* Transmit or receive address        */
    USHORT         lg                ; /* Count of bytes to read or write    */
  END              I2c_io            ; /* ---------------------------------- */

/*
   2.5  I2C tunel - I2c_node
   -------------------------
*/
typedef struct                         /* I2C node parameters                */
  BEGIN                                /* ---------------------------------- */
    UCHAR          type              ; /* FREE/MASTER/SLAVE                  */
    UCHAR          bus               ; /* I2c bus/context index              */
    UCHAR          i2caddr           ; /* I2C address                        */
    UCHAR          status            ; /* Local and remote node status       */
    UCHAR          sta1              ; /* Master ready to send status        */
    UCHAR          sta2              ; /* Slave ready to receive status      */
    UCHAR          sta3              ; /* Slave ready to send status         */
    USHORT         length1           ; /* Master count of attached bytes     */
    USHORT         length2           ; /* Slave count ready to receive bytes */
    USHORT         length3           ; /* Slave  count of attached bytes     */
    USHORT         failed            ; /* Count of failed pollings           */
    Job            *o_rcv            ; /* O_RCV job address                  */
  END              I2c_node          ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________
*/

extern UCHAR           i2c_numdrv    ; /* Number of the system driver I2C    */

/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD i2c_set             (INT,INT                                         );
  IWORD i2c_send            (INT,INT,INT,INT,UCHAR*,INT,INT,INT              );
  IWORD i2c_receive         (INT,INT,INT,INT,UCHAR*,INT,INT,INT              );
  IWORD i2c_io              (INT,I2c_io*,INT,INT                             );
  IWORD i2c_write           (INT,INT,INT,INT,UCHAR*,INT,INT                  );
  IWORD i2c_read            (INT,INT,INT,INT,UCHAR*,INT,INT                  );
  IWORD putevt_i2c          (S_evt*,INT                                      );
  void  bufevt_i2c          (UCHAR*                                          );
  void  encode_evt          (S_evt*,INT,UCHAR*,INT*                          );
  void  decode_evt          (UCHAR*,S_evt*,INT*,INT*                         );

  void  i2c_evtlow          (INT,INT,UCHAR*,INT,INT                          );

#endif                                 /* End of DRV_I2C_H.                  */

/*
    5  RAM data to be included in board_xyz.c
    _________________________________________
*/

#ifdef BOARD_XYZ_DATA_C

#define I2C_NBLW (1+I2C_NBCTX+I2C_NBSCTX)
#define I2C_SBSIZE sizeof(S_evt)
#define I2C_SSZ (I2C_SBSIZE*I2C_SBCNT)
#define I2C_BSZ (I2C_BBSIZE*I2C_BBCNT)

const UCHAR  i2c_ctxmax = I2C_NBCTX  ; /* Count of I2C controllers contexts  */
const UCHAR  i2c_sctxmax= I2C_NBSCTX ; /* Count of slave devices contexts    */
const USHORT i2c_nblw   = I2C_NBLW   ; /* Count of logical ways              */
const USHORT i2c_nbiod  = I2C_NBCTX  ; /* Size of sessid/con2lw              */
const USHORT i2c_nbnode = I2C_NBNODE ; /* Size of "i2c_node[]"               */

I2c_ctx      i2c_ctx   [I2C_NBCTX]   ; /* Controller context table           */
I2c_slactx   i2c_slactx[I2C_NBSCTX]  ; /* Slave devices context table        */
USHORT       i2c_state [I2C_NBLW]    ; /* Logical Ways states                */
I2c_node     i2c_node  [I2C_NBNODE]  ; /* Tunel nodes                        */

UCHAR        i2c_spool[I2C_SSZ]      ;
UINT         i2c_sallbtm[(I2C_SBCNT+31)/32] = { 0 }   ;
const USHORT i2c_sbsize = I2C_SBSIZE ;
const USHORT i2c_sbcnt  = I2C_SBCNT  ;

#endif                                 /* End of BOARD_XYZ_C                 */

