/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_rad.h"             */
/*  Constants and definitions for RAD driver.                                */
/*                                                                           */
/*  Author : TE                         Last modification : 07/25/2024       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Modem Radio Type
            1.2  Radio Operating Mode
            1.3  Radio Timeout event
            1.4  Radio Signal
            1.5  LoRa Parameters
            1.6  FSK  Parameters
            1.7  MSK  Parameters
            1.8  BPSK Parameters
            1.9  Register numbers
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures
         5  RAM data to be inclueded in board_xyz.c


*/


#ifndef DRV_RAD_H                      /* If this file is not already        */
#define DRV_RAD_H                      /* included, include it.              */


/*
    1    Defines
    ____________

    1.1  Modem Radio Type
    ---------------------
*/

#define MODEM_FSK              0       /* FSK  modem type                    */
#define MODEM_LORA             1       /* LoRa modem type                    */
#define MODEM_BPSK             2       /* BPSL modem type                    */
#define MODEM_MSK              3       /* MSK  modem type                    */
#define MODEM_NONE             4       /* NONE modem type                    */


/*
    1.2  Radio Operating Mode
    -------------------------
*/

#define MODE_SLEEP             0       /* The radio is in Sleep Mode         */
#define MODE_STDBY             1       /* The radio is in Standby Mode       */
#define MODE_RX                2       /* The radio is in Receive Mode       */
#define MODE_TX                3       /* The radio is in Transmit Mode      */
#define MODE_FS                4       /* The radio is in freq. synthesis Mod*/
#define MODE_CAD               5       /* The radio is in CAD Mode           */
#define MODE_RX_DC             6       /* The radio is in RX Duty-Cycle Mode */

#define STDBY_RC               0       /* The radio is in Standby Mode       */
#define STDBY_XOSC             1       /* The radio is in Standby Mode       */


/*
    1.3  Radio Timeout error
    ------------------------
*/

#define RX_TX_TIMEOUT         -1       /* Tx/Rx timeout error code           */
#define HEADER_ERR            -2       /* HeaderErr     error code           */
#define CRC_ERR               -3       /* CRCErr        error code           */


/*
    1.4  Radio Signals
    ------------------
*/

#define CAD_SIG                0       /* Channel Activity Detection signal  */
#define HEADER_SIG             1       /* Preamble/synch/header detected     */


/*
    1.5  LoRa Parameters
    --------------------
*/

#define LORA_BW_500            6       /* Modulation Bandwidth = 500   kHz   */
#define LORA_BW_250            5       /* Modulation Bandwidth = 250   kHz   */
#define LORA_BW_125            4       /* Modulation Bandwidth = 125   kHz   */
#define LORA_BW_62             3       /* Modulation Bandwidth = 62.5  kHz   */
#define LORA_BW_41            10       /* Modulation Bandwidth = 41.67 kHz   */
#define LORA_BW_31             2       /* Modulation Bandwidth = 31.25 kHz   */
#define LORA_BW_20             9       /* Modulation Bandwidth = 20.83 kHz   */
#define LORA_BW_15             1       /* Modulation Bandwidth = 15.63 kHz   */
#define LORA_BW_10             8       /* Modulation Bandwidth = 10.42 kHz   */
#define LORA_BW_7              0       /* Modulation Bandwidth =  7.81 kHz   */

#define LORA_SF5            0x05       /* Spreading Factor =  5              */
#define LORA_SF6            0x06       /* Spreading Factor =  6              */
#define LORA_SF7            0x07       /* Spreading Factor =  7 (default)    */
#define LORA_SF8            0x08       /* Spreading Factor =  8              */
#define LORA_SF9            0x09       /* Spreading Factor =  9              */
#define LORA_SF10           0x0A       /* Spreading Factor = 10              */
#define LORA_SF11           0x0B       /* Spreading Factor = 11              */
#define LORA_SF12           0x0C       /* Spreading Factor = 12              */

#define LORA_CR_4_4         0x00       /* Coding Rate = 4/4                  */
#define LORA_CR_4_5         0x01       /* Coding Rate = 4/5                  */
#define LORA_CR_4_6         0x02       /* Coding Rate = 4/6 (default)        */
#define LORA_CR_4_7         0x03       /* Coding Rate = 4/7                  */
#define LORA_CR_4_8         0x04       /* Coding Rate = 4/8                  */

#define LORA_CAD_ONLY       0x00       /*                                    */
#define LORA_CAD_RX         0x01       /*                                    */
#define LORA_CAD_LBT        0x10       /*                                    */

#define LORA_CAD_01_SYMBOL  0x00       /*                                    */
#define LORA_CAD_02_SYMBOL  0x01       /*                                    */
#define LORA_CAD_04_SYMBOL  0x02       /*                                    */
#define LORA_CAD_08_SYMBOL  0x03       /*                                    */
#define LORA_CAD_16_SYMBOL  0x04       /*                                    */

#define LORA_IQ_NORMAL      0x00       /* IQ mode for LoRa packet type       */
#define LORA_IQ_INVERTED    0x01       /* IQ mode for LoRa packet type       */

#define LORA_DR0               0       /* Data Rate 0                        */
#define LORA_DR1               1       /* Data Rate 1                        */
#define LORA_DR2               2       /* Data Rate 2                        */
#define LORA_DR3               3       /* Data Rate 3                        */
#define LORA_DR4               4       /* Data Rate 4                        */
#define LORA_DR5               5       /* Data Rate 5                        */
#define LORA_DR6               6       /* Data Rate 6                        */
#define LORA_DR7               7       /* Data Rate 7                        */
#define LORA_DR8               8       /* Data Rate 8                        */
#define LORA_DR9               9       /* Data Rate 9                        */
#define LORA_DR10             10       /* Data Rate 10                       */
#define LORA_DR11             11       /* Data Rate 11                       */


/*
    1.6  FSK  Parameters
    --------------------
*/


/*
    1.7  MSK  Parameters
    --------------------
*/


/*
    1.8  BPSK Parameters
    --------------------
*/


/*
    1.9  Register numbers
    ---------------------
*/

                                       /* Commons Registers ---------------- */
#define NR_RAD_TYPE               0    /* string  TYPE                     1 */
#define NR_RAD_NAME               1    /* string  NAME                     2 */
#define NR_RAD_NUMDEV             3    /* integer NUM                      1 */
#define NR_RAD_MODEM              4    /* integer ENUM                     1 */
#define NR_RAD_MODE               5    /* integer ENUM                     1 */
#define NR_RAD_CMD                6    /* integer ENUM                     1 */
#define NR_RAD_SIG                7    /* integer CADSTATE, PBLDETEC, ...  1 */

                                       /* Modulation Registers ------------- */
#define NR_RAD_BR                 8    /* integer NUM                      1 */
#define NR_RAD_FDEV               9    /* integer NUM                      1 */
#define NR_RAD_SF                10    /* integer NUM                      1 */
#define NR_RAD_BW                11    /* integer NUM                      1 */
#define NR_RAD_CR                12    /* integer NUM                      1 */
#define NR_RAD_PULSHAP           13    /* integer NUM                      1 */

                                       /* Packet Registers ----------------- */
#define NR_RAD_PBDLEN            14    /* integer NUM                      1 */
#define NR_RAD_PBLEN             15    /* integer NUM                      1 */
#define NR_RAD_HDTYP             16    /* integer ENUM                     1 */
#define NR_RAD_PLDLEN            17    /* integer NUM                      1 */
#define NR_RAD_INVIQ             18    /* integer ENUM                     1 */
#define NR_RAD_SYNWLG            19    /* integer NUM                      1 */
#define NR_RAD_RADCMP            20    /* integer NUM                      1 */
#define NR_RAD_PKTTYP            21    /* integer ENUM                     1 */
#define NR_RAD_CRCEN             22    /* integer ENUM                     1 */
#define NR_RAD_WHITEN            23    /* integer ENUM                     1 */

                                       /* Radio Registers ------------------ */
#define NR_RAD_FREQ              24    /* integer NUM                      1 */
#define NR_RAD_RXTIMSTOP         25    /* integer NUM                      1 */
#define NR_RAD_SLPTIM            26    /* integer NUM                      1 */
#define NR_RAD_POWER             27    /* integer NUM                      1 */
#define NR_RAD_FBMOD             28    /* integer NUM                      1 */
#define NR_RAD_RXTIMEOUT         29    /* integer NUM                      1 */
#define NR_RAD_TXTIMEOUT         30    /* integer NUM                      1 */

                                       /* LoRa Specific Registers ---------- */
#define NR_RAD_CADNSYM           31    /* integer NUM                      1 */
#define NR_RAD_CADDPK            32    /* integer NUM                      1 */
#define NR_RAD_CADTO             33    /* integer NUM                      1 */
#define NR_RAD_CADDMIN           34    /* integer NUM                      1 */
#define NR_RAD_CADEXMD           35    /* integer NUM                      1 */
#define NR_RAD_SYMBNUM           36    /* integer NUM                      1 */
#define NR_RAD_LORANET           37    /* integer ENUM                     1 */

#define RAD_NBREG                38    /* Number of registers (UINT)         */


/*
    2    Structures and typedefs
    ____________________________
*/

/* Low-level driver interface description ...................................*/

typedef struct                         /* Hardware dependant procedures      */
  BEGIN                                /* ---------------------------------- */
    void         (*info)         ()  ; /* Get the Radio hardware name        */
    IWORD        (*open)         ()  ; /* Open one Radio modem               */
    void         (*close)        ()  ; /* Close one Radio modem              */
    void         (*setmode)      ()  ; /* Set the radio operating mode       */
    IWORD        (*setmodul)     ()  ; /* Set Radio modulation parameters    */
    IWORD        (*setpkt)       ()  ; /* Set Radio packet parameters        */
    IWORD        (*setrxcfg)     ()  ; /* Set Radio RX parameters            */
    IWORD        (*settxcfg)     ()  ; /* Set Radio TX parameters            */
    IWORD        (*setfreq)      ()  ; /* Set Radio frequency                */
    IWORD        (*setcad)       ()  ; /* Set Radio channel activity detect. */
    IWORD        (*send)         ()  ; /* Write data buffer inside payload   */
    IWORD        (*receive)      ()  ; /* Read data buffer inside payload    */
  END            Rad_desc            ; /* ---------------------------------- */


typedef struct                         /* RAD Device Definition              */
  BEGIN                                /* ---------------------------------- */
    UCHAR        lctx                ; /* Low level device/context index     */
  END            Rad_drvdef          ; /* ---------------------------------- */


typedef struct                         /* RAD Device context                 */
  BEGIN                                /* ---------------------------------- */
    Hd_ctx           hctx            ; /* Context header                     */
    Req_ctx          rctx            ; /* The "Job" queues                   */
    Key_lnk          key             ; /* A single key by device             */
                                       /*                                    */
    const Rad_desc  *desc            ; /* Hardware dependant procedures      */
    void           (*setmode)    ()  ; /* Configure receive filter banks     */
    IWORD          (*send)       ()  ; /* Send transmission request          */
    IWORD          (*receive)    ()  ; /* Retrieve receive message           */
    IWORD          (*setmodul)   ()  ; /* Set Radio modulation parameters    */
    IWORD          (*setpkt)     ()  ; /* Set Radio packet parameters        */
    IWORD          (*setrxcfg)   ()  ; /* Set Radio RX parameters            */
    IWORD          (*settxcfg)   ()  ; /* Set Radio TX parameters            */
    IWORD          (*setfreq)    ()  ; /* Set Radio frequency                */
    IWORD          (*setcad)     ()  ; /* Set Radio channel activity detect. */
                                       /*                                    */
    const Rad_drvdef *def            ; /* Definitions of the RAD device      */
    UINT           lctx              ; /* Low-level device index             */
    UINT           reg[RAD_NBREG]    ; /* Register list array                */
    UCHAR          closing           ; /* 1 if a REQ_CLOSE has been received */
                                       /*                                    */
    UCHAR          trl               ; /* Trailer size in buffer             */
    USHORT         mtu               ; /* Maximum Transmission Unit (MTU)    */
    USHORT         of7               ; /* Offset of the 1st byte in buffer   */
                                       /*                                    */
    UCHAR          *snd              ; /* Address of the transmission buffer */
    UCHAR          *rcv              ; /* Addess of the reception buffer     */
    USHORT         sndsize           ; /* Number of bytes to send            */
    USHORT         rcvsize           ; /* Size of the reception buffer       */
  END            Rad_ctx             ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

extern const Vmio_desc rad_vmio_desc ; /* RAD automaton description          */
extern UCHAR           rad_sysdrv    ; /* RAD driver number                  */


/*
    4    Prototypes of public procedures
    ____________________________________
*/

/* Sub-procedures for interruptions handlers ................................*/

  void  itradrx             (IWORD,UCHAR*,IWORD,UINT                         );
  void  itradtx             (IWORD,UCHAR*,IWORD                              );
  void  itradsig            (IWORD,IWORD,IWORD                               );


/* System driver interface ..................................................*/

extern UCHAR rad_numdrv              ; /* Numero du driver RAD               */
extern UCHAR rad_sysdrv              ; /* Numero du driver RAD               */

  IWORD rad_get_rctx        (INT,Req_ctx**,Key_lnk**,Req_subctx**            );
  IWORD rad_send            (Req_ctx*,Req_subctx*,
                             UCHAR*,INT,INT,void(*)(),INT,INT,INT            );
  IWORD rad_receive         (Req_ctx*,Req_subctx*,
                             UCHAR*,INT,INT,INT,INT,INT                      );
  IWORD rad_setval          (Req_ctx*,Req_subctx*,UCHAR*,INT,INT,INT,INT     );
  IWORD rad_getval          (Req_ctx*,Req_subctx*,UCHAR*,INT,INT,INT,INT     );
  IWORD rad_io              (Req_ctx*,Req_subctx*,Job*,INT,INT               );


/* System driver interface (blocking mode) ..................................*/

  IWORD rad_write           (INT, UCHAR *, INT                               );
  IWORD rad_read            (INT, UCHAR *, INT, INT *                        );


#endif                                 /* End of DRV_RAD_H.                  */

/*
    5  RAM data to be inclueded in board_xyz.c
    ___________________________________________
*/

#ifdef BOARD_XYZ_DATA_C

const USHORT rad_nbiod  = RAD_NBCTX  ; /* RAD seen by the VMK                */
const USHORT rad_nblw   = RAD_NBCTX+1; /* Count of logical ways of AUT_RAD   */
const USHORT rad_nbctx  = RAD_NBCTX  ; /* Count of device contexts           */

USHORT       rad_state [RAD_NBCTX+1] ; /* Etat de voie logique               */
Rad_ctx      rad_ctx   [RAD_NBCTX]   ; /* Controller's context table         */

Itf_ctx rad_itfctx[1+RAD_NBCTX]      ; /*                                    */

#endif                                 /* End of BOARD_XYZ_C                 */


