/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/event.l"               */
/*  Constants and definitions for VMIO and VMK events.                       */
/*                                                                           */
/*  Author : FA/JYA/TE                  Last modification : 04/11/2024       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
         3  Public variables
         4  Prototypes of public procedures

*/


#ifndef EVENT_H                        /* If this file is not already        */
#define EVENT_H                        /* included, include it.              */


/*
    1    Defines
    ____________
*/

#define ALL_EVENT         0xFFFE   /* All events                             */
#define END_MARKER        0xFFFF   /* End of list of transitions             */
#define OLD_SESSID        -2       /* Obsolet event                          */

/* VMK events ----------------------------------------------------------------

  VMK event codes : You have to modify the "nomcode[]" table in "vmk2.c" as
  soon as you add / delete / modify one of these definitions. The addition
  should not be done "at the economy", i.e. by simply adding the code at the
  end of the list, using the first free number. Even if the numbers have to be
  reassigned and it takes time, it should be done. Beware of telecom event
  codes, also present in assembler handlers.      

   [  0,..., 399]                     VMK events                      
      0,...,   9                      Driver generic events           
     10,...,  19                      EEPROM/FLASH driver events      
     20,...,  29                      Alarm message events            
     30,..., 199                      (unused)                        
    200,..., 249                      Telecom API / char driver events
    250,..., 269                      Memory allocation events        
    270,..., 309                      File API / block driver events  
    310,..., 399                      (unused)
                                                                      
   [400,...,1023]                     VMIO events                     
    400,..., 409                      I/O and IT events               
    410,..., 419                      (unused)                        
    420,..., 459                      Request events                  
    460,..., 479                      Time-out events                 
    480,..., 509                      Contition events                
    510,...,1023                      (unused)                

*/

/* Driver generic events ....................................................*/

#define DRV_KEY_PRESS     0            /* Keyboard - Key press               */
#define DRV_KEY_RELEASE   1            /* keyboard - Key release             */
#define DRV_MOUSE_PRESS   2            /* Mouse    - Key press               */
#define DRV_MOUSE_RELEASE 3            /* Mouse    - Key release             */
#define DRV_MOUSE_MOVE    4            /* Mouse    - Mouse move              */
#define DRV_TOUCH_PRESS   5            /* Touch    - Finder press            */
#define DRV_TOUCH_RELEASE 6            /* Touch    - Finder release          */
#define DRV_TOUCH_MOVE    7            /* Touch    - Finder move             */
                                       /* 8 to 9 unused                      */

/* EEPROM/FLASH driver events ...............................................*/

#define PLR_DONE          10           /* PLR ok                             */
#define PLR_ERROR         11           /* PLR impossible -> format           */
#define ERASE_DONE        12           /* Erase completion event             */
#define ERASE_ERROR       13           /* Erase error event                  */
#define PGM_DONE          14           /* Program completion event           */
#define PGM_ERROR         15           /* Program error event                */
#define BACKUP_WRITE      16           /* EEPROM request event               */
#define BACKUP_DONE       17           /* EEPROM completion  event           */
#define BACKUP_ERROR      18           /* Error event                        */
                                       /* 19 unused                          */

/* Alarm message events .....................................................*/

#define WRITE_ALARM       20           /* Alarm sending                      */
                                       /* 21 to 199 unused                   */

/* Telecom API / character driver events ....................................*

   - The order of codes below is used by the "dpsfsm(]" table of drv_req.c
   - Codes from RESP_INIT are indices in the "evt2state" of "drv.c"

*/

#define REQ_INIT         200           /* TLCM request - Initialisation      */
#define REQ_END          201           /* TLCM request - Termination         */
#define REQ_ALLOC        202           /* TLCM request - Allocation          */
#define REQ_FREE         203           /* TLCM request - Free                */
#define REQ_OPEN         204           /* TLCM request - Open                */
#define REQ_CLOSE        205           /* TLCM request - Close               */
#define REQ_ALLOC_SUB    206           /* TLCM request - Alloc sub-channel   */
#define REQ_FREE_SUB     207           /* TLCM request - Free sub-channel    */
#define REQ_CONNECT      208           /* TLCM request - Connection          */
#define REQ_LISTEN       209           /* TLCM request - Wait for connection */
#define REQ_DISCONNECT   210           /* TLCM request - Disconnection       */
#define REQ_ABORT        211           /* TLCM request - Cancel connection   */
#define REQ_WRITE        212           /* TLCM request - Write               */
#define REQ_READ         213           /* TLCM request - Read                */
#define REQ_SIGNAL       214           /* TLCM request - Signal              */
#define REQ_PUT          215           /* TLCM request - Write block         */
#define REQ_GET          216           /* TLCM request - Read block          */
#define REQ_SETVAL       217           /* TLCM request - Parameter           */
#define REQ_GETVAL       218           /* TLCM request - Read parameter      */
#define REQ_RPC          219           /* TLCM request - RPC                 */
                                                                                 
#define REQ_FIRST        REQ_INIT      /*                                    */
#define REQ_LAST         REQ_RPC       /*                                    */
#define REQ2RESP (RESP_INIT-REQ_INIT)  /*                                    */

#define RESP_INIT        220           /* TLCM answer - Initialisation       */
#define RESP_END         221           /* TLCM answer - Termination          */
#define RESP_ALLOC       222           /* TLCM answer - Allocation           */
#define RESP_FREE        223           /* TLCM answer - Free                 */
#define RESP_OPEN        224           /* TLCM answer - Open                 */
#define RESP_CLOSE       225           /* TLCM answer - Close                */
#define RESP_ALLOC_SUB   226           /* TLCM answer - Alloc sub-channel    */
#define RESP_FREE_SUB    227           /* TLCM answer - Free sub-channel     */
#define RESP_CONNECT     228           /* TLCM answer - Connection           */
#define RESP_LISTEN      229           /* TLCM answer - Wait for connection  */
#define RESP_DISCONNECT  230           /* TLCM answer - Disconnection        */
#define RESP_ABORT       231           /* TLCM answer - Cancel connection    */
#define RESP_WRITE       232           /* TLCM answer - Write                */
#define RESP_READ        233           /* TLCM answer - Read                 */
#define RESP_SIGNAL      234           /* TLCM answer - Signal               */
#define RESP_PUT         235           /* TLCM answer - Write block          */
#define RESP_GET         236           /* TLCM answer - Read block           */
#define RESP_SETVAL      237           /* TLCM answer - Parameter            */
#define RESP_GETVAL      238           /* TLCM answer - Read parameter       */
#define RESP_RPC         239           /* TLCM answer - RPC                  */
                                                                                 
#define RESP_FIRST       RESP_INIT
#define RESP_LAST        RESP_RPC

#define IND_DISCONNECT   240           /* TLCM report - Disconnected         */
#define IND_SIGNAL       241           /* TLCM report - Incoming signal      */
#define IND_REPORT       242           /* TLCM report - Update param         */
#define IND_PNP          243           /* TLCM report - Connection           */
#define IND_DATA         244           /* TLCM report - Data                 */
                                       /* TLCM report - 245 to 249 unused    */
#define IND_FIRST        240           /* First IND_xyz event                */
#define IND_LAST         244           /* Last  IND_xyz event                */

/* Memory allocation events .................................................*/

#define REQ_ALLOC_BUF    250           /* MEMBUF request - Allocation        */
#define REQ_FREE_BUF     251           /* MEMBUF request - Liberation        */
#define REQ_ALLOC_LINK   252           /* MEMBUF request - Allocation        */
#define REQ_FREE_LINK    253           /* MEMBUF request - Liberation        */
#define REQ_ALLOC_ALI    254           /* MEMBUF request - Allocation        */
#define REQ_REALLOC      255           /* MEMBUF request - Re-allocation     */
#define REQ_POST_BUF     256           /* MEMBUF request - Post_buffer       */
#define REQ_TAKE_MUT     257           /* MEMBUF request - Mutex request     */
#define REQ_RELEASE_MUT  258           /* MEMBUF request - Mutex release     */
                                       /* MEMBUF request - 259 unused        */
                                                                                 
#define RESP_ALLOC_BUF   260           /* MEMBUF answer - Allocation         */
#define RESP_FREE_BUF    261           /* MEMBUF answer - Liberation         */
#define RESP_ALLOC_LINK  262           /* MEMBUF answer - Allocation         */
#define RESP_FREE_LINK   263           /* MEMBUF answer - Liberation         */
#define RESP_ALLOC_ALI   264           /* MEMBUF answer - Allocation         */
#define RESP_REALLOC     265           /* MEMBUF answer - Re-allocation      */
#define RESP_POST_BUF    266           /* MEMBUF answer - Post buffer        */
#define RESP_TAKE_MUT    267           /* MEMBUF answer - Mutex request      */
#define RESP_RELEASE_MUT 268           /* MEMBUF answer - Mutex release      */
                                       /* MEMBUF answer - 269 unused         */

/* File API / block driver events ...........................................*

  Warning, the order of the codes below is used by the table "volfsm[]"
  from drv_req.c.     

*/

#define REQ_MOUNT        270           /* FILE request - Mount partition     */
#define REQ_UMOUNT       271           /* FILE request - Umount partition    */
#define REQ_VINFO        272           /* FILE request - Volume informations */
#define REQ_CREDIR       273           /* FILE request - Create directory    */
#define REQ_DELDIR       274           /* FILE request - Delete directory    */
#define REQ_CREFILE      275           /* FILE request - Create file         */
#define REQ_DELFILE      276           /* FILE request - Delete file         */
#define REQ_RENFILE      277           /* FILE request - Rename file         */
#define REQ_FLIST        278           /* FILE request - Files list          */
#define REQ_FINFO        279           /* FILE request - File informations   */
#define REQ_FOPEN        280           /* FILE request - Open file           */
#define REQ_FCLOSE       281           /* FILE request - Close file          */
#define REQ_PSEEK        282           /* FILE request - Write pointer       */
#define REQ_FPUT         283           /* FILE request - Block write         */
#define REQ_GSEEK        284           /* FILE request - Read pointer        */
#define REQ_FGET         285           /* FILE request - Read block          */
                                       /* FILE requet  - 286 to 289 unusued  */
                                                                             
#define RESP_MOUNT       290           /* FILE answer - Mount                */
#define RESP_UMOUNT      291           /* FILE answer - Umount               */
#define RESP_VINFO       292           /* FILE answer - Volume informations  */
#define RESP_CREDIR      293           /* FILE answer - Create directory     */
#define RESP_DELDIR      294           /* FILE answer - Delete directory     */
#define RESP_CREFILE     295           /* FILE answer - Create file          */
#define RESP_DELFILE     296           /* FILE answer - Delete file          */
#define RESP_RENFILE     297           /* FILE answer - Rename file          */
#define RESP_FLIST       298           /* FILE answer - Files list           */
#define RESP_FINFO       299           /* FILE answer - File informations    */
#define RESP_FOPEN       300           /* FILE answer - Open file            */
#define RESP_FCLOSE      301           /* FILE answer - Close file           */
#define RESP_PSEEK       302           /* FILE answer - Write pointer        */
#define RESP_FPUT        303           /* FILE answer - Block write          */
#define RESP_GSEEK       304           /* FILE answer - Read pointer         */
#define RESP_FGET        305           /* FILE answer - Read block           */
                                       /* FILE answer - 306 to 399 unused    */

/* VMIO events --------------------------------------------------------------*/

/* IO and IT events .........................................................*/

#define DRV_END_FIRST    400           /* Alias for the first DRV_xyz_END    */
#define DRV_IO_END       400           /* I/O end - Disk / other driver      */
#define DRV_ASY_END      401           /* I/O end - ASY driver               */
#define DRV_I2C_END      402           /* I/O end - I2C driver               */
#define DRV_SPI_END      403           /* I/O end - SPI driver               */
#define DRV_USB_END      404           /* I/O end - USB driver               */
#define DRV_ETH_END      405           /* I/O end - ETH driver               */
#define DRV_RAD_END      406           /* I/O end - RAD driver               */
#define DRV_END_LAST     406           /* Alias for the last DRV_xyz_END     */
#define IT_RX            407           /* IT - Reception                     */
#define IT_TX            408           /* IT - End of transmiting            */
#define IT_PRESS         409           /* IT - Key press                     */
#define IT_RELEASE       410           /* IT - Key release                   */
#define IT_MSE_MOVE      411           /* IT - Mouve move                    */
#define IT_INS           412           /* IT - Card insertion                */
#define IT_EXT           413           /* IT - Card extraction               */
#define IT_SYN0          414           /* IT - Synchro HDCP                  */
#define IT_SYN127        415           /* IT - Synchro HDCP                  */
                                       /* IT - 416 to 419 unused             */

/* Request events ...........................................................*/

#define REQ_CMD          420           /* Request - Starting                 */
#define REQ_RESET        421           /* Request - Reset card / device      */
#define REQ_PTS          422           /* Request - Send PTS                 */
#define REQ_I2C          423           /* Request - Write I2C                */
#define REQ_ENDS         424           /* Request - System ending            */
#define REQ_CHANGE       425           /* Request - RCU selection            */
                                       /* Request - 426 to 390 unused        */

/* Answer events ............................................................*/

#define RESP_CMD         440           /* Answer - Starting                  */
#define RESP_RESET       441           /* Answer - Reset card / device       */


/* Time-out events ..........................................................*/

#define TO_POW           460           /* Time-out - POWER                   */
#define TO_LED           461           /* Time-out - LED                     */
#define TO_DIS           462           /* Time-out - DISPLAY                 */
#define TO_FRE           463           /* Time-out - FRONT-END               */
#define TO_SWI           464           /* Time-out - SWITCH                  */
#define TO_SMC           465           /* Time-out - SMC                     */
#define TO_COM           466           /* Time-out - I2C                     */
#define TO_STEP          467           /* Time-out - I2C next step           */
#define TO_POLL          468           /* Time-out - I2C Tunnel polling      */
#define TO_KBD           469           /* Time-out - KBD press time-out      */
#define TO_SCAN          470           /* Time-out - KBD periodic scan       */
#define TO_RCU           471           /* Time-out - RCU no reception        */
#define TO_SKIP          472           /* Time-out - RCU end of skip         */
#define TO_DSK1          473           /* Time-out - DSK disk I/O            */
#define TO_DSK2          474           /* Time-out - DSK disk standby        */
#define TO_FAN           475           /* Time-out - FAN                     */
#define TO_ADC           476           /* Time-out - ADC inputs scan         */
#define TO_SCSI          477           /* Time-out - SCSI                    */
#define TO_PROCAN        478           /* Time-out - PROCAN                  */
                                       /* Time-out - 479 unused              */

/* Condition events .........................................................*/

#define C_WAIT           480           /* Cond - Wait                        */
#define C_ERROR          481           /* Cond - Error                       */
#define C_RUN            482           /* Cond - Run                         */
#define C_HALT           483           /* Cond - Halt                        */
#define C_MORE           484           /* Cond - Continue                    */
#define C_OPOSITIF       485           /* Cond - RESP_OPEN positive          */
#define C_ONEGATIF       486           /* Cond - RESP_OPEN negative          */
#define C_OPENED         487           /* Cond - Open ok                     */
#define C_OPEN           488           /* Cond - Open without error          */
#define C_NOTOPENED      489           /* Cond - Close failed                */
#define C_NOTOPEN        490           /* Cond - Close failed without error  */
#define C_ERRIO          491           /* Cond - I/O error                   */
#define C_CLOSED         492           /* Cond - Closed driver               */
#define C_NOMORE         493           /* Cond - C_MORE negation             */
#define C_INACTIVE       494           /* Cond - Inactivation                */
#define C_IDLE           495           /* Cond - Idle mode                   */
#define C_END            496           /* Cond - End of sequence             */
#define C_READ           497           /* Cond - Read to do                  */
#define C_WRITE          498           /* Cond - Write to do                 */
#define C_SYN            499           /* Cond - Wait for IT_SYN             */
#define C_SATUR          500           /* Cond - FAN PWM saturation          */
                                       /* Cond - 501 to 1023 unused          */

/*
    2    Structures and typedefs
    ____________________________
*/

/*
    3    Public variables
    _____________________
*/

/*
    4    Prototypes of public procedures
    ____________________________________
*/

#endif                                 /* End of EVENT_H                     */
