/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/mem_user.h"            */
/*  Constants and definitions for user memory manager.                       */
/*                                                                           */
/*  Author : JYA                        Last modification : 05/07/2024       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Array sizes
            1.2  Sizes in bytes
            1.3  Masks for the "szcd" field of Blk_desc
            1.4  Blocks tag values

         2  Structures and typedefs
            2.1  One RAM segment descriptor - Ram_def_seg
            2.2  One RAM segment block list descriptor - Blk_seg
            2.3  One RAM block descriptor - Blk_desc
            2.4  One buffer header - Buf_head
            2.5  One moveableallocation header - Alloc_head
            2.6  One huge allocation header - Huge_head
            2.7  One memory partition context structure - Mem_ctx

         3  Public variables
            3.1  Memory partition context table - mem_ctx[]
            3.2  Telecom buffers user sizes in bytes - const mem_buf_sz[][]
            3.3  Telecom buffers cell sizes in bytes - const realbsz[][]
            3.4  User data guaranted alignments for buffers - const mem_buf_ali
            3.5  Cell to buffer header offsets - const head_of7[][]
            3.6  Bitstrings offsets lists addresses - const "btm_of7_lst[]
            3.7  Bitstring sizes lists addresses - const btm_nbbits_lst[]
            3.8  Bitstring sizes in bytes - const btm_sz_lst[]
            3.9  Control area sizes in bytes - const ctl_sz_lst[]

         4  Prototypes of public procedures

         5  RAM data to be included in space_appcont_xyz.c

*/


#ifndef MEM_USR_H                      /* If this file is not already        */
#define MEM_USR_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/

/*  1.1  Array sizes
    ----------------
*/
#define MMAXSEG     4                  /* Max count of segments per partition*/
#define NBSIZE      16                 /* Max count of buffer sizes          */
#define TMAX        (NBSIZE-1)         /* Plus grand code de size possible.  */


/*  1.2  Sizes in bytes
    -------------------
*/

#define LGDESC      sizeof(Blk_desc)   /* Size of one "Big_desc" in bytes    */
#define SMINA       64                 /* Taille min allocation deplacable.  */
#define SMOV        (SAH + SMINA)      /*                                    */
#define SBH         32                 /* Equal to sizeof(Buf_head)          */
#define SAH         18                 /* Equal to sizeof(Alloc_head)        */
#define SBTM        4112               /* Biggest possible bitstrings area   */
#define SCNT        32                 /* Size in bytes for 16 counters      */
#define SCTL        (SBTM+SCNT)        /* Biggest controla area size in bytes*/



/*  1.3  Masks for the "szcd" field of Blk_desc
    -------------------------------------------
*/
#define M_SZCD      0xF                /* Mask for the size code             */
#define B_BIG       (1<<4)             /* Big block flag                     */
#define B_FST       (1<<6)             /* First block of a segment flag      */
#define B_LST       (1<<7)             /* Last block of a segment flag       */


/*  1.4  Blocks tag values
    ----------------------
*/

#define BLK_FREE    0                  /* Block is currently free            */
#define BLK_BUFFER  1                  /* Block holds telecom buffers        */
#define BLK_MOV     2                  /* Block holds moveable allocations   */
#define BLK_HUGE    3                  /* Block holds one "huge" allocation  */


/*
    2    Structures and typedefs
    ____________________________


    2.1  One RAM segment descriptor - Ram_def_seg
    ---------------------------------------------

*/

typedef struct                         /* One RAM segment descriptor         */
  BEGIN                                /* ---------------------------------- */
    UCHAR       *adr                 ; /* Start of segment address           */
    UCHAR       *end                 ; /* End   of segment address           */
    UINT        size                 ; /* Segment size in bytes              */
    USHORT      flags                ; /* List of option flags               */
                                       /* b1  1:moveable       0:no moveable */
  END           Ram_def_seg          ; /* ---------------------------------- */


/*  2.2  One RAM segment block list descriptor - Blk_seg
    ----------------------------------------------------
*/

typedef struct                         /* RAM segment big blocks description */
  BEGIN                                /* ---------------------------------- */
    USHORT      first                ; /* First block number                 */
    USHORT      last                 ; /* Last block number                  */
    USHORT      blknb                ; /* Count of big blocks                */
    UCHAR       *adr                 ; /* 1st byte of first block address    */
    UCHAR       *end                 ; /* Last byte last block address , + 1 */
    UCHAR       blk0szcd             ; /* Block 0 size code or 0             */
    UCHAR       blkszcd              ; /* Big blocks size code or 0          */
    UINT        blk0sz               ; /* Block 0 size in bytes              */
    UINT        blksz                ; /* Big blocks size in bytes           */
  END           Blk_seg              ; /* ---------------------------------- */


/*  2.3  One RAM block descriptor - Blk_desc
    ----------------------------------------
*/

typedef struct                         /* Description of one RAM block       */
  BEGIN                                /* ---------------------------------- */
    UCHAR       *blkadr              ; /* Start of block address             */
    UINT        *ctladr              ; /* Start of control area address      */
    UCHAR       blkbits              ; /* Allocation tag value               */
    UCHAR       ctlbits              ; /* Control area size code             */
    UINT        usedbytes            ; /* Count of used bytes                */
  END           Blk_desc             ; /* ---------------------------------- */


/*  2.4  One buffer header - Buf_head
    ---------------------------------

    WARNING : The first fields of the "Buf_link" structure must be exactly
              the same ones has the "Buf_head" structure which follows. So,
              if "Buf_head" is updated, then "Buf_link" (mem_link.h) must be
              accordingly updated.
*/

typedef struct b_head                  /* Telecom buffer header              */
  BEGIN                                /* ---------------------------------- */
    struct b_head *next              ; /* Forward link                     4 */
    struct b_head *prev              ; /* Backward link                    4 */
    UINT        pos                  ; /* Fist user byte index             4 */
    UINT        nb                   ; /* Count of user bytes              4 */
    UINT        size                 ; /* Taille en octets/code de size    4 */
    UINT        blkoff               ; /* Block offset in bytes            4 */
    USHORT      blknum               ; /* Block number                     2 */
    USHORT      uid                  ; /* Memory user number               2 */
    USHORT      flags                ; /* List of option flags             2 */
    USHORT      ref                  ; /* Counter of references            2 */
  END           Buf_head             ; /* ------------------------------- 32 */


/*  2.5  One moveable allocation header - Alloc_head
    ------------------------------------------------
*/

typedef struct                         /* Moveable allocation header         */
  BEGIN                                /* ---------------------------------- */
    UINT        ind                  ; /* Index table index value          4 */
    UINT        size                 ; /* User size in bytes               4 */
    USHORT      nbulk                ; /* Item count for a bulk            2 */
    USHORT      lock                 ; /* Owner Thread / locks count       2 */
    USHORT      uid                  ; /* Memory user number               2 */
    USHORT      flags                ; /* List of option bits              2 */
    USHORT      ref                  ; /* Counter of references            2 */
  END           Alloc_head           ; /* ------------------------------- 18 */


/*  2.6  One huge allocation header - Huge_head
    -------------------------------------------
*/

typedef struct                         /* "huge" allocation header           */
  BEGIN                                /* ---------------------------------- */
    UINT        pos                  ; /* Fisrt user byte index            4 */
    UINT        nb                   ; /* Count of user bytes              4 */
    USHORT      bigfirst             ; /* First big block number           2 */
    USHORT      bigcnt               ; /* Count of big blocks              2 */
    USHORT      uid                  ; /* Memory user number               2 */
    USHORT      flags                ; /* List of option flags             2 */
    USHORT      ref                  ; /* Counter oif references           2 */
  END           Huge_head            ; /* ------------------------------- 16 */


/*  2.4  One memory partition context structure - Mem_ctx
    -----------------------------------------------------
*/

typedef struct                         /* Description of a RAM partition     */
  BEGIN                                /* ---------------------------------- */
    UCHAR       segnb                ; /* Number of segments (2 at least)    */
    Ram_def_seg ramseg[MMAXSEG]      ; /* Descriptions of the RAM segments   */
    Blk_seg     blkseg[MMAXSEG]      ; /* Big blocks of each segments        */
    Buf_head    *pall [NBSIZE ]      ; /* Head of pre-allocated buffer links */
                                       /*                                    */
    UCHAR       init                 ; /* Is set to 1 at initialisation time */
    UCHAR       part                 ; /* Number of this partition           */
    UCHAR       dec                  ; /* Memory split selected              */
    UCHAR       bigszcd              ; /* Code size for big blocks           */
                                       /*                                    */
    UINT        blkdescnb            ; /* Count of entries of "blkdesc[]"    */
    Blk_desc    *blkdesc             ; /* Ad of the big blocks address table */
    UINT        ctlsz                ; /* Control area size in bytes         */
    UINT        *begctl              ; /* Ad of the 1st control area         */
    UINT        indnb                ; /* Count of entries of "hindex"       */
    UCHAR       **hindex             ; /* Ad of the index table              */
    UINT        indfree              ; /* Head of chain of the free "hindex" */
                                       /*                                    */
    UINT        allocated            ; /* Debug, No of bytes allocated       */
    UINT        max_alloc            ; /* Debug, Max no of bytes allocated   */
    UINT        szsys                ; /* Debug, Size of the system zone     */
    UINT        rem                  ; /*                                    */
    UINT        chkblkbuf            ; /* Debug, for "chk_blkbuf"            */
  END           Mem_ctx              ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/

/*  3.1  Memory partition context table - mem_ctx[]
    -----------------------------------------------
*/

extern Mem_ctx  mem_ctx[]            ; /* Partition context table            */
extern const UINT mmaxpart           ;


/*  3.2  Telecom buffers user sizes in bytes - const mem_buf_sz[][]
    ---------------------------------------------------------------
*/

extern const UINT mem_buf_sz[4][NBSIZE]  ;


/*  3.3  Telecom buffers cell sizes in bytes - const realbsz[][]
    ------------------------------------------------------------
*/

extern const UINT realbsz[4][NBSIZE]     ;


/*  3.4  User data guaranted alignments for buffers - const mem_buf_ali[][]
    -----------------------------------------------------------------------
*/

extern const UINT mem_buf_ali[4][NBSIZE] ;


/*  3.5  Cell to buffer header offsets - const head_of7[][]
    -------------------------------------------------------
*/

extern const UINT head_of7[4][NBSIZE]    ;


/*  3.6  Bitstrings offsets lists addresses - const btm_of7_lst[]
    -------------------------------------------------------------
*/

extern const USHORT *btm_of7_lst[]       ;


/*  3.7  Bitstring sizes lists addresses - const btm_nbbits_lst[]
    -------------------------------------------------------------
*/

extern const UINT *btm_nbbits_lst[]      ;


/*  3.8  Bitstring sizes in bytes - const btm_sz_lst[]
    --------------------------------------------------
*/

extern const USHORT btm_sz_lst[]         ;


/*  3.9  Control area sizes in bytes - const ctl_sz_lst[]
    -----------------------------------------------------
*/

extern const USHORT ctl_sz_lst[]         ;


/*
    4    Prototypes of public procedures
    ____________________________________
*/

IWORD init_mem            (INT,INT,INT                                       );
IWORD end_mem             (INT                                               );
IWORD alloc_memuid        (UINT*                                             );
IWORD free_memuid         (UINT,UINT                                         );
IWORD get_part_dbg        (INT,INT*,INT*,INT*,INT*                           );
IWORD get_part_info       (INT,INT*,INT*,INT[],INT[],INT[],INT*              );
IWORD test_allocadr       (UCHAR*,INT*,INT*,INT*                             );
IWORD chk_size            (Mem_ctx*,INT,UINT*                                );
IWORD check_user_str      (IWORD                                             );


#endif                                 /* End of MEM_USR_H.                  */

/*
    5 RAM data to be included in space_appcont_xyz.c
    ------------------------------------------------
*/

#ifdef SPACE_APPCONT_XYZ_DATA_C
const UINT mmaxpart = MMAXPART       ;
Mem_ctx         mem_ctx[MMAXPART]    ; /* Partition context table            */
#endif                                 /* End of APP_CONT_XYZ_DATA_C          */


