/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/req_lib.h              */
/*  Constants and definitions for request library interface.                 */
/*                                                                           */
/*  Author : JYA/TE                     Last modification : 05/08/2024       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Bit masks for the "flags" field of S_evt
            1.2  Bit masks for the Job "code" field
            1.3  Values for bits b0..7 of the Job "code"field 
            1.4  Values for bits b12..15 of the Job "code"field 
            1.5  Bits of the "file" field and queue numbers
            1.6  Values of the "profile" parameter for link_snd & link_rcv
            1.7  Values of the "profile" parameter for link_reqevt
            1.8  Return values of "xyz_trt_job"
            1.9  Return values of "xyz_evt_end"

         2  Structures and typedefs
            2.1  One context table description structure - Tab_ctx
            2.2  Context header structure - Hd_ctx
            2.3  Channel context - Req_ctx/Chk_fifo4
            2.4  Sub-channel context - Req_subctx
            2.5  Contexte automate de dialogue - Dia_ctx
            2.6  Description d'un job - Job/Job_data/Job_dia
            2.7  Une clef et ses 4 tetes de chaines - Key_lnk
            2.8  Tete de chaine de jobs - Hd_joblnk/Hd_jobfree
            2.9  Structure de lien avec la couche basse ou a droite
            2.10 Managing slots structure - Slot_ctx
            2.11 Structure buffer circulaire
            2.12 Log for "set_job_state" structure
            2.13 Log for "xyz_evt_end" structure

         3  Public variables

         4  Prototypes of public procedures

         5  RAM data to be inclueded in space_iocont_xyz.c


*/


#ifndef REQ_LIB_H                      /* If this file is not already        */
#define REQ_LIB_H                      /* included, include it.              */


/*
    1    Defines
    ____________

    1.1  Bit masks for the "flags" field of S_evt
    ---------------------------------------------

    Whenever standard request events, such are REQ_SETVAL, are used by system
    interface procedures, bit b10 is set when the REQ_SETVAL has been built
    by a system interface procedure
*/

#define API_MODE        0x00004000     /* API is "drv.c" or system           */
#define DRV_API         0x00000000     /* 0:DRV API                          */
#define SYS_API         0x00004000     /* 1:SYS API                          */

#define RSP_MODE        0x00008000     /* API sends event responses or not   */
#define SEND_RSP        0x00000000     /* 0: Send RESP_xyz response events   */
#define NO_RSP          0x00008000     /* 1: Do not send RESP_xyz events     */

/*  1.2  Bit masks for the Job "code" field
    ---------------------------------------
*/

#define MSK_CODE           0x007F      /* b0..6 are the "code" value mask    */
#define B_SYS              (1<<7)      /* b7  Demander 1:Drv systeme 0;cara  */
#define B_MORE             (1<<8)      /* b8  More jobs follows              */
#define B_RST              (1<<9)      /* b9  Demande de reset code erreur   */
#define B_EVST            (1<<10)      /* b10 Wait an event before starting  */
#define B_EVEND           (1<<11)      /* b11 Wait for an end event          */
#define MSK_STATE          0xF000      /* b12..15 are the Job current state  */


/*  1.3  Values for bits b0..7 of the Job "code"field 
    -------------------------------------------------
*/

#define O_CONNECT               0      /* Issue outgoing cnx request         */
#define O_LISTEN                1      /* Wait for incoming cnx request      */
#define O_DISCONNECT            2      /* Deconnexion request                */
#define O_ABORT                 3      /* Brutal termination/deconnexion     */
#define O_START_SINK            4      /* start data flow indication         */
#define O_START_SOURCE          5      /* Start data flow request            */
#define O_STOP                  6      /* Stop data flow                     */
#define O_SND                   7      /* Send data request                  */
#define O_RCV                   8      /* Receive data request               */
#define O_IO                    9      /* Composite IO request               */
#define O_SIGNAL               10      /* Send signal request                */
#define O_SETVAL               11      /* Set parameter value request        */
#define O_GETVAL               12      /* Get parameter value request        */
#define O_RPC                  13      /* Remote Procedure Call request      */

#define O_EVTCON               14      /* Send a RESP_CONNECT                */
#define O_EVTLISTEN            15      /* Send a RESP_LISTEN                 */
#define O_EVTDISC              16      /* Send a RESP_DISC                   */
#define O_EVTABORT             17      /* Send a RESP_ABORT                  */
#define O_EVTSTART             18      /* Send a RESP_START                  */
#define O_EVTSTOP              19      /* Send a RESP_STOP                   */
#define O_EVTWRITE             20      /* Send a RESP_WRITE                  */
#define O_EVTREAD              21      /* Send a RESP_READ                   */
#define O_EVTSIGN              22      /* Send a RESP_SIGNAL                 */
#define O_EVTSETV              23      /* Send a RESP_SETVAL                 */
#define O_EVTGETV              24      /* Send a RESP_GETVAL                 */
#define O_EVTRPC               25      /* Send a RESP_RPC                    */
#define O_EVTIOEND             26      /* Send a EVTS_IO_END - xxx_io        */
#define O_EVTIOERR             27      /* Send a EVTS_IO_END si erreur       */

#define O_CALLBACK             28      /* Call a call-back function          */
#define O_REQSKIP              29      /* Skip following jobs if error       */
#define O_WAITMSEC             30      /* Wait for a number of millisecs.    */
#define O_SNDNEXT              31      /* Send data to next entity           */
#define O_RESET                32      /* Perform a reset sequence           */
#define O_POLL                 33      /* Perform a polling sequence         */
#define O_WAITCMD              34      /* Slaves wait for an incoming CMD    */


/*  1.4  Values for bits b12..15 of the Job "code"field 
    ----------------------------------------------------

    This value is the Job "state" current value
*/

#define W_FREE                  0      /* Job structure is free              */
#define W_INREQ                 1      /* Job waiting into "inreq"           */
#define W_RDY                   2      /* Job is ready to run                */
#define W_EVST                  3      /* Job waiting for a start event      */
#define W_EVCNT                 4      /* Job waiting for a continue event   */
#define W_EVEND                 5      /* Job waiting for a end event        */
#define RESTART             W_EVST     /* Alias for W_EVST                   */


/*  1.5  Bits of the "file" field and queue numbers
    -----------------------------------------------
*/

#define W_FILE0            (1<< 4)     /* Queue lnk[0] is blocking us        */
#define W_FILE1            (1<< 5)     /* Queue lnk[1] is blocking us        */
#define W_FILE2            (1<< 6)     /* Queue lnk[2] is blocking us        */
#define W_FILE3            (1<< 7)     /* Queue lnk[3] is blocking us        */

#define QUEUE0                   0     /* Queue number is 0                  */
#define QUEUE1                   1     /* Queue number is 1                  */
#define QUEUE2                   2     /* Queue number is 2                  */
#define QUEUE3                   3     /* Queue number is 3                  */
#define INREQ                    4     /* Selector for "inreq"               */

/*  1.6  Values of the "profile" parameter for link_snd & link_rcv
    --------------------------------------------------------------
*/

#define PROF_EVUN           0          /* Bit B_EVUN of O_SND/RCV set        */
#define PROF_EVST           1          /* Bit B_EVST of O_EVTWRITE/READ set  */
#define PROF_SLOTS          2          /* Low-level provides slots           */

/*  1.7  Values of the "profile" parameter for link_req
    ---------------------------------------------------
*/

#define PROF_NOWAIT         0          /* Bit B_EVUN of O_REQ set            */
#define PROF_RQWAIT         1          /* Bit B_EVEND of O_REQ set           */
#define PROF_EVWAIT         2          /* Bit B_EVST of O_EVT set            */

/*  1.8  Return values of "xyz_trt_job"
    -----------------------------------
*/

#define STA_FREEJOB         0          /* Free the job                       */
#define STA_WAITEVCNT       1          /* Wait for a continue event          */
#define STA_WAITEVEND       2          /* Wait for an end event              */
#define STA_USEEVEND        3          /* Use value of bit B_EVUN            */

/*  1.9  Return values "xyz_evt_end"
    --------------------------------
*/
#define LOOP_STOP           (0<<0)     /* Stop calling "xyz_evt_end"         */
#define LOOP_AGAIN          (1<<0)     /* Call again "xyz_evt_end"           */
#define EXEC_SEARCH         (0<<1)     /* Call "search_joblnk"               */
#define EXEC_FIRST          (1<<1)     /* Execute firts job of link          */

/*
    2    Structures and typedefs
    ____________________________


   2.1  One context table description structure - Tab_ctx
   ------------------------------------------------------
*/

typedef struct     tctx                /* One context table description      */
  BEGIN                                /* ---------------------------------- */
    struct hctx  **ctxptr            ; /* Adr Context array table adress     */
    UINT           szctxptr          ; /* Size of adress array               */
    struct hctx   *stactx            ; /* Adr static context array           */
    UINT           szstactx          ; /* Size of context array              */
    UINT           ctxlen            ; /* Size in byte of a context          */
    UINT           ctxnball          ; /* Dynamic allocation grain           */
    UINT           flags             ; /* Flags                              */
                                       /* ---------------------------------- */
    UINT           of7rctx           ; /* Offset in context for  Req_ctx     */
    UINT           of7keylnk         ; /* Offset in context for  Key_lnk     */
    UINT           keylen            ; /* Size in byte of a key              */
                                       /* ---------------------------------- */
    UINT           nbbuf             ; /* Number of allocated buffers        */
    UINT           nbctx             ; /* Number of allocated contexts       */
    struct hctx   *first             ; /* Adress of first allocated buffer   */
  END              Tab_ctx           ; /* ---------------------------------- */


/*
   2.2  Context header structure - Hd_ctx
   --------------------------------------
*/

typedef struct   hctx                  /* En-tete d'un contexte              */
  BEGIN                                /* ---------------------------------- */
    struct tctx   *tabctx            ; /* Adresse de la table de contextes   */
    USHORT         ictx              ; /* Indice/Identifiant de ce contexte  */
    USHORT         nbref             ; /* Compteur de references             */
    struct hctx   *next              ; /* Adresse contexte alloue suivant    */
  END              Hd_ctx            ; /* ---------------------------------- */

/*
   2.3  Description d'un contexte de "req_lib.c" - Req_ctx/Chk_req_ctx
   -------------------------------------------------------------------
*/

typedef struct                         /* Files req/ecr/lec/aut et compteur  */
  BEGIN                                /* ---------------------------------- */
    UINT           iodsys            ; /* IOD systeme   de ce contexte       */
    USHORT         myaut             ; /* Numero automate VMIO associe       */
    USHORT         mylw              ; /* Numero Voie Logique associee       */
    USHORT         mydrvsys          ; /* Numero VMIO de driver SYSTEME      */
                                       /*                                    */
    UINT           iodcar            ; /* IOD VMOS    de ce contexte         */
    UINT           flags             ; /* Champ "flags" du REQ_ALLOC         */
    USHORT         mydrvcar          ; /* Numero VMIO de driver CARACTERE    */
    Resp_info      resp_info         ; /* reqaut/reqlw/reqnode/reqid/sessid  */
                                       /*                                    */
    IWORD        (*trt_job)()        ; /* Ad procedure trt specifique job    */
    IWORD        (*evt_end)()        ; /* Ad procedure evenement de fin      */
                                       /*                                    */
    void           *useradr          ; /* User defined address               */
  END              Req_ctx           ; /* ---------------------------------- */


typedef struct                         /* Param pour procedure "chk_req_ctx" */
  BEGIN                                /* ---------------------------------- */
    USHORT         aut               ; /* Valeur attendue du numero d'auto   */
    USHORT         lwmin             ; /* Plus petite valeur de "mylw"       */
    USHORT         lwmax             ; /* Plus grande valeur de "mylw"       */
  END              Chk_req_ctx       ; /* ---------------------------------- */


/*
   2.4  Sub-channel context - Req_subctx
   -------------------------------------
*/

typedef struct                         /* Files req/ecr/lec/fsm et compteur  */
  BEGIN                                /* ---------------------------------- */
    USHORT         myaut             ; /* associated VMIO FSM number         */
    USHORT         mylw              ; /* associated Logical Way number      */
                                       /*                                    */
    UINT           iodsub            ; /* sub-channel VMK identifier         */
    USHORT         reqaut            ; /* Numero automate emetteur requete   */
    USHORT         reqlw             ; /* Numero VL       emetteur requete   */
    USHORT         reqnode           ; /* Numero noeud    emetteur requete   */
    USHORT         reqid             ; /* Identifiant de requete             */
    USHORT         sessid            ; /* Identifiant de la session          */
  END              Req_subctx        ; /* ---------------------------------- */


/*
   2.5  Contexte automate de dialogue - Dia_ctx
   --------------------------------------------
*/

typedef struct    dia                  /* Dialog FSM context                 */
  BEGIN                                /* ---------------------------------- */
    USHORT         myctx             ; /* Associated context number          */
    SHORT          sta               ; /* State      dialog FSM              */
    SHORT          evt               ; /* Event code dialog FSM              */
    USHORT         cnt               ; /* Loop counter                       */
    UINT           idto              ; /* Running timer identifier           */
  END              Dia_ctx           ; /* ---------------------------------- */

/*
   2.6  Description d'un job - Job/Job_data/Job_dia
   ------------------------------------------------
*/

typedef struct    job                  /* One Job structure descriptor       */
  BEGIN                                /* ---------------------------------- */
    USHORT         mylw              ; /* Associated Logical Way Number      */
                                       /*                                    */
    USHORT         code              ; /* Reqested operation code            */
    USHORT         file              ; /* Target queue + blocking queues     */
    UCHAR         *address           ; /* Buffer address                     */
    INT            length            ; /* Buffer size or length in bytes     */
    UINT           flags             ; /* Flags                              */
    UINT           param             ; /* Maximum duration or job tag        */
                                       /*                                    */
    USHORT         reqaut            ; /* Request automaton number           */
    USHORT         reqlw             ; /* Request logical way number         */
    USHORT         reqnode           ; /* Request originator node            */
    USHORT         reqid             ; /* User request identifier            */
    USHORT         sessid            ; /* User session identifier            */
                                       /*                                    */
    USHORT         run_cnt           ; /* Incremented by "run_job"           */
    struct job    *next              ; /* Next item in the linked queue      */
    struct jobf   *jfree             ; /* Link we were allocated from        */
    struct job    *last              ; /* Pointer to the "context job"       */
  END              Job               ; /* ---------------------------------- */
 
typedef struct                         /* One Job + 40 data bytes            */
  BEGIN                                /* ---------------------------------- */
    Job            job               ; /* One Job structure                  */
    USHORT         data[20]          ; /* User data bytes           40 bytes */
  END              Job_data          ; /* ---------------------------------- */
 
typedef struct                         /* One Job + 26 bytes + One Dia_ctx   */
  BEGIN                                /* ---------------------------------- */
    Job            job               ; /* One Job structure                  */
    USHORT         data[13]          ; /* User data bytes           26 bytes */
    Dia_ctx        dia               ; /* One dialog FSM context    14 bytes */
  END              Job_dia           ; /* ---------------------------------- */
 


/*
   2.7  Tete de chaine de jobs - Hd_joblnk/Hd_jobfree
   --------------------------------------------------
*/

typedef struct                         /* Head of link of Job structures     */
  BEGIN                                /* ---------------------------------- */
    USHORT         ilnk              ; /* This head of link index            */
    struct job    *first             ; /* First link job address             */
    struct job    *last              ; /* Last link job address              */
    USHORT         nb                ; /* Count of linked jobs               */
    USHORT         skip              ; /* Count of following jobs to skip    */
    UINT           idto              ; /* Timer identifier                   */
  END              Hd_joblnk         ; /* ---------------------------------- */


typedef struct   jobf                  /* Tete de Chaine de Job libres       */
  BEGIN                                /* ---------------------------------- */
    struct job    *first             ; /* Ad premier element de la chaine    */
    USHORT         nb                ; /* Nombre d'elements de la chaine     */
    USHORT         sz                ; /* Taille en octets d'une structure   */
  END              Hd_jfree          ; /* ---------------------------------- */

/*
   2.8  Une clef et ses 4 tetes de chaines - Key_lnk
   -------------------------------------------------
*/
typedef struct                         /* Job structure chain                */
  BEGIN                                /* ---------------------------------- */
    UINT           key[4]            ; /* 16 bytes key value                 */
                                       /*                                    */
    Hd_joblnk      inreq             ; /* Jobs input queue                   */
    Hd_joblnk      lnk[4]            ; /* 4 job chain heads                  */
    USHORT         nbsys             ; /* Accumulation of "system" jobs      */
    USHORT         nbusr             ; /* Accumulation of "user" jobs        */
                                       /*                                    */
    Hd_jfree       jfree             ; /* Free jobs head chain               */
                                       /*                                    */
    USHORT         mylw              ; /* Logical channel associated with key*/
    Dia_ctx        dia[4]            ; /* Small context for each chani head  */
                                       /*                                    */
    INT            err               ; /* Last error code value              */
    INT            retlgrcv          ; /* Last number of received bytes      */
                                       /*                                    */
  END              Key_lnk           ; /* ---------------------------------- */



/*
   2.9 Structure de lien avec la couche basse ou a droite
   ------------------------------------------------------
*/
typedef struct                         /* Pour appeler le bas niveau         */
  BEGIN                                /* ---------------------------------- */
    UINT           modd              ; /* Reference VMOS de ce contexte      */
    USHORT         lw0               ; /* REQ_ALLOC Logical way              */
    UINT           iod               ; /* Reference VMOS entite du dessous   */
    USHORT         numdrv            ; /* Numero driver VMIO du dessous      */
    USHORT         aut               ; /* Numero automate VMIO du dessous    */
    USHORT         lw                ; /* Numero VL VMIO du dessous          */
    const Tlcm_desc *tlcm            ; /* Ad structure Tlcm_desc du dessous  */
    Req_ctx       *rctx              ; /* Ad structure Req_ctx du dessous    */
    Key_lnk       *key               ; /* Ad structure Key_lnk du dessous    */
    Req_subctx    *rsctx             ; /* Req_subctx structure address if any*/
    USHORT         state             ; /* Etat du pipe                       */
    SHORT          first             ; /* Tete de chaine de contextes        */
  END              Next_lnk          ; /* ---------------------------------- */


/*
   2.10 Managing slots structure - Slot_ctx
   -----------------------------------------
*/

typedef struct                         /* Pour appeler le bas niveau         */
  BEGIN                                /* ---------------------------------- */
    UINT           btm               ; /* Bitmap of slots currently busy     */
    UCHAR          slotmax           ; /* Max count of slots 1..32           */
    UCHAR          slotbusy          ; /* Count of currently busy slots 0..32*/
  END              Slot_ctx          ; /* ---------------------------------- */


/*
   2.11 Structure buffer circulaire
   --------------------------------
*/

typedef struct                         /* Buffer circulaire pour pipes       */
  BEGIN                                /* ---------------------------------- */
    UCHAR         *baseadr           ; /* Circular buffer base address       */
    UINT           segsz             ; /* Segment size in bytes              */
    UINT           segnb             ; /* Total number of segments           */
    UINT           segwrt            ; /* Number of next segment to write    */
    UINT           segrd             ; /* Number of next segment to be read  */
    UINT           segused           ; /* Number of used segments            */
  END              Circ_buf          ; /* ---------------------------------- */

/*
   2.12 Log for "set_job_state" structure
   --------------------------------------
*/
typedef struct                         /* On entry of "job_state_log[]"      */
  BEGIN                                /* ---------------------------------- */
    INT            num               ; /* Jon structure number               */
    UCHAR          pstate            ; /* Previous job state                 */
    UCHAR          nstate            ; /* New job state                      */
  END              Job_sta_log       ; /* ---------------------------------- */

/*
    2.13 Log for "xyz_evt_end" structure
    ------------------------------------
*/
typedef struct                         /* chk_evt_end log entry              */
  BEGIN                                /* ---------------------------------- */
    UCHAR          par               ; /* "xyz_evt_end" input parameter      */
    UINT           nq                ; /* Selected queue number              */
  END              Evt_end_log       ; /* ---------------------------------- */

/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* Procedures de gestion des tables de contextes ............................*/

  void  init_ctx_tab    (Tab_ctx*,Hd_ctx**,UINT,Hd_ctx*                   ,
                             UINT,UINT,UINT,UINT,UINT,UINT,UINT,INT*         );
  void  alloc_ctx       (Tab_ctx*,INT,Hd_ctx**,INT*                          );
  void  alloc_ctx_end   (Hd_ctx**,INT*                                       );
  void  post_ctx        (Hd_ctx*,INT*                                        );
  void  free_ctx        (Hd_ctx*,INT*                                        );
  void  free_ctx_end    (Hd_ctx**,INT*                                       );
  IWORD set_keylnk      (Hd_ctx*,UCHAR*                                      );
  IWORD reset_keylnk    (Hd_ctx*                                             );
  IWORD search_keylnk   (Tab_ctx*,UCHAR*,Hd_ctx**                            );
  IWORD get_ctx_adr     (Tab_ctx*,INT,Hd_ctx**                               );
  IWORD get_subctx_adr  (Hd_ctx*,Req_ctx**,Key_lnk**                         );


/* Procedures de gestion des contextes de requetes ..........................*/

  void  init_rctx       (Req_ctx*,INT,INT,INT,INT,INT,INT                 ,
                         IWORD(*)(),IWORD(*)()                               );
  IWORD chk_rctx        (Req_ctx*,Chk_req_ctx*                               );
  void  init_rsctx      (Req_subctx*,INT,INT,INT                             );


/* Procedures d'allocation et de liberation des structures "Job" ............*/

  void  init_job        (void                                                );
  void  init_jfree      (Hd_jfree*,Job*,INT,INT                              );
  void  alloc_job       (Hd_jfree*,INT,Job**                                 );
  void  free_job        (Job*                                                );
  IWORD init_slot       (Slot_ctx*,INT                                       );
  IWORD alloc_slot      (Slot_ctx*                                           );
  IWORD free_slot       (Slot_ctx*,INT                                       );
  IWORD alloc_bitid     (UINT*                                               );
  IWORD free_bitid      (UINT*,INT                                           );
  void  job_name        (Job*,CHAR*,INT*                                     );


/* Procedures de chainage ou dechainage de structures "Job" .................*/

  void  link_job        (Key_lnk*,Hd_joblnk*,Job*                            );
  IWORD unlink_job      (Key_lnk*,Hd_joblnk*,Job*                            );
  void  unlink_first_job(Key_lnk*,Hd_joblnk*,INT,Job**                       );
  void  unlink_last_job (Key_lnk*,Hd_joblnk*,Job*                            );
  IWORD test_job_head   (Hd_joblnk*,UINT,UINT                                );
  IWORD test_job_link   (Hd_joblnk*,UINT,UINT,Job**                          );
  IWORD set_bits_jlink  (Hd_joblnk*,UINT,UINT                                );
  IWORD flag_job_link   (Hd_joblnk*,UINT,UINT,UINT,UINT                      );


/* Procedures generique d'ajout de jobs dans la file d'entrees ..............*/

  void  link_reqevt     (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT,INT,UCHAR*,
                         INT, INT, INT,INT,INT,INT,INT,INT*                  );
  void  link_snd        (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT,INT,INT,
                         UCHAR*, INT,INT,void (*)(),INT,INT,INT,INT*         );
  void  link_rcv        (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT,INT,INT,
                         UCHAR*, INT,INT,void (*)(),INT,INT,INT,INT*         );
  void  link_wait       (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT*              );
  void  link_prcv       (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT*               );


/* Procedures de demarrage et de terminaison de jobs ........................*/

  void  inreq_job       (Req_ctx*, Key_lnk*                                  );
  void  evtend_job      (Req_ctx*,Key_lnk*,S_evt*,INT,INT,INT                );
  void  push_job        (Key_lnk*                                            );
  void  pull_job        (Key_lnk*,Hd_joblnk*,Job*                            );
  IWORD ready_job       (Key_lnk*,Hd_joblnk**                                );
  IWORD run_job         (Req_ctx*,Key_lnk*,Hd_joblnk*,Job*                   );
  void  set_job_state   (Job*,INT                                            );
  IWORD resp_heap_job   (Req_ctx*,Key_lnk*,Hd_joblnk*,INT                    );
  IWORD flush_jobs      (Req_ctx*,Key_lnk*,INT,INT                           );


/* Procedures d'interface systeme par defaut ................................*/

  IWORD xxx_alloc_sub   (Req_ctx*,UCHAR*,INT,UINT,INT,INT,INT                );
  IWORD xxx_free_sub    (Req_ctx*,UINT,INT,INT,INT                           );
  IWORD xxx_connect     (Req_ctx*,Key_lnk*,INT,INT,UCHAR*,INT,INT,INT,INT,INT);
  IWORD xxx_listen      (Req_ctx*,Key_lnk*,INT,INT,UCHAR*,INT,INT,INT,INT,INT);
  IWORD xxx_disconnect  (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT,INT           );
  IWORD xxx_abort       (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT               );
  IWORD xxx_start       (Req_ctx*,Key_lnk*,INT,UCHAR*,INT,INT,INT,INT,INT    );
  IWORD xxx_stop        (Req_ctx*,Key_lnk*,INT,INT,INT,INT,INT               );
  IWORD xxx_send        (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT,INT,INT,
                         UCHAR*,INT,INT,INT, void(*)(), INT,INT,INT          );
  IWORD xxx_receive     (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT,INT,UCHAR*,
                         INT,INT,INT,INT,INT                                 );
  IWORD xxx_signal      (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT,INT,INT,
                         INT,INT,INT                                         );
  IWORD xxx_setval      (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT,UCHAR*,INT,
                         INT,INT,INT                                         );
  IWORD xxx_getval      (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT,UCHAR*,INT,
                         INT,INT,INT                                         );
  IWORD xxx_rpc         (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT,UCHAR*,
                         INT,INT,INT,INT,INT,INT                             );
  IWORD xxx_io          (Req_ctx*,Key_lnk*,Job*,INT                          );
  IWORD xxx_alloc_buf   (UCHAR*,INT,INT,UINT*,UCHAR**                        );
  IWORD xxx_free_buf    (UCHAR*,INT,INT,UINT*,UCHAR*                         );
  IWORD xxx_chk_buf     (UCHAR*,INT,INT,UINT*,UCHAR*                         );
  IWORD xxx_get_free_cnt(UCHAR*,INT,INT,UINT*                                );
  IWORD vvv_write       (Req_ctx*,Req_subctx*,Key_lnk*,
                         INT,INT,INT,UCHAR*,INT,INT                          );
  IWORD vvv_read        (Req_ctx*,Req_subctx*,Key_lnk*,
                         INT,INT,INT,UCHAR*,INT,INT*                         );


/* Procedures de traitements d'automates VMIO generiques ....................*/

  IWORD x_init          (INT                                                 );
  IWORD x_end           (void                                                );
  IWORD x_alloc_dev     (Req_ctx*,INT,INT,INT,IWORD(*)(),const USHORT*,INT*  );
  IWORD x_free_dev      (Req_ctx*,INT,INT,INT*                               );
  IWORD x_open_dev      (Req_ctx*,void (*)(),UINT                            );
  IWORD x_close_dev     (Req_ctx*,UINT                                       );
  IWORD x_alloc_sub     (Req_subctx*,INT,INT,INT,INT,
                         const void*,void(*)(),INT,INT,INT*                  );
  IWORD x_free_sub      (Req_subctx*,INT,INT,INT*                            );
  IWORD x_connect       (Req_ctx*,Key_lnk*,INT,INT                           );
  IWORD x_listen        (Req_ctx*,Key_lnk*,INT,INT                           );
  IWORD x_disconnect    (Req_ctx*,Key_lnk*,INT,INT                           );
  IWORD x_abort         (Req_ctx*,Key_lnk*,INT,INT                           );
  IWORD x_write         (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT,INT           );
  IWORD x_read          (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT,INT           );
  IWORD x_signal        (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT               );
  IWORD x_setval        (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT               );
  IWORD x_getval        (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT               );
  IWORD x_rpc           (Req_ctx*,Req_subctx*,Key_lnk*,INT,INT               );
  IWORD x_evtend        (Req_ctx*,Key_lnk*,INT,INT,INT                       );
  IWORD x_pstart        (Req_ctx*,Key_lnk*,INT,Circ_buf*,Next_lnk*           );
  IWORD x_rwrite        (Req_ctx*,Key_lnk*,INT,Circ_buf*                     );
  IWORD x_req_clear     (Req_ctx*,Key_lnk*,INT                               );


/* Procedures sous-programmes de depot d'evenements .........................*/
 
  void  evt_resperror   (Req_ctx*,Req_subctx*,INT,INT,UCHAR*,INT,INT         );
  void  evt_respjob     (Req_ctx*,Job*,INT,INT,UCHAR*,INT                    );
  void  evt_ioend       (Req_ctx*,Key_lnk*,Job*,INT                          );
  void  evt_resprctx    (Req_ctx*,INT,INT,UCHAR*,INT,UINT,UINT               );
  void  evt_respany_job (Req_ctx*,Req_subctx*,Key_lnk*,Job*,INT,INT          );


/* Procedures d'interface systeme avec la couche du bas ou a droite .........*/

  void  init_next_lnk   (Next_lnk*,UINT,INT,UINT,INT,INT                     );
  IWORD connect_next    (Next_lnk*,UCHAR*,INT,INT,INT,INT,INT                );
  IWORD listen_next     (Next_lnk*,UCHAR*,INT,INT,INT,INT,INT                );
  IWORD disconnect_next (Next_lnk*,INT,INT,INT,INT                           );
  IWORD abort_next      (Next_lnk*,INT,INT,INT                               );
  IWORD start_next      (Next_lnk*,UCHAR*,INT,INT,INT,INT,INT                );
  IWORD stop_next       (Next_lnk*,INT,INT,INT,INT                           );
  IWORD send_next       (Next_lnk*,UCHAR*,INT,INT,void(*)(),INT,INT,INT      );
  IWORD receive_next    (Next_lnk*,UCHAR*,INT,INT,INT,INT,INT                );

  IWORD setval_next     (Next_lnk*,UCHAR*,INT,INT,INT,INT                    );
  IWORD getval_next     (Next_lnk*,UCHAR*,INT,INT,INT,INT                    );
  IWORD rpc_next        (Next_lnk*,UCHAR*,INT,INT,INT,INT,INT,INT            );
  IWORD io_next         (Next_lnk*,Job*,INT,INT                              );
  IWORD get_bufsize_next(Next_lnk*,INT,INT,INT*,INT*,INT*                    );
  IWORD aut2drv         (INT,INT                                             );
  IWORD drv2aut         (INT                                                 );
  void  search_name2    (CHAR*,CHAR*,INT*                                    );

/* Trace functions ..........................................................*/

  void  sto_evt_end_log (INT,INT                                             );

#endif                                 /* End of REQ_LIB_H.                  */


/*  5  RAM data to be inclueded in space_iocont_xyz.c
    _________________________________________________
*/

#ifdef SPACE_IOCONT_XYZ_C

Job_data       staticjob[NBJOB]      ; /* Pool of static Job_data structs    */
const INT      nbjob     = NBJOB     ; /* Size of the "staticjob[]" array    */

#endif
