/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/vmio.h"                */
/*  Constants and definitions for VMIO's.                                    */
/*                                                                           */
/*  Author : JYA/FA                     Last modification : 03/07/2024       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Note_evt_vmio structure field values
            1.2  Error codes
            1.3  Durations table sizes

         2  Structures and typedefs
            2.1  VMIO automaton description structure - Vmio_desc
            2.2  Time-out structure for VMIO - S_to_vmio
            2.3  VMIO queue head of link
            2.4  Debug information structures - Err_evt_vmio
            2.5  Debug information structures - Note_evt_vmio
            2.6  Debug information structures - Dura_fa_vmio
            2.7  Debug information structures - Dura_trt_vmio

         3  Public variables
            3.1. Defined in space_iocont.c  
            3.2  Defined in vmio.c 

         4  Prototypes of public procedures
             4.1  Initializations and Main Loop
             4.2  Writing and Sending Events, Conditions
             4.3  Timers
             4.4  Automaton Wait Queues
             4.5  Event Routes
             4.6  Miscellaneous Functions

         5  RAM data to be inclueded in space_iocont_xyz.c
            5.1  Tables for "vmio.c"
            5.2  Table sizes as const variables
            5.3  Ofenly used array indexes
            5.4  Debug tables
            5.5  Times measurements

*/


#ifndef VMIO_H                         /* If this file is not already        */
#define VMIO_H                         /* included, include it.              */


/*
    1    Defines
    ____________
*/


/* 1.1  Note_evt_vmio structure field values
   -----------------------------------------
*/

/* Values for the "code" field ..............................................*/

#define ADD_EVT_VMIO 0x01              /* ecr_evt_vmio                       */
#define DEL_EVT_VMIO 0x10              /* free_evt_vmio                      */
#define BEG_TRT_VMIO 0x20              /* vmio, before calling "run_fsm_vmio"*/
#define ACT_TRT_VMIO 0x21              /* run_fsm_vmio, before calling treat */


/* 1.2  Error codes
   ----------------
*/

#define BAD_PARAMETER       -1341      /* Parameter error                    */
#define ROUTE_UNKNOWN       -1410      /* Bad route id                       */
#define EFAFULL             -13121     /* File d'attente pleine              */
#define EFAEFULL            -13118     /* File FAE pleine                    */
#define ERREVT              -13116     /* Case vide dans table de trans      */
#define FINCHAINE           -13115     /* Fin de chaine des TO non prevue    */
#define EITCURRPRIO         -13117     /* "itcurrprio" innatendu             */
#define ROUTE_EXIST         -1411      /* Route already declared             */

#define TOO_MANY_EVTROUTE   -1184      /* Too many event route               */
#define TTOPLEINE           -13114     /* Table des TO pleine                */
#define TUNNODE             -13119     /* putevt_tun - node est incorrect    */
#define TUNNOTOPEN          -13120     /* putevt_tun - pas de tunnel         */


/* 1.3  Durations table sizes
   --------------------------
*/

#define MAXDUR_VMIO  5                 /*                                    */
#define MAXDUR_IT    3                 /*                                    */




/*
    2    Structures and typedefs
    ____________________________
*/

/* 2.1  VMIO automaton description structure - Vmio_desc
   -----------------------------------------------------
*/

typedef struct                         /* An entry in the TIMER table        */
  BEGIN                                /* ---------------------------------- */
    CHAR          *name              ; /* Automaton name                     */
    UCHAR         *autnum            ; /* Address of UCHAR automaton number  */
    S_trans const *atrans            ; /* Transition table address           */
    USHORT  const *alim              ; /* Trans table states indexes         */
    USHORT        *astate            ; /* State variables array address      */
    USHORT  const *anblw             ; /* Count of logicam ways address      */
  END             Vmio_desc          ; /* ---------------------------------- */


/* 2.2  Time-out structure for VMIO - S_to_vmio
   --------------------------------------------

   The table of current timers of the VMIO is an array of "S_to_vmio" 
   structures.  There is one "S_to_vmio" structure per time-out.
*/

typedef struct                         /* An entry in the TIMER table        */
  BEGIN                                /* ---------------------------------- */
    UINT        ident                ; /* Timer identifier                   */
    USHORT      aut                  ; /* Requesting automaton number        */
    USHORT      lw                   ; /* Requesting logical way number      */
    USHORT      code                 ; /* Event code sent on timeout         */
    UINT        reserve              ; /* Notification event reserve field   */
    USHORT      link                 ; /* Pointer to the next running timer  */
    INT         date                 ; /* Expiration date                    */
  END           S_to_vmio            ; /* ---------------------------------- */


/* 2.3  VMIO queue head of link
   ----------------------------
*/

typedef struct                         /* VMIO queue head of link            */
  BEGIN                                /* ---------------------------------- */
    USHORT      nbevt                ; /* Count of queued events             */
    USHORT      first                ; /* First event index                  */
    USHORT      last                 ; /* Last  event index                  */
  END           Fah_vmio             ; /* ---------------------------------- */


/* 2.3  Debug information structures - Err_evt_vmio
   ------------------------------------------------
*/

typedef struct                         /* Informations written by "err_vmio" */
  BEGIN                                /* ---------------------------------- */
    INT         err                  ; /* FAPLEINE, EERNEVT, ...             */
    USHORT      naut                 ; /* Current automaton number           */
    USHORT      state                ; /* Current logical way state          */
    USHORT      nlw                  ; /* Current logical way number         */
    USHORT      code                 ; /* Current event code                 */
    INT         reserve              ; /* Current event reserve field value  */
  END           Err_evt_vmio         ; /* ---------------------------------- */



/* 2.4  Debug information structures - Note_evt_vmio
   -------------------------------------------------
*/

typedef struct                         /* Informations written by "note_vmio"*/
  BEGIN                                /* ---------------------------------- */
    INT         temps                ; /* Call time                          */
    INT         code                 ; /* EVT_ACT_S,EVT_ADD_S,...            */
    INT         n                    ; /* Queue number                       */
    INT         nbevt                ; /* Count of queued events             */
    INT         param4               ; /* Code dependant parameter value     */
    INT         param5               ; /* Code dependant parameter value     */
    INT         param6               ; /* Code dependant parameter value     */
    INT         param7               ; /* Code dependant parameter value     */
  END           Note_evt_vmio        ; /* ---------------------------------- */



/* 2.5  Debug information structures - Dura_fa_vmio
   ------------------------------------------------
*/

typedef struct                         /* Event queued time measurment       */
  BEGIN                                /* ---------------------------------- */
    UINT        dura                 ; /* Duration spent in the queue        */
    USHORT      aut                  ; /* Event "fsm"  field                 */
    USHORT      code                 ; /* Event "code" field                 */
  END           Dura_fa_vmio         ; /* ---------------------------------- */



/* 2.6  Debug information structures - Dura_trt_vmio
   -------------------------------------------------
*/

typedef struct                         /* Measurement of treatment duration  */
  BEGIN                                /* ---------------------------------- */
    UINT        dura                 ; /* Duration of the treatment          */
    USHORT      code                 ; /* Code of the cause event            */
    IWORD       (*proc)()            ; /* C procedure address                */
    INT         par                  ; /* Procedure input parameter          */
    UINT        cpt                  ; /* Counter                            */
  END           Dura_trt_vmio        ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*  3.1  Defined in space_iocont.c  
    -------------------------------
*/

/* Constant variables .......................................................*/

extern const INT   fan_idx           ; /* Wait queue - FAN index for normal  */
extern const INT   fau_idx           ; /* Wait queue - FAU index for urgent  */
extern const INT   fal_idx           ; /* Wait queue - FAL index for slow    */
extern const INT   aut_vmio_last     ; /* Last VMIO automaton number         */


/*  3.2  Defined in vmio.c 
    -----------------------
*/

/* Current event handling ...................................................*/

extern S_evt       c_evt             ; /* Current event descripion           */
extern USHORT      naut_vmio         ; /* Current corresponding FSM          */
extern USHORT      nlw_vmio          ; /* Current corresponding VL           */
extern USHORT      condition_vmio    ; /* <> 0 if condition setted           */

extern UINT        cnt_putevt_vmio   ; /* Calls to "putevt_vmio"             */
extern UINT        cnt_vmio_trans    ; /* Counts of VMIO transitions         */

/*
    4    Prototypes of public procedures
    ____________________________________
*/


/* 4.1  Initializations and Main Loop
   ----------------------------------
*/

IWORD init_vmio           (IWORD                                             );
IWORD end_vmio            (IWORD                                             );
IWORD vmio                (IWORD,IWORD                                       );
IWORD vmio_trigto         (IWORD,IWORD                                       );


/* 4.2  Writing and Sending Events, Conditions
   -------------------------------------------
*/

IWORD putevt_vmio         (S_evt*                                            );
IWORD putevt_vmio_urg     (S_evt*                                            );
IWORD send_vmio           (S_evt*                                            );
IWORD clrevt_vmio         (INT,UINT,UINT,UINT, IWORD (*)()                   );
void  relfsm_vmio         (UINT                                              );


/* 4.3  Timers
   -----------
*/

UINT  setto_vmio          (UINT,UINT,INT,IWORD,IWORD                         );
IWORD clearto_vmio        (INT,UINT                                          );
IWORD clear_fsmlwto_vmio  (INT,INT                                           );


/* 4.4  Automaton Wait Queues
   --------------------------
*/
IWORD stoevt_vmio         (S_evt*                                            );
IWORD getstoevt_vmio      (IWORD,IWORD*,IWORD,IWORD,S_evt*                   );
IWORD clrstoevt_vmio      (IWORD,IWORD (*)()                                 );


/* 4.5  Event Routes
   -----------------
*/

IWORD add_route_vmio      (INT,INT,INT,INT,INT,INT,UINT,UINT,INT,UINT*       );
IWORD del_route_vmio      (UINT*,INT, IWORD (*)()                            );
IWORD set_route_vmio      (UINT*,INT,INT,INT                                 );
IWORD route_evt_vmio      (S_evt*                                            );
IWORD del_evt_route_vmio  (INT,IWORD (*)()                                   );


/* 4.6  Miscellaneous Functions
   ----------------------------
*/

static IWORD tnull_vmio   (INT                                               );
IWORD get_autlw           (INT*,INT*                                         );
IWORD send_resp           (INT,INT,UCHAR*,INT,UINT,UINT                      );
void  set_respinfo        (Resp_info*                                        );
void  reset_respinfo      (Resp_info*                                        );
IWORD send_respinfo       (INT,INT,UCHAR*,INT,UINT,UINT,Resp_info*           );
void  get_vmio_stats      (UINT*,UINT*                                       );
void  clear_vmio_stats    (void                                              );
void  set_putevt_vmio_err (void (*)(),INT,INT                                );
void  reset_table_vmio    (INT,INT,INT,INT                                   );
IWORD alloc_fa_vmio       (IWORD*                                            );
void  free_fa_vmio        (IWORD                                             );
void  link_fa_vmio        (IWORD,IWORD                                       );


#endif                                 /* End of VMIO_H                      */


/*  5  RAM data to be inclueded in space_iocont_xyz.c
    _________________________________________________
*/

#ifdef SPACE_IOCONT_XYZ_C

#define FAL      (MAXAUT_VMIO     )    /* Waiting queue                      */
#define FAN      (MAXAUT_VMIO + 1 )    /* Waiting queue                      */
#define FAU      (MAXAUT_VMIO + 2 )    /* Waiting queue                      */
#define FFREE    (MAXAUT_VMIO + 3 )    /* Waiting queue                      */

#define AUT_VMIO_LAST (AUT_VMIO_FIRST+MAXAUT_VMIO-1)

/* 5.1  Tables for "vmio.c"
   ------------------------
*/

S_evt     fa_vmio      [MAXEVT_VMIO]  ;/* Storage for all waiting queues     */
Fah_vmio  fah_vmio     [MAXAUT_VMIO+4];/* Heads of waiting queus             */
S_to_vmio tabto_vmio   [NTOMAX_VMIO]  ;/* Time-out table                     */
Route     route_vmio   [MAXRTE_VMIO]  ;/* Route table                        */


/* 5.2  Table sizes as const variables
        ------------------------------
*/

const INT maxaut_vmio = MAXAUT_VMIO  ; /* "vmio_desc" size                   */
const INT maxevt_vmio = MAXEVT_VMIO  ; /* "fa_vmio" size                     */
const INT ntomax_vmio = NTOMAX_VMIO  ; /* "tabto_vmio" size                  */
const INT maxrte_vmio = MAXRTE_VMIO  ; /* "route_vmio" dize                  */


/* 5.3  Ofenly used array indexes
        -------------------------
*/

const INT fal_idx = FAL           ; /* Indice de la FAL   = maxfsm_vmio + 0  */
const INT fan_idx = FAN           ; /* Indice de la FAN   = maxfsm_vmio + 1  */
const INT fau_idx = FAU           ; /* Indice de la FAU   = maxfsm_vmio + 2  */
const INT faf_idx = FFREE         ; /* Indice de la FFREE = maxfsm_vmio + 3  */

const INT aut_vmio_last = (AUT_VMIO_FIRST + MAXAUT_VMIO -1) ;


/* 5.4  Debug tables
   -----------------
*/

Note_evt_vmio tnote_evt_vmio[MAX_NOTE_EVT]; /* Event trace circular buffer   */
Err_evt_vmio  terr_evt_vmio [MAX_ERR_EVT] ; /* Error trace circular buffer   */

const INT max_note_evt_vmio = MAX_NOTE_EVT; /* tnote_evt_vmio[] table size   */
const INT max_trace_evt_vmio= MAX_ERR_EVT ; /* terr_evt_vmio[] table size    */


/* 5.5  Times measurements
   -----------------------
*/

Dura_fa_vmio  tdura_fa_vmio[MAXAUT_VMIO+3][MAXDUR_VMIO]; /* Dur in wait queue*/
Dura_trt_vmio tdura_trt_vmio[MAXAUT_VMIO][MAXDUR_VMIO] ; /* C treatments     */
Dura_crit_it   tdura_crit_it[MAXDUR_IT]                ; /* Critical sections*/

const INT      tdura_fa_vmio_sze  = sizeof(tdura_fa_vmio) ;
const INT      tdura_trt_vmio_sze = sizeof(tdura_trt_vmio);

#endif
