/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS - VMK                 File "os/vmk/lib/tsk_fsmlib.c"       */
/*  VMK task FSM library                                                     */
/*                                                                           */
/*  Author : JYA                        Last modification : 10/28/2022       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================



    1    Purpose of this module
    ___________________________


         This module is a VMK library intended to be use by VMK tasks. Il
allows to create FSM with treatments which can be unscheduled, allowing to
create a high level FSM without the burden to use I/O ends wait states whenever
a treatement requests an I/O.
   
    Some rules for on-line documentation generation :

    #PRIVATE   ... #END     Not generated in on-line documentation.
    #DIAGRAM   ... #END     Diagram, drawing, schematic.
    #CODE      ... #END     Source code example (ex: typedef).
    #ARRAY     ... #END     Array (ex: explanation following typedef).
    #INDEX     ... #END     Sub-chapters list, procedure list.
    #PCODE     ... #END     Pseudo-code.
    #PROCDESCR ... #END     Procedure interface description.

#PRIVATE ----------------------------------------------------------------------

NOTES ========================================================================

CODE =========================================================================

DOC ==========================================================================

TESTS ========================================================================

EVOLUTIONS ===================================================================


#END --------------------------------------------------------------------------



    2    User manual
    ________________


         2.1  xxxxxxx
              2.1.1  xxxxxx

    2.1  xxxxxxx
         Bla bla

    3    Porting guide
    __________________


         3.1  xxxxxxx
              3.1.1  xxxxxx

    3.1  xxxxxxx
         Bla bla


    4    Include files and external references
    __________________________________________
*/


#include <stdtyp.l>                    /* Hyperpanel definitions & constants */
#include <aut.h>                       /* S_evt                              */
#include <tsk_fsmlib.h>                /* Conv_evt                           */

#include <systemm.txt>                 /* DEFAULT                            */

#ifdef TEST                            /* For unitary tests.                 */
#endif                                 /* End of ifdef TEST.                 */


/* External reference from ..................................................*/


/*
    5    Inner workings of the module
    _________________________________


         5.1  xxxxx
              5.1.1  xxxxx

         5.2  xxxxx
              5.2.1  xxxx


    6    Exported (public) variables
    ________________________________
*/

/*
    7    Internal constants of the module
    _____________________________________
*/

/*
    8    Internal data types of the module
    ______________________________________
*/

/*
    9    Internal (private) variables of the module
    _______________________________________________
*/

INT      fsm_state                   ; /* FSM state variable                 */
INT      fsm_evt                     ; /* FSM event code                     */
INT      fsm_cond                    ; /* FSM condition code value           */
INT      fsm_condret                 ; /* FSM return condition code value    */
INT      fsm_err                     ; /* FSM global error code              */

/*
    10   Beginning of the code
    __________________________


------------------  FSM main loop -------------------------------------------
fsm_conv_evt        Get a task FSM event code from a S_evt structure
fsm_run             Runs the FSM for one incoming event

------------------  FSM event treatements -----------------------------------
empt                Do nothing
next                Update the FSM state variable
sret                Set the "fsm_condret" global variable with "n"
scon                Set "fsm_cond" with "fsm_condret"

*/


/*  Procedure fsm_conv_evt --------------------------------------------------*/
/*
    Purpose : Get a TASK FSM event code from a S_evt structure.

    Author : JYA                                 Last modification : 10/18/2022

...............................................................................

    Calling example : fsm_conv_evt( evt, tconv )

    Input     : evt      S_evt*     Incoming event address
                tconv    Conv_evt*  Table of conversion functions

    Output    : valproc  IWORD      Converted event code or -1

    Notes : - If the "iod" address of a "Conv_evt" entry is the HNULL address
              then we have a match with any "evt->reserve" value.

...............................................................................
*/

IWORD  PROCEDURE fsm_conv_evt(S_evt *evt, const Conv_evt *tconv)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    USHORT         code              ; /* Copy of "evt->code"                */
    INT            reserve           ; /* Copy of "evt->reserve"             */
    const Conv_evt *p                ; /* Address in the "evttrt" table      */
    INT            *iodad            ; /* Value of "p->iod"                  */
    INT            iod               ; /* Value of "*p->iod"                 */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("fsm_conv_evt")          ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Initialization of local variables:                              *
 * -------   - We set the "r" return event code value to the -1 empty value  *
 *           - Copy of the "evt->code" field,  event code                    *
 *           - Copy of the "evt->reserve" field, the involved IOD            *
 *****************************************************************************/

    r       = -1                     ; /* Count of called functions          */
    code    = evt->code              ; /* event code                         */
    reserve = evt->reserve           ; /* reserve is an IOD                  */


/*****************************************************************************
 * Step  2 : Loop on the entries of the "tconv[]" table, using "p" as the    *
 * -------   current entry pointer. The loop stops when "p->code" is equal   *
 *           to the 0xFFFF end of table marker. At each step loop we :       *
 *           - Retreive the IOD associated to current entry. The "p->iod"    *
 *             field is the address of an INT global variable which holds    *
 *             the IOD, so we set "iod" to "*p->iod"                         *
 *           - We check if we have a match between the event and the entry,  *
 *             if "p->codeevt" is equal to "code" (evt->code) and if         *
 *             "iod" is equal to "reserve" (evt->reserve) then we have a     *
 *             match.                                                        *
 *           - If we do not have a match, we simply continue with next loop  *
 *             step                                                          *
 *           - If we have a match, we call the "p->conv" function, with      *
 *             "evt" and "p->param" as a parameters. This function may       *
 *             return us a positive or value, in such a case this is the FSM *
 *             event code, we are happy and we stop looping. If the function *
 *             returns us any negative value, we continue our loop           *
 *           --------------------------------------------------------------- *
 *           We have a special case to handle: If the "iod" field of the     *
 *           "p" entry is HNULL, this means that any value for the incoming  *
 *           event "reserve" field is accepted. In such a case, we simply    *
 *           set "iod" to "reserve" and so the test "iod NE reserve" will    *
 *           be FALSE.                                                       *
 *****************************************************************************/

    for ( p = tconv            ;       /* We loop on the entries of the      */
          p->codeevt NE 0xFFFF ;       /* "tconv" table, we stop when the    */
          p++                   )      /* 0xFFFF end of table marker is      */
      BEGIN                            /* encountered                        */
        iodad = p->iod               ; /* Get the address of the INT which   */
                                       /* holds the "iod" value              */
        if ( iodad )                   /* If the "p->iod" address is <>HNULL */
          iod = *iodad               ; /* get the IOD                        */
        else                           /* If "p->iod" is HNULL, we set       */
          iod = reserve              ; /* "iod" to "reserve", so they have   */
                                       /* one same value                     */
        if (p->codeevt NE code LOGOR   /* If there is no match between the   */
            iod        NE reserve    ) /* table entry and the event, we      */
          continue                   ; /* continue with next loop step       */
                                       /*                                    */
        r = p->conv(evt,               /* We have a match of "code" and      */
                    p->param)        ; /* "reserve", call the user           */
                                       /* conversion function                */
        if (r GE 0) break            ; /* If we got a converted code, we     */
                                       /* stop looping                       */
      END_FOR                          /*                                    */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("fsm_conv_evt")          ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure fsm_run -------------------------------------------------------*/
/*
    Purpose : Treat one UI incoming event. We call 5 C functions provided by
              the current page transition table then we update if necessary
              the "ui_page" global variable

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : fsm_run( state, evt, trans )

    Input     : ui_page  INT        Current displayed page number

    Output    : ui_page  INT        May have been updated

    Notes : 

...............................................................................
*/

void   PROCEDURE fsm_run( const M_trans * const trans[])

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* Return code of the procedure       */
    const M_trans  *tra              ; /* UI transition table entry address  */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("fsm_run")               ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Search into the "trans[*state]" transition table the row        *
 * -------   corresponding to the "evt" incoming UI event code. We remind    *
 *           that "trans[]" is a list of lists. Each state is associated     *
 *           with a list of M_trans structures (A one dimension array of     *
 *           M_trans const structures) and the "trans[]" list holds all the  *
 *           addresses of those arrays. The "state" variable is intended     *
 *           to be an index in "trans[]"                                     *
 *****************************************************************************/

    loop                             : /* UI condition loop                  */
                                       /*                                    */
    for (tra = trans[fsm_state]   ;    /* Search for "evt" in the            */
         tra->evtcode NE fsm_evt &&    /* transition table which starts at   */
         tra->evtcode NE DEFAULT  ;    /* "trans[fsm_states]"                */
         tra ++                    ) ; /*                                    */

/*****************************************************************************
 * Step  2 : Call the 5 C functions held by the M_trans structure.           *
 * -------   Each time a UI treatment is executed at step 2, it may set the  *
 *           "fsm_cond" global variable in order to start a new loop step:   *
 *           - Before calling the functions, we clear "fsm_cond", which      *
 *             means "no condition"                                          *
 *           - After the 5 calls, we test the "fsm_cond" global variable,    *
 *             if we detect a non 0 value then we set the "evt" incoming     *
 *             event code to "fsm_cond" and we jump to the loop beginning    *
 *****************************************************************************/

    fsm_cond = 0                     ; /* Reset the FSM condition code       */
                                       /*                                    */
    if( (r = tra->trt1(tra->p1)) )     /* Execute p1                         */
      goto error                     ; /* and stop if we have an error       */
                                       /*                                    */
    if( (r = tra->trt2(tra->p2)) )     /* Execute p2                         */
      goto error                     ; /* and stop if we have an error       */
                                       /*                                    */
    if( (r = tra->trt3(tra->p3)) )     /* Execute p3                         */
      goto error                     ; /* and stop if we have an error       */
                                       /*                                    */
    if( (r = tra->trt4(tra->p4)) )     /* Execute p4                         */
      goto error                     ; /* and stop if we have an error       */
                                       /*                                    */
    if( (r = tra->trt5(tra->p5)) )     /* Execute p5                         */
      goto error                     ; /* and stop if we have an error       */
                                       /*                                    */
    if ( fsm_cond )                    /* If any of the treatment as set a   */
      BEGIN                            /* non 0 value in the "fsm_cond"      */
        fsm_evt = fsm_cond           ; /* global variable, this is our FSM   */
        goto loop                    ; /* condition, the condition becomes   */
      END_IF                           /* the new event code and we do       */
                                       /* another transition                 */
    goto exit                        ; /* We are done and successful         */

/*****************************************************************************
 * Step  3 : If a treatement returned an error, if "r" is not 0, then we     *
 * --------  set the "fsm_err" global variable the the "r" error code, we    *
 *           set the "evt" incoming event to FSM_ERROR and we do another     *
 *           loop step to treat this new event code                          *
 *****************************************************************************/

    error                            : /* In case a treatment returned any   */
    fsm_err  = r                     ; /* error code, we store it in the     */
    fsm_evt  = FSM_ERROR             ; /* "fsm_err" global variable, we set  */
    goto loop                        ; /* a new FSM_ERROR event code and we  */
                                       /* do another transition with         */
                                       /* FSM_ERROR                          */

/*****************************************************************************
 * Step  4 : Procedure exit label. We are done, we exit                      *
 * -------                                                                   *
 *****************************************************************************/

    exit                             : /* Procedure exit label               */
    ;

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("fsm_run")               ; /* Message                            */
#endif                                 /* End   conditional compilation      */

  END_PROC                             /* End of the procedure               */


/*  Procedure empt ----------------------------------------------------------*/
/*
    Purpose : UI treatment which does nothing

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : empt( n )

    Input     : n        int        Unused

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE empt(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("empt")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */


#ifdef TEST                            /* Begin conditional compilation      */
    procfin("empt")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure next ----------------------------------------------------------*/
/*
    Purpose : UI treatment which updates the FSM state variable

    Author : JYA                                 Last modification : 10/28/2022

...............................................................................

    Calling example : empt( state )

    Input     : state    int        New state

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE next(int state)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("next")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    fsm_state = state                ;

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("next")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */



/*  Procedure sret ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Affects parameter to "fsm_condret".

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : 

    Input     : x        IWORD      C

    Output    : 

    Notes : - After the "return" condition has been saved to "fsm_condret",
              the "scon" treatement will set "fsm_cond" to "fsm_condret"

...............................................................................
*/

IWORD  PROCEDURE sret(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("sret")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    fsm_condret = n                  ;

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("sret")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure scon ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Affects "fsm_condret" or a constant to 
              "fsm_condcode".

    Author : JYA                                 Last modification : 11/16/2022

...............................................................................

    Calling example : scon ( n )

    Input     : n        int        0:Affect "fsm_condret" else affect "n"

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE scon(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("scon")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  x : .                                                               *
 * -------   .                                                               *
 *           .                                                               *
 *****************************************************************************/

    fsm_cond = n          ?
               n          :
               fsm_condret           ;

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("scon")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


