/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS -                     File "os/vmk/lib/tsk_ttylib.c"       */
/*  VMK simple UI library                                                    */
/*                                                                           */
/*  Author : JYA                        Last modification : 06/08/2023       */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================



    1    Purpose of this module
    ___________________________

         This module provides to applicative software a simple UI management
library for TTY like displays. An UI is a set of predefined screen. Each
screen can receive events and react according to a screen associated
transition table.

         Bla Bla
   
    Some rules for on-line documentation generation :

    #PRIVATE   ... #END     Not generated in on-line documentation.
    #DIAGRAM   ... #END     Diagram, drawing, schematic.
    #CODE      ... #END     Source code example (ex: typedef).
    #ARRAY     ... #END     Array (ex: explanation following typedef).
    #INDEX     ... #END     Sub-chapters list, procedure list.
    #PCODE     ... #END     Pseudo-code.
    #PROCDESCR ... #END     Procedure interface description.

#PRIVATE ----------------------------------------------------------------------

NOTES ========================================================================

CODE =========================================================================

DOC ==========================================================================

TESTS ========================================================================

EVOLUTIONS ===================================================================


#END --------------------------------------------------------------------------



    2    User manual
    ________________


         2.1  xxxxxxx
              2.1.1  xxxxxx

    2.1  xxxxxxx
         Bla bla

    3    Porting guide
    __________________


         3.1  xxxxxxx
              3.1.1  xxxxxx

    3.1  xxxxxxx
         Bla bla


    4    Include files and external references
    __________________________________________
*/


#include <stdtyp.l>                    /* Hyperpanel definitions & constants */
#include <aut.h>                       /* S_evt                              */
#include <vmk.h>                       /* vhalt                              */

#include <tsk_iolib.h>                 /* io_write                           */
#include <tsk_fsmlib.h>                /* fsm_err fsm_evt                    */
#include <tsk_ttylib.h>                /* M_trans structure                  */
#include <vmk_tim.h>                   /* tim_get                            */
#include <text.h>                      /* hsprintf                           */

#include <systemm.txt>                 /* DEFAULT                            */

#ifdef TEST                            /* For unitary tests.                 */
#endif                                 /* End of ifdef TEST.                 */


/* Const UI menu data .........................................................

   Each menu page is described by 3 main const data items:
   - A string which is to be sent to draw the menu static content
   - A string which is one patch to the previous string
   - A list of const M_trans structures which is the UI event table

   We have one set of those for every menu, so we have 3 arrays of the same
   "ui_nbpage" size.
*/

extern const CHAR *const *             /* One string of character for        */
                const ui_menu[]      ; /* every menu (the menu characters)   */
                                       /*                                    */
extern const char *                    /* One patch string for every menu    */
                      ui_patch[]     ; /* or HNULL                           */
                                       /*                                    */
extern const INT      ui_nbpage      ; /* Number of menu pages/screens       */


/* Const UI app_cfg description data ..........................................

   Each variable of the "app_cfg[]" array, which is an array of "app_nbcfg" 
   UINT's, must be described with two items:

   - A list of strings addresses. Most values of "app_cfg[]" are in fact 
     enumerated values, and we provide one string for every possible value of
     the enumeration, those strings are intended to be displayed. So, 
     "ui_str_cfg[]" holds "app_nbcfg" addresses, each address is the one of an 
     array of string addresses. For example, the strings may be "POLL", "DIFI"
     and "IRQ", for one dedicated value of "app_cfg[VMON]"

   - A second list of string addresses. For every possible value of any
     "app_cfg[]" integer, we have a list of tag lists, such as "MONITOR=POLL",
     "MONITOR=DIFI", "MONITOR=IRQ". Those strings are intended to be given to
     "io_setval" and are corresponding to the 0/1/2 possible values of the
     "app_cfg[VMON]" UI variable.

   Both "ui_str_cfg[]" and "app_str_tag[]" are holding "app_nbfcg" addresses.
   The "app_cfg[]" array of UINTs holds "app_nbcfg" integers.
*/

extern UINT           app_cfg[]      ; /* UI configuration variables         */
extern UINT           app_iod[]      ; /* Variables corresponding IODs       */
                                       /*                                    */
extern const CHAR     ui_str_format[]; /* Output format codes                */
                                       /*                                    */
extern const CHAR * const *            /* List of lists of string addresses  */
                const ui_str_cfg[]   ; /* We have "app_nbcfg" addresses      */
                                       /*                                    */
extern const CHAR * const *            /* List of lists of string addresses  */
                const ui_str_tag[]   ; /* We have "app_nbcfg" addresses      */
                                       /*                                    */
extern const UCHAR * const ui_group[]; /* List of arrays of UCHARs which are */
                                       /* indexes in "app_cfg[]              */
extern const INT      app_nbcfg      ; /* Size of "app_cfg[]", ui_str_cfg[]  */
                                       /* "app_str_tag[]" "ui_str_format[]"  */

/* ..........................................................................*/


/*
    5    Inner workings of the module
    _________________________________


         5.1  xxxxx
              5.1.1  xxxxx

         5.2  xxxxx
              5.2.1  xxxx


    6    Exported (public) variables
    ________________________________
*/

#define LGCHR 20

/* UI global variables ......................................................*/

INT             ui_ttyiod            ; /* IOD for TTY                        */
UCHAR          *ui_sndbuf            ; /* Send buffer for TTY port           */
INT             ui_page              ; /* Current displayed page number      */
INT             ui_err               ; /* UI treatement error code           */
CHAR            ui_bufchr[LGCHR+1]   ; /* UI input buffer                    */
INT             ui_lgchr             ; /* Count of bytes currently in bufchr */
UCHAR          *ui_hexadr            ; /* For "phex"                         */

/* VT 100/200/220 Escape sequence .............................................

   Those TTY command/control character sequences are named "Escape sequences"
   because they begin with the ASCII "EScape" character (0x1B / 27)

   - scr_clear    Clears the TTY screen
   - loc_tlc      Moves the cursor to Tol Left Corner
   - loc_save     Request to save current cursor location
   - loc_restore  Request to restore cursor location
   - loc_set      Request to move the cursor to a given location
*/

const char   scr_clear   []     = "\x1B[2J"                             ;
const char   loc_tlc     []     = "\x1B[H"                              ;
const char   loc_save    []     = "\x1Bs"                               ;
const char   loc_restore []     = "\x1Bu"                               ;
const char   loc_set     []     = "\x1B[%d;%dH"                         ;

/*
    7    Internal constants of the module
    _____________________________________
*/

/*
    8    Internal data types of the module
    ______________________________________
*/

/* Structures and typedefs ..................................................*/

typedef struct                         /* One UI paramete group descriptor   */
  BEGIN                                /* ---------------------------------- */
    CHAR        *name                ; /* Location name, witout the @        */
    UCHAR       *ligadr              ; /* Line number variable address       */
    UCHAR       *coladr              ; /* Column number variable address     */
  END           Start_loc            ; /* ---------------------------------- */

/* Private function prototypes ..............................................*/

static void     get_start_loc   (UCHAR*,INT                                  );
static void     encode          (INT,CHAR*,INT                               );
static void     print_str       (INT,CHAR*                                   );

/*
    9    Internal (private) variables of the module
    _______________________________________________
*/

static UCHAR time_lig  , time_col    ; /* @time   location  line/column      */
static UCHAR uid_lig   , uid_col     ; /* @uid    location  line/column      */
static UCHAR brd_lig   , brd_col     ; /* @brd    location  line/column      */
static UCHAR cmd_lig   , cmd_col     ; /* @cmd    location  line/column      */
static UCHAR grp1_lig  , grp1_col    ; /* @grp    Location  line/column      */
static UCHAR grp2_lig  , grp2_col    ; /* grp@    Location  line/column      */
static UCHAR hex1_lig  , hex1_col    ; /* @hex    location  line/column      */
static UCHAR hex2_lig  , hex2_col    ; /* hex@    location  line/column      */
static UCHAR error_lig , error_col   ; /* @error  location  line/column      */
static UCHAR l_lig1[4] , l_col1[4]   ; /* @l0..3  location  line/column      */
static UCHAR l_lig2[4] , l_col2[4]   ; /* l0..3@  location  line/column      */

#define NB_STLOC  17

static const Start_loc start_loc[NB_STLOC] = /* Location labels              */
  BEGIN                                      /* -----------------------------*/
/*  name        ligadr        coladr                                         */
/*  ---------   -----------   -----------                                    */
    "@time"   , &time_lig   , &time_col   ,   /* Current hh/mm/ss time       */
    "@uid"    , &uid_lig    , &uid_col    ,   /* MCU Unique IDentifier       */
    "@brd"    , &brd_lig    , &brd_col    ,   /* Board name                  */
    "@cmd"    , &cmd_lig    , &cmd_col    ,   /* User input area for commands*/
    "@grp"    , &grp1_lig   , &grp1_col   ,   /* Top Left corner for group   */
    "grp@"    , &grp2_lig   , &grp2_col   ,   /* Bottom right corner group   */
    "@hex"    , &hex1_lig   , &hex1_col   ,   /* Top Left corner for hex dump*/
    "hex@"    , &hex2_lig   , &hex2_col   ,   /* Bottom right corner for hex */
    "@error"  , &error_lig  , &error_col  ,   /* Error message location      */
    "@l0"     , &l_lig1[0]  , &l_col1[0]  ,   /* TL Label 0 location         */
    "@l1"     , &l_lig1[1]  , &l_col1[1]  ,   /* TL Label 1 location         */
    "@l2"     , &l_lig1[2]  , &l_col1[2]  ,   /* TL Label 2 location         */
    "@l3"     , &l_lig1[3]  , &l_col1[3]  ,   /* TL Label 3 location         */
    "l0@"     , &l_lig2[0]  , &l_col2[0]  ,   /* BR Label 0 location         */
    "l1@"     , &l_lig2[1]  , &l_col2[1]  ,   /* BR Label 1 location         */
    "l2@"     , &l_lig2[2]  , &l_col2[2]  ,   /* BR Label 2 location         */
    "l3@"     , &l_lig2[3]  , &l_col2[3]  ,   /* BR Label 3 location         */
  END                                ; /* ---------------------------------- */

#define LOC_TIME    0                  /* @time  index                       */
#define LOC_UID     1                  /* @uid   index                       */
#define LOC_BRD     2                  /* @brd   index                       */
#define LOC_CMD     3                  /* @cmd   index                       */
#define LOC_GRPTL   4                  /* @grp   index                       */
#define LOC_GRPBR   5                  /* grp@   index                       */
#define LOC_HEXTL   6                  /* @hex   index                       */
#define LOC_HEXBR   7                  /* hex@   index                       */
#define LOC_ERROR   8                  /* @error index                       */
#define LOC_L0TL    9                  /* @l0    index                       */
#define LOC_L1TL   10                  /* @l1    index                       */
#define LOC_L2TL   11                  /* @l2    index                       */
#define LOC_L3TL   12                  /* @l3    index                       */

static const char * const hexdig = "0123456789ABCDEF";

/*
    10   Beginning of the code
    __________________________

------------------  UI event driven treatments ------------------------------
pmen                Print the menu on ASY and the time
ptim                Print the time in seconds
pgrp                Print the status strings
plab                Print one label string value
pchr                Print echo of one ASCII character
puid                Print the UID of the device
pbrd                Print the board name
pcmd                Print the last received character of user command
perr                Print the error message

tcfg                Togles a "cfg[]" value
icfg                Increments a "cfg[]" value/tcfg
tchr                Add character to "bufchr

scfg                Sets a "cfg[]" value
sval                Do a drv_detval with a "cfg[]" value
sint                Converts the ASCII decimal string from "ui_bufchr" to integer
sipa                Converts the ASCII IP address string from "ui_bufchr" to int

------------------  UI subroutines ------------------------------------------
roll_cfg            Roll a value of "cfg[]"

------------------  Internal subroutines ------------------------------------
get_start_loc       Parses the page and extract locations (lig,col)
encode              Converts a "app_cfg[]" integer to an ASCII string
print_str           Prints one string at its location

*/

/*  Procedure pmem ----------------------------------------------------------*/
/*
    Purpose : UI treatments - Prints (sends) on the ASY1 serial lune one
              application menu, one sequence of ASCCI characters.

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : pmen ( n )

    Input     : n        int        The menu number

    Output    : 

    Notes : - The n" value is here an index in the "ui_menu" const table.

...............................................................................
*/

IWORD  PROCEDURE pmen(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    UCHAR          *snd              ; /* Buffer to be sent address          */
    UCHAR          *pt               ; /* Write pointer                      */
    INT            sz                ; /* Written item length in bytes       */
    INT            lg                ; /* Count of bytes to be sent          */
    const CHAR     *ps               ; /* Patch address                      */
    CHAR           *pd               ; /* Pointer into the "snd" buffer      */
    INT            i                 ; /* Loop counter                       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("pmen")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Local variables initialization:                                 *
 * -------   - We retreive the "ui_sndbuf ASY transmit buffer address and    *
 *             copy it to the "snd" local variable                           *
 *           - We set the "pt" write pointer to "snd"                        *
 *****************************************************************************/

    snd = ui_sndbuf                  ; /* Get ASY1 transmit buffer addr      */
    pt  = snd                        ; /* Initialize the write pointer       */
    lg  = 0                          ; /* Clear the count of written bytes   */


/*****************************************************************************
 * Step 2 : Copy in the transmit buffer the characters that have to be sent  *
 * ------   in order to print the menu. The message is built as follows:     *
 *          - Put into "snd" the "Clear screen" ANSI Escape sequence and     *
 *            then increment the "pt" write pointer and the "lg" count       *
 *          - Add the "Move cursor to Top Left Corner ANSI Escape sequence,  *
 *            then increment "pt" and "lg" with the "sz" sequence size       *
 *          - Finally copy the "*(ui_menu[n])" menu string and we increment  *
 *            the "lg" count of bytes and the "pt" write pointer with the    *
 *            "sz" menu string size                                          *
 *          ---------------------------------------------------------------- *
 *          The "Clear screen" sequence is only used with the first app      *
 *          screen, with is by convention screen number 0.                   *
 *****************************************************************************/

    if (n EQ 0)                        /* If we have screen number 0, so     */
      BEGIN                            /* "ui_page" is 0, we copy to "snd"   */
        Strcpy(pt,scr_clear)         ; /* ANSI Escape sequence               */
        sz  = Strlen(pt)             ; /* "Clear TTY screen" then            */
        pt += sz                     ; /* increment the "pt" write pointer   */
        lg += sz                     ; /* and set count of written bytes     */
      END_IF                           /*                                    */
                                       /*                                    */
    Strcpy(pt,loc_tlc)               ; /* Copy to "snd" the "Move to Top     */
    sz  = Strlen(pt)                 ; /* Left Corner" ANSI Escape sequence  */
    pt += sz                         ; /* Increment the "pt" write pointer   */
    lg += sz                         ; /* and increment the count of bytes   */
                                       /*                                    */
    Strcpy(pt,*(ui_menu[n]))         ; /* Copy to "snd" the "menu[n]" string */
    sz  = Strlen(pt)                 ; /* Count od added bytes               */
    pt += sz                         ; /* Increment the "pt" write pointer   */
    lg += sz                         ; /* and increment the count of bytes   */


/*****************************************************************************
 * Step 3 : If there is a title or another item to be replaced, search for   *
 * ------   the first "#" and patch at that point:                           *
 *          - If the patch address "ui_patch[n]" is HNULL, nothing to do, go *
 *            to the next step.                                              *
 *          - Search in "ui_sndbuf" for the first '#' character. We use      *
 *            "pd" as a current character pointer and "i" as a counter.      *
 *          - If we found a '#' before the end of the "lg" characters, we    *
 *            copy "patch" to location "pt". We take care not to copy the    *
 *            nul byte that terminates the "patch" string.                   *
 *****************************************************************************/

    ps = ui_patch[n]                 ; /* The patch address. If we           */
    if (ps EQ HNULL) goto step3      ; /* do not have a patch, continue      */
                                       /*                                    */
    for(i = 0, pd = (CHAR*)snd    ;    /* Scan the characters until we find  */
        i LT lg LOGAND *pd NE '#' ;    /* '#' or we reach the buffer end, so */
        i++, pd++                  ) ; /* "lg" characters                    */
                                       /*                                    */
    if (i LT lg )                      /* If we found a '#', copy the "patch"*/
      Strncpy(pd        ,              /* at location "pd" in the buffer     */
              ps        ,              /* We do NOT copy the nul terminator  */
              strlen(ps)  )          ; /* of "patch"                         */


/*****************************************************************************
 * Step  4 : Parse the whole buffer, looking for "@xyz" sequences and set    *
 * -------   the "xyz" and "xyz" global variables. We put in "xyz_lig" the   *
 *           line number of '@" and in "xyz_col" the column number of '@'.   *
 *           The list of possible "xyz" names is given by the "start_loc[]"  *
 *           const table. All this is done by "get_start_loc" subroutine     *
 *****************************************************************************/

    step3                            : /* Send the "snd" buffer              */
    get_start_loc(snd,                 /* Parse the screen and set the       */
                  lg  )              ; /* "xyz_lig" and "xyz_col" variables  */

/*****************************************************************************
 * Step  5 : Add the ANSI Escape sequence "Move cursor to", using the two    *
 * -------   "cmd_lig" and "cmd_col" global variables, wich correspond to    *
 *           the '@' character of the "@cmd" string. We remind that the two  *
 *           "cmd_lig" and "cmd_col" global variables have just been set by  *
 *           the above call to "get_start_loc"                               *
 *****************************************************************************/

    hsprintf((CHAR*)pt ,              /* Add the ANSI Escape sequence        */
             loc_set   ,              /* "Move cursor" to                    */
             cmd_lig   ,              /* line number "cmd_lig" and           */
             cmd_col    )           ; /* column number "cmd_col"             */
                                      /*                                     */
    lg += Strlen(pt)                ; /* Increment the count of bytes        */


/*****************************************************************************
 * Step 6 : Send using serial ASY, so IOD "ui_ttyiod", the "lg" characters   *
 * ------   that are in the buffer pointed by "ui_sndbuf"". We call the      *
 *          "io_write" subroutine that sends the buffer and unschedules us   *
 *          until the effective transmission of the last byte.               *
 *****************************************************************************/

    io_write (ui_ttyiod,               /* I/O descriptor for ASY1            */
              snd      ,               /* Address of buffer                  */
              lg       ,               /* Number of bytes to be sent         */
              0          )           ; /* Flags, unused here                 */


/*****************************************************************************
 * Step 7 : Most menus are displaying time. We could wait for the next       *
 *          time-out event but we may have the XX:XX:XX displayed up to one  *
 *          second. So we do not wait for the time-out and we call here the  *
 *          "ptim" procedure that just rraws the time. We may also display   *
 *          the CPU identfier as XXXXXXXX:XXXXXXXX:XXXXXXXX                  *
 *          ---------------------------------------------------------------- *
 *          The "ptim" function draws at cursor location "time_lig" and      *
 *          "time_col", which have been extracted by "get_start_loc", from   *
 *          the "@time" location. Also, "puid" draws the UID at location     *
 *          "uid_lig" and "uid_col", which are the location of the '@' byte  *
 *          which begins the "@uid" string. Finally "pbrd" retreives the     *
 *          board name and displays it at location @brd                      *
 *          ---------------------------------------------------------------- *
 *          "ptim", "puid" and "pbrd" are including a test: If the @time/    *
 *          @uid/@brd string is not found in the screen, then time/uid/brd   *
 *          is not drawn. This allows to select, screen by screen, what is   *
 *          drawn                                                            *
 *****************************************************************************/

    ptim(0)                          ; /* Read time, convert in a string and */
                                       /* send it on the serial line         */
    puid(0)                          ; /* Read the CPU identifier, convert   */
                                       /* it to a string and send it         */
    pbrd(0)                          ; /* Read the board name an send it     */

/*****************************************************************************
 * Step 8 : Update the "ui_page" global variable that is the number of the   *
 * ------   menu that is currently displayed. We also reset the "ui_lgchr"   *
 *          count of characters of the "ui_bufchr" input buffer to 0.        *
 *****************************************************************************/

    ui_page = n                       ; /* Note the currently displayed menu */
                                        /* This is the UI state variable     */
    ui_lgchr     = 0                  ; /* Force to empty "ui_bufchr" buffer */
    ui_bufchr[0] = 0                  ; /* String nul terminator             */


#ifdef TEST                            /* Begin conditional compilation      */
    procfin("pmen")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure ptim ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Read time, convert it in a ASCII string and then 
              prints (sends) it on the ASY1 serial line.

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : ptim ( n )

    Input     : n        int        UNUSED

    Output    : vaproc   IWORD      always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE ptim(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    INT             d, m, y          ; /* Day, Month, Year                   */
    INT             h, mi, s, ms     ; /* Hour, minutes, seconds, msec       */
    CHAR            str[STDLINE]     ; /* String to be printed               */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("ptim")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : We first retrieve date and time since boot. We call the          *
 * ------   "tim_get" procedure to get date and time.                        *
 *****************************************************************************/

    tim_get(&d  ,                      /* day                                */
            &m  ,                      /* Month                              */
            &y  ,                      /* Year                               */
            &h  ,                      /* Hour (0..23)                       */
            &mi ,                      /* Minutes (0..59)                    */
            &s  ,                      /* Seconds (0..59)                    */
            &ms   )                  ; /* Milliseconds (0..999)              */

/*****************************************************************************
 * Step 2 : Build into "str" the string to be printed HH:MM:SS. We simply    *
 * ------   use "hsprintf".                                                  *
 *****************************************************************************/

    hsprintf(str              ,        /* Put in the string to be sent       */
             "%02d:%02d:%02d" ,        /* 8 characters HH:MM:SS              */
             h, mi, s              ) ; /*                                    */

/*****************************************************************************
 * Step 3 : Put in the transmit buffer the characters that have to be sent   *
 * ------   in order to print elapsed time since boot:                       *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will save current ANSI terminal pointer position.              *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will move the console cursor the location where the first      *
 *            character of the date will be printed.                         *
 *          - Write into the config buffer the "HH:MM:SS" string             *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will restore terminal pointer position.                        *
 *          - Using "io_write", to a blocking write of the buffer            *
 *****************************************************************************/

    print_str(LOC_TIME ,               /* Build the buffer contents and      */
              str       )            ; /* then send it with "io_write"       */


#ifdef TEST                            /* Begin conditional compilation      */
    procfin("ptim")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure pgrp ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Prints (sends) on the serial line the group of 
              string statuses. The input parameter "n" value is the group of
              statuses to be printed.

    Author : JYA                                 Last modification : 10/26/2022

...............................................................................

    Calling example : pgrp( n )

    Input     : n        int        First variable index in "app_cfg[]"

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE pgrp(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    const UCHAR    *idx              ; /* Copy of "ui_group[n]"              */
    UCHAR          *snd              ; /* Address of buffer to be sent       */
    INT            i                 ; /* Loop counter                       */
    INT            sz                ; /* Length of string item              */
    CHAR           *pt               ; /* Write pointer                      */
    INT            lg                ; /* Number of bytes to be sent         */
    CHAR           buf[STDLINE]      ; /* "hsprintf" string buffer           */
    INT            cnt               ; /* Count of "app_cfg[]" values        */
    INT            lig               ; /* First line number                  */
    INT            col               ; /* First column number                */
    INT            len               ; /* Item width in characters           */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("pgrp")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Initialization of local variables:                               * 
 * -------- - We copy "ui_group[n]" to "idx", this is the address of an      *
 *            array of UCHARS, each UCHAR is an index in "app_cfd[]". The    *
 *            "idx[]" contains the list of variable indexes for group "n".   *
 *          - We set "cnt" to "grp2_lig - grp1_lig+1" count of values/lines  *
 *            to be displayed                                                *
 *          - The "lig" first line number to be displayed is "grp1_lig"      *
 *            and the "col" first column number is "grp1_col"                *
 *          - The "len" line length is "grp2_col - grp1_col +1"              *
 *          - We use "pt" as a write pointer in the "ui_asysnd" transmit     *
 *            buffer. We initialize "pt" to "ui_sndbuf" start address.       *
 *          - We use "lg" as a count of written bytes in "ui_sndbuf", we     *
 *            initialize to 0 this count.                                    *
 *          -----------------------------------------------------------------*
 *          We remind that "grp1_lig" and "grp1_col" are the location of the *
 *          '@' character of the "@grp" string. In the same way, "grp2_lig"  *
 *          and "grp2_col" are the location of the '@' character of the      *
 *          "grp@" string                                                    *
 *****************************************************************************/

    idx   = ui_group[n]              ; /* List of variable indexes           */
    cnt   = grp2_lig - grp1_lig + 1  ; /* Count of "app_cfg[]" values        */
    lig   = grp1_lig                 ; /* First line number                  */
    col   = grp1_col                 ; /* Column number                      */
    len   = grp2_col - grp1_col + 1  ; /* Item length                        */
                                       /*                                    */
    snd   = ui_sndbuf                ; /* Buffer to be sent address          */
    pt    = (char*) snd              ; /* Init write pointer = beg of buffer */
    lg    = 0                        ; /* Init number of written bytes       */


/*****************************************************************************
 * Step 2 : Put in the transmit buffer the characters that have to be sent   *
 * ------   in order to print the "cnt" menu statuses. We are using "pt" as  *
 *          a current write position in the transmit buffer and "lg" is the  *
 *          current number of bytes inside the transmit buffer. We start by  *
 *          saving the current terminal pointer position (input):            *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will save current pointer position. Then we increment the "pt" *
 *            current write address and the "lg" number of bytes.            *
 *          Then we loop on the "cnt" values. For each value:                *
 *          - Copy into the transmit buffer the ASCII escape sequence that   *
 *            will move the console cursor to the location where the first   *
 *            status character will be drawn. The cursor has to ne moved to  *
 *            line "lig" column "col". We use "hsprintf" with "loc_set" as   *
 *            format string in order tu build in "buf" the ESC sequence,     *
 *            then "Strcpy(pt,buf)". We increment "pt" and "lg" with the     *
 *            length of "buf" (0 terminator not included)                    *
 *          - Write at address "pt" the "len" characters. We fistly fill     *
 *            with "len" characters "-" (background) then we overwrite the   *
 *            beginning (left justified) or the last (right justified) ones. *
 *            We call "encode" to do that, which takes on input the "ind"    *
 *            variable index in "app_cfg[]", the "pt" write address and the  *
 *            "len" display area length. The "encode" procedure will get the *
 *            "ui_str_format[ind]" value which indicates the output string   *
 *            format, including the justification bit.                       *
 *          Finally we restore the terminal pointer/cursor state:            *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will restore pointer position. Then we increment the "pt"      *
 *            current write address and the "lg" number of bytes.            *
 *****************************************************************************/

    strcpy(pt, loc_save)             ; /* Copy into the transmit buffer      */
                                       /* the ANSI escape sequence           */
    sz  = Strlen(loc_save)           ; /* Count of bytes, without the 0      */
    pt += sz                         ; /* Increment the write pointer        */
    lg += sz                         ; /* Increment the number of bytes      */
                                       /*                                    */
    for (i = 0     ;                   /* We loop on the "cnt" consecutive   */
         i LT cnt  ;                   /* entries of "app_cfg[]", we         */
         i++, lig++ )                  /* increment the "ind" index in       */
      BEGIN                            /* "app_cfg[]" at each loop step      */
        hsprintf(buf     ,             /* Build in "buf" the ANSI escape     */
                 loc_set ,             /* sequence which moves the cursor    */
                 lig     ,             /* to line "lig" column "col"         */
                 col      )          ; /*                                    */
                                       /*                                    */
        sz  = strlen(buf)            ; /* Copy into the transmit buffer      */
        Strcpy(pt,buf)               ; /* the ANSI escape sequence           */
        pt += sz                     ; /* Increment the write pointer        */
        lg += sz                     ; /* Increment the number of bytes      */
                                       /*                                    */
        memcpy(pt,                     /* Fill "len" characters with "-"     */
               "-----------------",    /* wich makes up the                  */
               len                  ); /* background                         */
                                       /*                                    */
        encode(idx[i] ,                /* Encode the "app_cfg[ind]" 32 bits  */
               pt     ,                /* value, starting at "pt" with a     */
               len      )            ; /* dislpy length of "len" bytes       */
                                       /*                                    */
        pt += len                    ; /* Increment the write pointer        */
        lg += len                    ; /* Increment the number of bytes      */
      END_FOR                          /*                                    */
                                       /*                                    */
    Strcpy(pt, loc_restore)          ; /* Copy into the transmit buffer      */
                                       /* the ANSI escape sequence           */
    sz  = Strlen(loc_restore)        ; /* Count of bytes, without 0          */
    pt += sz                         ; /* Increment the write pointer        */
    lg += sz                         ; /* Increment the number of bytes      */


/*****************************************************************************
 * Step 3 : Send using serial ASY, so IOD "ui_ttyiod", the "lg" characters   *
 * ------   that are in the buffer pointed by "ui_sndbuf". We call the       *
 *          "io_write" procedure that sends a REQ_WRITE request event to     *
 *          the AUT_ASY VMIO automaton. Then we unschedule until the         *
 *          RESP_WRITE response event is received.                           *
 *****************************************************************************/

    io_write (ui_ttyiod  ,            /* I/O descriptor for ASY\ASY1         */
              snd        ,            /* Address of buffer                   */
              lg         ,            /* Number of bytes to be sent          */
              0           )         ; /* Flags, unused here                  */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("pgrp")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure plab ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Prints (sends) on the serial line one string
              representation of one UI variable.

    Author : JYA                                 Last modification : 06/11/2023

...............................................................................

    Calling example : plab( n )

    Input     : n        int        Packed value
                                    . b0..15   Index in the app_cfg[] array
                                    . b16..31  Label number 0..9

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/
IWORD  PROCEDURE plab(int n)

  BEGIN                                /* Begin of this procedure            */

/*  Declaration of local variables of this procedure ........................*/

    INT            labidx            ; /* Involved label number 0..9         */
    INT            varidx            ; /* Index in "app_cfg[]" "ui_str_cfg[]"*/
    INT            col               ; /* First column number                */
    INT            len               ; /* Display width in bytes             */
    CHAR           str[STDLINE]      ; /* "hsprintf" string buffer           */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("plab")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Initialization of local variables:                               *
 * -------
 *****************************************************************************/

    labidx = n >> 16                 ; /* Extract the label number 0..9      */
    varidx = n & 0xFFFF              ; /* Extract index in app_cfg/ui_str_cfg*/
                                       /*                                    */
    col    = l_col1[labidx]          ; /* Involved label column number       */
    len    = l_col2[labidx] - col + 1; /* Width of label area                */
                                       /*                                    */

/*****************************************************************************
 * Step  2 : Build in "str" the string to be displayed. We call "encode"     *
 * -------   which receives the "varidx" app_cfg[] index and writes the      *
 *           ASCII encoded string to "str"                                   *
 *****************************************************************************/

    Memset(str,' ',len)              ; /* Fill with blank characters         */
                                       /*                                    */
    encode ( varidx ,                  /* Convert "app_cfg[varidx]" to an    */
             str    ,                  /* ASCII string and put it at "pt"    */
             len     )               ; /* with a "len" field width           */


/*****************************************************************************
 * Step 3 : Put in the transmit buffer the characters that have to be sent   *
 * ------   in order to print the "val" value at location @lx. We put into   *
 *          the "ui_snd" transmit buffer following items:                    *
 *          - Put into the transmit buffer the ANSI escape sequence that     *
 *            will save current pointer position.                            *
 *          - Put into the transmit buffer the ANSI escape sequence that     *
 *            will move the pointer to line "lig" and column "col".          *
 *          - Copy the "str" string into "ui_snd".                           *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will restore pointer position.                                 *
 *          Finally, "io_write" is called in order to perform a blocking     *
 *          write of th" ui_snd" transmit buffer                             *
 *****************************************************************************/

    print_str(LOC_L0TL + labidx,       /* Index in "start_loc[]"             */
              str               )    ; /* String to be displayed/printed     */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("plab")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure pchr ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Prints (sends) on the serial line one ASCII
              character (ASCII code is c).

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : pchr( c )

    Input     : c        int        Character ASCII code

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE pchr(int c)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    UCHAR           *snd             ; /* Address of buffer to be sent       */
    INT             lg               ; /* Number of bytes to be sent         */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("pchr")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : In the standard case, we simple send one byte, so "c" to the     *
 * ------   byte. However, we have some special cases to handle:             *
 *          - If we have a BACKSPACE and the count of characters is 0, so    *
 *            if "lgchr" is 0, there is nothing to do, we exit.              *
 *          - If we have a BACKSPACE and the count of displayed characters   *
 *            is not 0, we erase the last displayed bytes and move the       *
 *            cursor to the right. We have to send 3 bytes:                  *
 *            . A first BACKSPACE to move the cursor BACK, cursor is under   *
 *              the character to be erased                                   *
 *            . A space which erases the character. Cursor is moved.         *
 *            . A second BACKSPACE to move the cursor back                   *
 *          - If we have any character which is not a BACKSPACE but if the   *
 *            "ui_bufchr[]" buffer is already full, we do not echo anything, *
 *            we exit from the procedure                                     *
 *          - Finally, if the character is not a BACKSPACE and the buffer is *
 *            not full (ui_lgchr lower than LGCHR) we send the 'c' byte.     *
 *****************************************************************************/

    snd    = ui_sndbuf               ; /* Address of buffer to be sent       */
                                       /*                                    */
    if (c EQ 8)                        /* If we have a BACKSPACE to echo     */
      BEGIN                            /*                                    */
        if (ui_lgchr EQ 0)             /* Do nothing if buffer is empty      */
          goto exit                  ; /*                                    */
                                       /* Erase the last displayed character */
        snd[0] = snd[2] = 8          ; /* ASCII code for BACKSPACE           */
        snd[1] =          32         ; /* ASCII code for SPACE               */
        lg     =          3          ; /* We send 3 bytes                    */
      END_IF                           /*                                    */
    else                               /* If we have a "normal" ASCII code   */
      BEGIN                            /*                                    */
        if (ui_lgchr GE LGCHR)         /* Do noting if buffer is full        */
          goto exit                  ; /*                                    */
                                       /*                                    */
        snd[0] = c                   ; /* Copy the charcater to be sent      */
        lg     = 1                   ; /* We send one byte                   */
      END_ELSE                         /*                                    */


/*****************************************************************************
 * Step 2 : Send using serial ASY, so IOD "ui_ttyiod", the "lg" characters   *
 * ------   from "ui_sndadr". We call the "io_write" procedure that          *
 *          sends a REQ_WRITE request event to the AUT_ASY VMIO automaton.   *
 *          Then we unschedule until the RESP_WRITE response event is        *
 *          received.                                                        *
 *****************************************************************************/

    io_write (ui_ttyiod   ,            /* I/O descriptor for ASY             */
              snd         ,            /* Address of buffer                  */
              lg          ,            /* Number of bytes to be sent         */
              0             )        ; /* Flags, unused here                 */

    exit                             :

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("pchr")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure puid ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Prints (sends) on the serial line the UID of the
              MCU device.

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : puid ( n )

    Input     : n        int        Unused value

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE puid(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    CHAR            str[STDLINE]     ; /* MCU UID string                     */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("puid")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Get the MCU unique identifier using the "get_mcu_uid" function, *
 * -------   which returns a "str" ASCII character string.                   *
 *****************************************************************************/

    get_mcu_uid ( str )              ; /* Get the MCU unique Identifier      */

/*****************************************************************************
 * Step  2 : Build into "ui_sndbuf" the message to be sent to the TTY. This  *
 * -------   message is the concatenation of the following strings:          *
 *           - The "loc_save" string which tells the remote TTY to save the  *
 *             cursor/pointre location                                       *
 *           - The string which moves the pointer at the MCU UID TTY         *
 *             location. We encode using the "loc_set" format the "uid_lig"  *
 *             line number and the "uid_col" column number                   *
 *           - The "str" character representation of the MCU ID.             *
 *             applicative function "get_mcu_uid"                            *
 *           - Finally, the "loc_restore" string which tells the TTY to      *
 *             restore the saved cursor/pointer location                     *
 *           Then call "io_write" to perform a blocking write of "ui_sndbuf" *
 *****************************************************************************/

    print_str(LOC_UID  ,               /* Location of "@uid"                 */
              str       )            ; /* MCU UID unique string              */


#ifdef TEST                            /* Begin conditional compilation      */
    procfin("puid")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */
    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure pbrd ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Prints (sends) on the serial line the board name.

    Author : JYA                                 Last modification : 07/16/2023

...............................................................................

    Calling example : pbrd ( n )

    Input     : n        int        Unused value

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE pbrd(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    CHAR            str[STDLINE]     ; /* Board name string                  */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("pbrd")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Retreive the "str" board name using "get_board_name"            *
 * --------                                                                  *
 *****************************************************************************/

    get_board_name( str )            ; /* Copy to "pt" the board name        */
                                       /* character string                   */

/*****************************************************************************
 * Step  2 : Build into "ui_sndbuf" the message to be sent to the TTY. This  *
 * -------   message is the concatenation of the following strings:          *
 *           - The "loc_save" string which tells the remote TTY to save the  *
 *             cursor/pointre location                                       *
 *           - The string which moves the pointer at the LOC_BRD TTY         *
 *             location.                                                     *
 *           - The "str" board name                                          *
 *           - Finally, the "loc_restore" string which tells the TTY to      *
 *             restore the saved cursor/pointer location                     *
 *           Then call "io_write" to perform a blocking write of "ui_sndbuf" *
 *****************************************************************************/

    print_str(LOC_BRD  ,               /* Location of "@uid"                 */
              str       )            ; /* MCU UID unique string              */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("pbrd")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */
    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure pcmd ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Prints (sends) on the serial line the one byte
              command character.

    Author : JYA                                 Last modification : 10/16/2022

...............................................................................

    Calling example : pcmd( c )

    Input     : c        int        Character/byte to be echoed

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE pcmd(int c)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    UCHAR          *snd              ; /* Address of buffer to be sent       */
    INT            lg                ; /* Number of bytes to be sent         */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("pcmd")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : We copy to the local "snd" the ASY1 transmit buffer address,    *
 * -------   which we get from "ui_sndbuf"                                   *
 *****************************************************************************/

    snd = ui_sndbuf                  ; /* Addr of buffer to be sent          */


/*****************************************************************************
 * Step 2 : Put in the transmit buffer the characters that have to be sent   *
 * ------   in order to echo the 'c' command character. We are using "lg"    *
 *          as the current number of bytes inside the transmit buffer. We:   *
 *          - Encode into the transmit buffer the ANSI escape sequence that  *
 *            will move pointer to the right place. The "get_start_loc"      *
 *            procedure had set "cmd_lig" and "cmd_col" to the location of   *
 *            the '@' character of the "@cmd" string. Then set "lg" to the   *
 *            count of written ASCII characters/bytes.                       *
 *          - Add  in the transmit buffer the character to be echoed, then   *
 *            increment by 1 the "lg" count of bytes                         *
 *****************************************************************************/

    hsprintf((CHAR*)snd ,                /* Put the ANSI Escape sequence       */
             loc_set    ,              /* Move the cursor to                 */
             cmd_lig    ,              /* line "cmd_lig" and                 */
             cmd_col     )           ; /* column "cmd_col"                   */
                                       /*                                    */
    lg  = Strlen( snd )              ; /* Number of bytes in the buffer      */
                                       /*                                    */
    snd[lg++] = (CHAR)c              ; /* Put the echo in transmit buffer    */
                                       /* then increment "lg"                */

/*****************************************************************************
 * Step 3 : Send using serial ASY, so IOD "ui_ttyiod", the "lg" characters   *
 * ------   that are in the buffer pointed by "ui_sndbuf". We call the       *
 *          "io_write" procedure that sends a REQ_WRITE request event to     *
 *          the AUT_ASY VMIO automaton. Then we unschedule until the         *
 *          RESP_WRITE response event is received.                           *
 *****************************************************************************/

    io_write (ui_ttyiod   ,            /* I/O descriptor for ASY             */
              snd         ,            /* Address of buffer                  */
              lg          ,            /* Number of bytes to be sent         */
              0            )         ; /* Flags, unused here                 */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("pcmd")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure phex ----------------------------------------------------------*/
/*
    Purpose : UI treatement - Convert to hexadecimal the memory contents at
              "ui_hexadr". The number of printed bytes is defined by the "n"
               input parameter, this number may be truncated is the area
              defined by the "@hex" and "hex@" location tags is not enough.

    Author : JYA                                 Last modification : 10/26/2022

...............................................................................

    Calling example : phex( n )

    Input     : n        int        Count of bytes to be displayed

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE phex(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    UCHAR          *snd, c           ; /* Address of buffer to be sent       */
    INT            nb                ; /* Count of bytes per each line       */
    UCHAR          *pt               ; /* Write pointer                      */
    INT            lg                ; /* Number of bytes to be sent         */
    IWORD          i, j, k           ; /* Loop counter                       */
    IWORD          sz                ; /* Length of string item              */


/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("phex")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Initialization of local variables                               *
 * -------   - We retreive from "ui_sndbuf" the ASY transmit buffer address  *
 *             and put it in the "snd" local variable                        *
 *           - We initialize the "pt" write pointer to the "snd" address     *
 *           - We initialize the "lg" count of written bytes to 1            *
 *           - We compute the "nb" count of bytes we are display one each    *
 *             line. We use 3 characters for every byte, first display       *
 *             column is "hex1_col", last display column is "hex2_col" and   *
 *             so we compute "nb" as "hex2_col - hex1_col + 2) / 3           *
 *****************************************************************************/

    snd   = ui_sndbuf                ; /* Buffer to be sent address          */
    pt    = snd                      ; /* Init write pointer = beg of buffer */
    lg    = 0                        ; /* Init number of written bytes       */

    nb    = (hex2_col - hex1_col + 2)  /* Count of displayed hexadecimal     */
          / 3                        ; /* values per line                    */


/*****************************************************************************
 * Step 2 : Fill the "ui_sndbuf" transmit buffer with the characters we      *
 * ------   have to send to the VT terminal. We are using "pt" as a write    *
 *          pointer and "lg" as the current number of written bytes. The     *
 *          first operation is to put the ANSI sequence that requests the    *
 *          terminal to save its input position:                             *
 *          - Copy into the transmit buffer the "loc_save" ANSI escape       *
 *            sequence that requests to save the current pointer position.   *
 *            Then we increment "pt" and "lg" by strlen(loc_save)            *
 *          Then we loop on the 8 lines of text, each line is made of 16     *
 *          hexadecimal bytes (two hexadecimal digits per byte):             *
 *          - Copy into the transmit buffer the ASCII escape sequence that   *
 *            will move the console cursor to the location where the first   *
 *            character will be drawn. We have a static array of address     *
 *            strings "loc_hex[8]". Then we increment "pt" and "lg".         *
 *          - Add the 16 hexadecimal bytes, for each:                        *
 *            . the high order digit,                                        *
 *            . the low order digit,                                         *
 *            . and a "space" character                                      *
 *          Then we ask the terminal to restore the pointer state:           *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will restore the pointer position. Then we increment the "pt"  *
 *            current write address and the "lg" number of bytes.            *
 *****************************************************************************/

    Strcpy(pt, loc_save)             ; /* Copy into the transmit buffer      */
                                       /* the ANSI escape sequence           */
    sz  = Strlen(pt)                 ; /* Item lengh, without the 0          */
    pt += sz                         ; /* Increment the write pointer        */
    lg += sz                         ; /* Increment the number of bytes      */
                                       /*                                    */
    for (i = hex1_lig, k = 0     ;     /* Loop from line "hex1_lig" up to    */
         i LE hex2_lig && k LT n ;     /* line number "hex2_lig"             */
         i++                      )    /* Current line number is "i"         */
      BEGIN                            /*                                    */
        hsprintf((CHAR*)pt ,           /* Encode ANSI Escape sequence        */
                loc_set    ,           /* "Move cursor to"                   */
                i          ,           /* line number "i" and                */
                hex1_col    )        ; /* Culumn number "hex1_col"           */
                                       /*                                    */
        sz  = Strlen(pt)             ; /* Copy into the transmit buffer      */
        pt += sz                     ; /* Increment the write pointer        */
        lg += sz                     ; /* Increment the number of bytes      */
                                       /*                                    */
        for (j = 0; j LT nb ; j++,k++) /* Put "nb" hexadecimal numbers in    */
          BEGIN                        /* the transmit buffer                */
            c     = ui_hexadr[k]     ; /* Extract the current byte           */
            *pt++ = hexdig[c / nb]   ; /* High order hexadecimal digit       */
            *pt++ = hexdig[c % nb]   ; /* Low  order hexadecimal digit       */
            *pt++ = ' '              ; /* Space                              */
            lg   += 3                ; /* Increment the count of characters  */
          END_FOR                      /*                                    */
      END_FOR                          /*                                    */
                                       /*                                    */
    Strcpy(pt, loc_restore)          ; /* Copy into the transmit buffer      */
                                       /* the ANSI escape sequence           */
    sz  = Strlen(pt)                 ; /* Item length, without the 0         */
    pt += sz                         ; /* Increment the write pointer        */
    lg += sz                         ; /* Increment the number of bytes      */


/*****************************************************************************
 * Step 3 : Send using serial ASY, so IOD "ui_ttyiod", the "lg" characters   *
 * ------   that are in the buffer pointed by "ui_sndbuf". We call the       *
 *          "io_write" procedure that sends a REQ_WRITE request event to     *
 *          the AUT_ASY VMIO automaton. Then we unschedule until the         *
 *          RESP_WRITE response event is received.                           *
 *****************************************************************************/

    io_write (ui_ttyiod  ,             /* I/O descriptor for ASY\ASY1        */
              snd        ,             /* Address of buffer                  */
              lg         ,             /* Number of bytes to be sent         */
              0           )          ; /* Flags, unused here                 */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("phex")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure perr ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Prints (sends) on the serial line the decimal
              value or "fsm_err".

    Author : JYA                                 Last modification : 10/26/2022

...............................................................................

    Calling example : perr( n )

    Input     : n        int        Unused

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE perr(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    CHAR           str[STDLINE]      ; /* Decimal string for "fsm_err"       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("perr")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Build into "str" the error message, including the decimal       *
 * -------   encoding if the "fsm_err" global variable                       *
 *****************************************************************************/

    hsprintf(str,"Error code %d",      /* Build the error message string     */
             fsm_err             )   ; /*                                    */

/*****************************************************************************
 * Step  2 : Build into "ui_sndbuf" the message to be sent to the TTY. This  *
 * -------   message is the concatenation of the following strings:          *
 *           - The "loc_save" string which tells the remote TTY to save the  *
 *             cursor/pointre location                                       *
 *           - The string which moves the pointer at the @error TTY          *
 *             location.                                                     *
 *           - The "str" character string                                    *
 *           - Finally, the "loc_restore" string which tells the TTY to      *
 *             restore the saved cursor/pointer location                     *
 *           Then call "io_write" to perform a blocking write of "ui_sndbuf" *
 *****************************************************************************/

    print_str(LOC_ERROR ,              /* Location of "@uid"                 */
              str        )           ; /* MCU UID unique string              */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("perr")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure tcfg ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Toggles one "app_cfg[]" value (0->, 1->2, 2->3, ..)
              if "app_cfg[n]" has reached its maximum value, then it is set
              back to 0.

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : tcfg( n )

    Input     : n        int        Index in "app_cfg[]"

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE tcfg(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    const CHAR* const* estr          ; /* String list address                */
    INT                cnt           ; /* Count of enumerated values         */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("tcfg")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Local variable initialization. The "ui_str_cfg[]" array         *
 * -------   holds, for every "app_cfg[]" variable, the address of an array  *
 *           of string addresses. We "set "estr" to "ui_str_cfg[n]", so      *
 *           to the address of the array which contains the string addrs     *
 *****************************************************************************/

    estr = ui_str_cfg[n]             ; /* List of strings for the enumerated */
                                       /* value or 0 if no strings.          */


/*****************************************************************************
 * Step  2 : Compute the "cnt" count of enumerated value. We have two        *
 * -------   distinct cases to be distinguished:                             *
 *           - If "estr" is not HNULL, we simply count the number of         *
 *             string addresses, the array of addresses is always ended with *
 *             a HNULL address.                                              *
 *           - If "estr" is HNULL, we assume we have a two values toggle,    *
 *             so 0 and 1. We set "cnt" to 2.                                *
 *****************************************************************************/

    if ( estr )                        /* If we have a list of string addr   */
      for(cnt = 0; estr[cnt]; cnt++) ; /* we just count the addr with "cnt"  */
                                       /*                                    */
    else                               /* If there is no address list        */
      cnt = 2                        ; /* We assume only 0 and 1 are allowed */


/*****************************************************************************
 * Step  3 : Incrementing the "app_cfg[n]" integer is done using the         *
 * -------   "roll_cfg" subroutine, wich takes on input the "n" index, the   *
 *           minimum value (here 0) and the maximum value, here "cnt-1"      *
 *****************************************************************************/

    roll_cfg(n    ,                    /* Increment "app_cfg[n]"             */
             0    ,                    /* vmin = minimum allowed value       */
             cnt-1 )                 ; /* vmax = maximum allowed value       */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("tcfg")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */

/*  Procedure icfg ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Increment by 1 a "app_cgf[]" entry

    Author : JYA                                 Last modification : 06/22/2023

...............................................................................

    Calling example : icfg( n )

    Input     : n        int        Index in "app_cfg[]"

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE icfg(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("icfg")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    app_cfg[n] ++                    ; /* Increment by 1                     */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("icfg")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure tchr ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Add a new character to the input line

    Author : JYA                                 Last modification : 06/09/2023

...............................................................................

    Calling example : tchr( c )

    Input     : c        int        Character/byte to be added to "ui_bufchr"

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE tchr(int c)

  BEGIN                                /* Begin of this procedure            */

/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("tchr")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : The 'c' character is a backspace. We simply decrement the       *
 * -------   "ui_lgchr" count of stored characters/bytes. Unless the         *
 *           count is already 0.                                             *
 *****************************************************************************/

    if (c EQ 8)                        /* If we have a BACKSPACE             */
      BEGIN                            /*                                    */
        if ( ui_lgchr LE 0)            /* If the input buffer is empty,      */
          goto exit                  ; /* there is nothing to do, we exit    */
                                       /*                                    */
        ui_lgchr --                  ; /* We decrement the count of stored   */
        ui_bufchr[ui_lgchr] = 0      ; /* bytes and overwrite with a 0       */
      END_IF                           /*                                    */


/*****************************************************************************
 * Step  2 : The 'c' character is not a BACKSPACE. We store it into the      *
 * -------   "ui_bufchr[]" input buffer and we increment the "ui_lgchr"      *
 *           count, unless the buffer is already full                        *
 *****************************************************************************/

    else                               /* If we have a "normal" ASCII code   */
      BEGIN                            /*                                    */ 
        if ( ui_lgchr GE LGCHR)        /* If the input buffer is already     */
          goto exit                  ; /* full, we exit                      */
                                       /* else we store 'c' in "ui_bufchr"   */
        ui_bufchr[ui_lgchr++] = c    ; /* and we increment "ui_lgchr"        */
        ui_bufchr[ui_lgchr  ] = 0    ; /* End of C string terminator         */
      END_ELSE                         /*                                    */


    exit                             : /* Exit label                         */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("tchr")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return ( 0 )                     ;

  END_PROC                             /* End of the procedure               */


/*  Procedure scfg ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Sets one "cfg[]" value to "ui_evt -1".

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : scfg ( par )

    Input     : par      int        b0..15 Index in "app_cfg[]"
                                    b16.10 Value
                                    b31    Value selector

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE scfg(int par)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          idx               ; /* "app_cfg[]" index                  */
    UINT           val               ; /* Value of 'app_cfg[idx]"            */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("scfg")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : Local variable initialization :                                  *
 * ------   - "idx" is the index in "app_cfg[]", bits b0..15 of "par"        *
 *          - "val" is the value to be written to "app_cfg[idx". We have two *
 *            cases to be distinguished:                                     *
 *            . If bit 31 of "par" is clear, the value is "fsm_evt -1". We   *
 *              substract 1 because keyboard key 1 is always the first       *
 *              choice and is usually encoded as 1. With enumerated strings  *
 *              first string is number 0.                                    *
 *            . If bit 31 of "par" is set, then the value to be used is      *
 *              bits b16..30 (15 bits) of "par".                             *
 *           --------------------------------------------------------------- *
 *          The tables which will take "app_cfg[n]" as indexes are           *
 *          "ui_str_cfg[]" and "ui_str_tag[]".                               *
 *****************************************************************************/

    idx = par & 0xFFFF               ; /* Index in "app_cfg[]"               */
                                       /*                                    */
    val = par AND 0x80000000     ?     /* If bits b31 od "par" is set        */
          ((par >> 16) & 0x7FFF) :     /* value is bits b16..30 od "par"     */
          (fsm_evt -1          )     ; /* else value is "fsm_evt-1"          */


/*****************************************************************************
 * Step 2 : Now, we set "app_cfg[idx]" to "val"                              *
 * ------                                                                    *
 *****************************************************************************/

    app_cfg[idx] = val               ; /* Set "app_cfg[idx]"                 */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("scfg")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure sval ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Calls "io_setval", using "app_cfg[n]" as a taglist
              selector.

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : sval ( n )

    Input     : n        int        Index in "app_cfg[n]"

    Output    : 

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE sval(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD              r             ; /* Return code of the procedure       */
    const CHAR *const *tag           ; /* Taglist string address             */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("sval")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : We are going to call "io_setval" with a const string as second   *
 * ------   parameter. For each variable "n" ("n" is an index in "app_cfg[]")*
 *          we have a list of strings, that is in fact an array of string    *
 *          addresses. For example, for N = VBAUD, the corresponding list of *
 *          strings is "tag_baud". We have an array "ui_str_tag[]" that      *
 *          contains all the addresses of the string list, so the            *
 *          "ui_str_tag[VBAUD]" value is the address of "str_tag[VBAUD]".    *
 *          Here we set the "tag" local variable to "ui_str_tag[n]", so to   *
 *          "tag_baud" if "n" value is VBAUD.                                *
 *****************************************************************************/

    tag = ui_str_tag[n]              ; /* Address of an array of string      */
                                       /* addresses                          */


/*****************************************************************************
 * Step 2 : Now we call the "io_setval" procedure, that calls 'drv_setval"   *
 * -------  - The first parameter is the device IOD or the sub-device IOD.   *
 *            At step 3 of "loop_app_tsk" we had stored in "iods[n]" the     *
 *            reference to be used here. So "iods[n]" is our 1st parameter   *
 *          - The second parameter is the C string of index "cfg[n]" in the  *
 *            "tag" list of strings, so "tag[ cfg[n] ]" is the address of    *
 *            the const string to be provided as a second parameter.         *
 *          We have 3 variables that do not imply any call to "drv_setval",  *
 *          VASY2 (3), VDAC (4) and VSPEED (13). Indeed the present          *
 *          procedure must not be called with each of those values. We have  *
 *          HNULL values in "str_tag[VASY2]", in "str_tag[VDAC]" and         *
 *          "str_tag[VSPEED]". In case of any error in the const tables, we  *
 *          here check that "tag" is not HNULL (if yes, it's a bug...).      *
 *****************************************************************************/

    if ( tag )                         /* "tag" must not be HNULL            */
      r = io_setval(app_iod[n],        /* IOD to be used with "drv_setval"   */
                    tag[app_cfg[n]]) ; /* C string as a taglist              */
    else
      vhalt(0)                       ; /* Here we have a bug ...             */


#ifdef TEST                            /* Begin conditional compilation      */
    procfin("sval")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure sint ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Converts the decimal string from "bufstr" to a
              binary integer value.

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : sint( n )

    Input     : n        int        Index in "app_cfg[]"

    Output    : valproc  IWORD      Always 0

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE sint(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("sint")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : If the input buffer is empty, we put a 0 in it. We remind       *
 * -------   that the input buffer is "ui_bufchr" and that the count of      *
 *           stored bytes/characters is "ui_lgchr"                           *
 *****************************************************************************/

    if (ui_lgchr EQ 0)                 /* If the buffer is empty we put a    */
      ui_bufchr[ui_lgchr++] = '0'    ; /* '0' string (one byte), we          */
                                       /* increment "ui_lgchr" from 0 to 1   */

/*****************************************************************************
 * Step  2 : Convert the "ui_bufchr" content, which is supposed to be a      *
 * -------   decimal unsigned string, to an integer value and store it into  *
 *           "app_cfg[n]". We simply call "hsscanf". However, we must add a  *
 *           0 C string terminator before calling "hsscanf"                  *
 *****************************************************************************/

    ui_bufchr[ui_lgchr] = 0          ; /* Add the nul end of C string        */
                                       /*                                    */
    hsscanf(ui_bufchr  ,               /* Buffer to be scanned address       */
            "%d"       ,               /* Parsing format string              */
            &app_cfg[n]   )          ; /* store the integer to "app_cfg[n]"  */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("sint")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( 0 )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure sipa ----------------------------------------------------------*/
/*
    Purpose : UI treatment - Converts the string from "ui_bufchr" which is
              supposed to be an IP address "xxx.xxx.xxx.xxx" to a binary 
              integer value.

    Author : JYA                                 Last modification : 06/08/2023

...............................................................................

    Calling example : sipa( n )

    Input     : n        int        Index in "app_cfg[]"

    Output    : valproc  IWORD      0: Valid IP address 1: Invalid string

    Notes : 

...............................................................................
*/

IWORD  PROCEDURE sipa(int n)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD          r                 ; /* 0:OK 1:KO                          */
    CHAR           *p                ; /* Read pointer into "ui_bufchr"      */
    UINT           ipval             ;
    USHORT         v                 ;
    IWORD          i                 ; /* Loop counter on 4 blocks           */
    IWORD          j                 ; /* Loop counter on block digits       */
    UCHAR          c                 ;

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("sipa")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Initialitation of local variables:                              *
 * -------   - We will use "p" as a read character pointer into "ui_bufchr[]"*
 *             and here we initialize it to the address of "ui_bufchr[]"     *
 *           - The "ipval" 32 bits IP V4 address is cleared                  *
 *           - The "r" function rteurn value is initialize to 1 - FAIL       *
 *****************************************************************************/

    p     = ui_bufchr                ; /* "ui_bufchr[]" read pointer         */
    ipval = 0                        ; /* Final 32 bits IP address           */
    r     = 1                        ; /* Let's assume we will fail          */


/*****************************************************************************
 * Step  2 : Character parsing loop. This loop is in fact made of two nested *
 * -------   loops. The external loop is on the 4 decimal numbers, using "i" *
 *           ranging from 0 to 4. The internal loop is on the (at most) 4    *
 *           characters of each group: 3 decimal digits (max) and then one   *
 *           separator, which is a "." after the 3 first groups and a nul    *
 *           byte terminator after the 4th group, So the awaited string is   *
 *           something like xxx.xxx.xxx.xxx or x.x.x.x or any combination    *
 *           of groups of 1 to 3 decimal digits. Also, the value of any      *
 *           digit group must never exceed 255. For each of the 4 steps of   *
 *           the external loop, we perform as follows (i=0..3) :             *
 *           - We clear the "v" local wchich is the converted value of the   *
 *             group of decimal digits                                       *
 *           - We perform the inner loop with the (max) 4 characters of the  *
 *             group, 3 (max) decimal digits and one separator:              *
 *             . We read the "c" next character/byte and increment the "p"   *
 *               read pointer/address.                                       *
 *             . If "c" is a separator, so a '.' or the 0 nul terminator,    *
 *               we jump to the "separator" treatment.                       *
 *             . If "c" is not a decimal digit, in the ['0','9'] range, we   *
 *               have an invalid input string and we exit with "r=1"         *
 *             . Else we have a digit, we convert the character to a binary  *
 *               value by substracting '0' and we cumulate the digit value   *
 *               to "v". We simply compute "v = 10 * v + (c - '0')           *
 *           - If we reach the end of the loop on the 4 characters without   *
 *             jumping to "separator", this means we found 4 consecutive     *
 *             digits, which is too much, we exit with "r=1"                 *
 *           - The "separator" treatment firstly performs error checkings    *
 *             . If we did not had any digit before the separator, if "j" is *
 *               0, we have an invalid string                                *
 *             . If the "v" converted value is greater than 255, we have an  *
 *               invalid value.                                              *
 *             . If one of the 3 firsts separaror is not a '.', string is    *
 *               invalid.                                                    *
 *             . If fourth separator is not the 0 byte terminator, string    *
 *               is also invalid.                                            *
 *             If any error is detected, we exit with "r=1"                  *
 *           - We add "v" to "ipval". With the first group (i=0), we store   *
 *             "v" in bits b24..31 of "ipval. With second group (i=1) we     *
 *             put "v" in bits b16..23 of "ipval", and so on.                *
 *****************************************************************************/

    for ( i = 0; i LT 4 ; i++ )        /* We loop on the 4 groups of         */
      BEGIN                            /* characters                         */
        v = 0                        ; /* Converted value of the 1 to 3      */
                                       /* decimal digits                     */
        for (j = 0; j LT 4; j++)       /* Loop on the (max) 4 characters of  */
          BEGIN                        /* current group                      */
            c   = *p++               ; /* Read next byte/char from the       */
                                       /* "ui_bufchr[]" C string buffer      */
            if (c EQ '.' LOGOR         /* If we have a separator, so a '.'   */
                c EQ 0        )        /* or the nul byte terminator, jump   */
              goto separator         ; /* to separator treatment             */
                                       /*                                    */
            if (c LT '0' LOGOR         /* If character is not a decimal      */
                c GT '9'      )        /* digit 0..9 we have an error and    */
              goto exit              ; /* we exit with "r=1"                 */
                                       /*                                    */
            v = 10 * v                 /* Convert the "c" decimal digit to   */
              + (c - '0')            ; /* the c - '0" binary value and       */
          END_FOR                      /* cumulate with "v"                  */
                                       /*                                    */
        goto exit                    ; /* If we are here, the string has     */
                                       /* extra trailing bytes, this means   */
                                       /* the string is invalid.             */
        separator                    : /* "c" is a '.' or a 0 num byte       */
                                       /* Possible errors are:               */
        if((j EQ 0            ) ||     /* . No decimal digit at all          */
           (v GT 255          ) ||     /* . Converted value greater than 255 */
           (i LT 3 && c NE '.') ||     /* . Separator 1/2/3 is not a '.'     */
           (i EQ 3 && c NE 0  )   )    /* . Separator 4 is not a 0 byte      */
          goto exit                  ; /* We exit with "r=1"                 */
                                       /*                                    */
        ipval |= v << (8 * (3-i))    ; /* Add byte in b24..31, then b16..23, */
      END_FOR                          /* then b8..15 and b0..7              */


/*****************************************************************************
 * Step  3 : Here, we are successful :                                       *
 * -------   - We store the "ipval" 32 bits IP address to "app_cfg[n]"       *
 *           - We clear the "r" function returned vale                       *
 *****************************************************************************/

    app_cfg[ n ] = ipval             ; /* Store to "app_cfg[n]"              */
    r            = 0                 ; /* Clear function error code          */

    exit                             : /* Procedure exit label               */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("sipa")                  ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( r )                      ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure roll_cfg ------------------------------------------------------*/
/*
    Purpose : Increments by one an entry of "app_cfg[]"

    Author : JYA                                 Last modification : 10/24/2022

...............................................................................

    Calling example : roll_cfg( ind, vmin, vmax )

    Input     : ind      int        Index in "app_cfg[]"
                vmin     int        Minimum allowed value
                vmax     int        Maximum allowed value

    Output    : valproc  int        New value of "app_cfg[n]"

    Notes : 

...............................................................................
*/

int    PROCEDURE roll_cfg(int ind, int vmin, int vmax)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("roll_cfg")              ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : We increment by 1 "app_cfg[ind]" and if we exceed "vmax" then   *
 * -------   we set back "app_cfg[ind]" to "vmin"                            *
 *****************************************************************************/

    app_cfg[ind] ++                  ; /* Increment the value                */
                                       /*                                    */
    if ( app_cfg[ind] GT vmax )        /* If we are greater than "vmax"      */
      app_cfg[ind] = vmin            ; /* then reset to "vmin"               */


#ifdef TEST                            /* Begin conditional compilation      */
    procfin("roll_cfg")              ; /* Message                            */
#endif                                 /* End   conditional compilation      */

    return( app_cfg[ind] )           ; /* Return code of the procedure       */

  END_PROC                             /* End of the procedure               */


/*  Procedure get_start_loc -------------------------------------------------*/
/*
    Purpose : Parses a page, detects the @tagloc strings and save the
              corresponding line and column numbers into UCHAR global variables

    Author : JYA                                 Last modification : 10/25/2022

...............................................................................

    Calling example : get_start_loc( buf, lg )

    Input     : buf      UCHAR*     Page buffer address
                lg       INT        Count of stored bytes

    Output    : 

    Notes : 

...............................................................................
*/

static void   PROCEDURE get_start_loc(UCHAR *buf, INT lg)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    IWORD           i                ; /* Loop counter on characters         */
    UCHAR           *pt              ; /* Character pointer                  */
    UCHAR           c                ; /* Current character value            */
    IWORD           lig, col         ; /* Current character                  */
    IWORD           j                ; /* Loop counter on "start_loc" entries*/
    const Start_loc *sl              ; /* Current "start_loc" entry addr     */
    CHAR            *name            ; /* "sl->name" string address          */
    UCHAR           *cmp             ; /* First character to be compared add */
    IWORD           l                ; /* Current "start_loc" name length    */
    IWORD           k                ; /* Loop counter of space characters   */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("get_start_loc")         ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Initialization of local and global variables :                  *
 * -------   - We will use the two local variables "lig" et "col" as the     *
 *             current line and column numbers. The first line number is 1   *
 *             and the first column number is 1, we set both those to 1.     *
 *           - We have a number of ("xyz_lig","xyz_col") couples of global   *
 *             variables which are line and column numbers. Their addresses  *
 *             are kept in the "start_loc[]" const table, we clear the       *
 *             variable with a loop on the NB_STLOC table entries and we     *
 *             clear the UCHAR's using the "sl->lifadr" and "sl->coladr"     *
 *             addresses.                                                    *
 *****************************************************************************/

    lig =                              /* Current line number is 1           */
    col = 1                          ; /* Current column number is 1         */
                                       /*                                    */
    for (j = 0, sl = start_loc ;       /* We set to a 0 empty value all the  */
         j LT NB_STLOC         ;       /* UCHAR global variables numbers     */
         j++, sl++               )     /* which are:                         */
      *(sl->ligadr) =                  /* - a line number                    */
      *(sl->coladr) = 0              ; /* - a column number                  */


/*****************************************************************************
 * Step  2 : This is a character parsing loop, we search for the "name"      *
 * -------   string fields of the "loc_start[]" table. Those strings may be  *
 *           "@xyz" or "xyz@". We store in the "*ligadr" and "*coladr"       *
 *           global variables ("xyz_lig" and "xyz_col") the exact location   *
 *           "lig" and "loc" of the '@' character, whatever it is the first  *
 *           or the last character of the "name" field.                      *
 *           --------------------------------------------------------------- *
 *           The loop variables are the "i" input character number and its   *
 *           "pt" address. We use "lig" as current line number and "col" as  *
 *           current column number. For every character, we:                 *
 *           - Read the character at adress "pt" and put in "c"              *
 *           - If the character is a '\n" then we increment by 1 the "lig"   *
 *             current line number.                                          *
 *           - Else, if the character is a '\r" then we reset "col" to 1     *
 *           - Else if the character is not a '@", we increment "col" by 1   *
 *           - Else, we have a '@'. We loop on the NB_STLOC entries of the   *
 *             "start_loc[]" table, using "j" as the entry index and "sl" as *
 *             the entry address. The loop is a search into "start_loc[]"    *
 *             of the "@xyz" or "xyz@" potition tag. At every step, we:      *
 *             . Set the local "name" to "sl->name", so the "keyword" string *
 *               address. Then we compute the "l" length of this string.     *
 *             . We determine the "cmp" address of the first byte of "buf"   *
 *               to be compared with the "name" keyword. If the keyword      *
 *               begins with '@', then we set the "cmp" address to "pt".     *
 *               Else, the '@' is the last character of "name" and so the    *
 *               first character of "buf" to be compared with "name" is at   *
 *               address "cmp = pt - l + 1".                                 *
 *             . We put in 'c' the character we will use to overwrite the    *
 *               found keyword (if we have a match) in "buf". If we begin    *
 *               with '@' we will use the character that follows the keyword *
 *               in "buf", so at address "pt + l". If keyword ends by '@',   *
 *               we will use the byte wich is just before the keyword in     *
 *               "buf", so at address "cmp - 1"                              *
 *             . We compare the "l" bytes at "cmp" (in the buffer) with the  *
 *               "name" keyword. If they are not equal, we continue with the *
 *               next "start_loc[]" entry, with the next "j" index value.    *
 *             . Otherwise if we have a match, if the "l" bytes starting at  *
 *               "cmp" in the buffer are equel to "name", we store the       *
 *               current "lig" line number to "*(sl->ligadr)" and we store   *
 *               the current "col" column number to "*(sl->coladr)".         *
 *             . Finally we overwrite the "l" bytes, starting at "cmp", with *
 *               the same 'c' byte. This overwrites the "@xyz" or "xyz@"     *
 *               string in the "buf" buffer.                                 *
 *             We increment "col" by 1 and continue the loop with "i"        *
 *****************************************************************************/

    for( i = 0, pt = buf ;             /* At every step of this loop, we     */
         i LT lg         ;             /* parse one byte of "buf". We use "i"*/
         i++, pt++        )            /* as the loop counter/byte index     */
      BEGIN                            /* and "pt" as the byte address       */
        c = *pt                      ; /* Make a copy of the current byte    */
                                       /* in the "c" local variable          */
        if ( c EQ '\n')                /* If we have a line-feed, increment  */
          lig ++                     ; /* the "lig" current line number      */
                                       /*                                    */
        else if ( c EQ '\r')           /* If we have a carriage return,      */
          col = 1                    ; /* reset to 1 the "col" current       */
                                       /* column number.                     */
        else if (c NE '@')             /* With any other character which is  */
          col ++                     ; /* not a '@", we only increment the   */
                                       /* "col" current column number.       */
        else                           /*                                    */
          BEGIN                        /* Character "*pt" is a '@'.          */
            for(j=0, sl=start_loc ;    /* We loop on the entries of the      */
                j LT NB_STLOC     ;    /* const "start_loc[]" table of       */
                j++, sl++           )  /* keywords. Keywords may be "@xyz"   */
              BEGIN                    /* or "xyz@".                         */
                name = sl->name      ; /* Get table keyword address and      */
                l = Strlen(name)     ; /* get its "l" string length.         */
                                       /*                                    */
                if (name[0] EQ '@')    /* If the table keyword begins with a */
                  BEGIN                /* '@', we compare the keyword with   */
                    cmp = pt         ; /* the buffer data starting at "pt"   */
                    c   = pt[l]      ; /* Replacement byte is the buffer     */
                  END_IF               /* byte at address "pt + l".          */
                else                   /* Else the keyword ends with '@', we */
                  BEGIN                /* compare the "name" keyword with    */
                    cmp = pt -l +1   ; /* buffer data startting at "pt-l+1"  */
                    c   = *(cmp-1)   ; /* Replacement byte is the buffer     */
                  END_ELSE             /* byte at address "pt - l"           */
                                       /*                                    */
                if( Memcmp(cmp   ,     /* Compare the "l" bytes of "name"    */
                            name ,     /* with the "l" buffer bytes at       */
                            l     ))   /* address "cmp" (so "pt" or "pt-l+1")*/
                  continue           ; /* If we do not match, continue with  */
                                       /* next keyword of "start_loc[]"      */
                                       /* We have a match !!                 */
                *(sl->ligadr) = lig  ; /* Store the current line and column  */
                *(sl->coladr) = col  ; /* numbers into 2 UCHAR global        */
                                       /* variables "xyz_lig" and "xyz_col"  */
                for(k=0; k LT l;k++)   /* Inside the "buf" buffer, overwrite */
                  cmp[k] = c         ; /* the "@xyz" or "xuz@" string        */
                                       /*                                    */
                break                ; /* break the loop on "start_loc[]"    */
              END_FOR                  /* entries.                           */
                                       /*                                    */
            col ++                   ; /* We had one '@', we increment by 1  */
          END_ELSE                     /* the current column number          */
                                       /*                                    */
      END_FOR                          /* End of the loop on characters      */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("get_start_loc")         ; /* Message                            */
#endif                                 /* End   conditional compilation      */

  END_PROC                             /* End of the procedure               */

/*  Procedure encode --------------------------------------------------------*/
/*
    Purpose : Converts one "app_cfg[]" integer

    Author : JYA                                 Last modification : 06/09/2023

...............................................................................

    Calling example : encode ( n, str, len )

    Input     : n        IWORD      Index in "app_cfg[]"
                len      INT        Display area length

    Output    : str      CHAR*      ASCII string

    Notes : 

...............................................................................
*/

static void   PROCEDURE encode(INT n, CHAR *str, INT len)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    UINT           v                 ; /* Copy of "app_cfg[n]"               */
    UCHAR          f                 ; /* Copy of "ui_str_format[n]"         */
    CHAR           buf[STDLINE]      ; /* Encoded ASCII string               */
    INT            l                 ; /* Encoded ASCII string length        */
    const CHAR     *const *tab       ; /* List of string addresses (enum)    */
    INT            idx               ; /* Character index in "str[]"         */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("encode")                ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step  1 : Initialization of local variables                               *
 * -------   - The 32 bits integer to be converted to an ASCCI string is     *
 *             "app_cfg[n]", we copy it to local "v".                        *
 *           - The output format code is "ui_str_format[]" that we copy to   *
 *             local "f". We reming that bit b0 is the justification bit     *
 *             indicator (0->left 1->right).                                 *
 *****************************************************************************/

    v  = app_cfg      [n]            ; /* 32 bits integer value to convert   */
    f  = ui_str_format[n]            ; /* Output format conversion code      */


/*****************************************************************************
 * Step  2 : Build into "str" the ASCCI string, according to the "f" output  *
 * -------   format:                                                         *
 *           .                                                               *
 *****************************************************************************/

    switch ( f )                       /* According to the output format     */
      BEGIN                            /*                                    */
        case F_DECL                  : /* Decimal integer left  justified    */
        case F_DECR                  : /* Decimal integer right justified    */
          hsprintf(buf,"%d",v)       ; /* Build a decimal ASCII string       */
          break                      ; /* ---------------------------------- */
                                       /*                                    */
        case F_ENUML                 : /* Enumerated      left  justified    */
        case F_ENUMR                 : /* Enumerated      right justified    */
          tab = ui_str_cfg[n]        ; /* Adr of a list of string addresses  */
          Strcpy(buf,tab[v])         ; /* Copy the "tab[n]" enumerated string*/
          break                      ; /* ---------------------------------- */
                                       /*                                    */
        case F_IPL                   : /* IP address      left  justified    */
        case F_IPR                   : /* IP address      right justified    */
           hsprintf(buf            ,   /* Build an IP address ASCII string   */
                   "%d.%d.%d.%d"   ,   /* xxx.xxx.xxx.xxxx                   */
                    v >> 24        ,   /* Bits b24..31                       */
                   (v >> 16) & 0xFF,   /* Bits b16..23                       */
                   (v >>  8) & 0xFF,   /* Bits b8..15                        */
                   (v      ) & 0xFF ); /* Bits b0..7                         */
          break                      ; /* ---------------------------------- */
      END

/*****************************************************************************
 * Step  3 : If necessary, truncate to "len" characters the "buf" string     *
 * --------  We compute the "l" string length of "buf". If "l" is greater    *
 *           that "len", we truncate "buf" to "len" characters:              *
 *           - We overwrite "buf[len]" with a 0 byte string terminator       *
 *           - We update l" with the smaller "len" value.                    *
 *****************************************************************************/

    l = Strlen(buf)                  ; /* Compute the "l" length of the      */
                                       /* "buf" string we just built"        */
    if ( l GT len )                    /* If we overflow the "len" display   */
      BEGIN                            /* width, we truncate to "len" bytes  */
        buf[len] = 0                 ; /* with an end of string terminator   */
        l        = len               ; /* and we udate the "l" length to     */
      END_IF                           /* "len"                              */

/*****************************************************************************
 * Step  4 : Copy "buf" to "str", taking in account bit b0 of "f", 0 means   *
 * ------    "left" justification and 1 means "right" justification.         *
 *           - With "left", we simply copy "buf" to "str"                    *
 *           - With "rifgt", we copy "buf" to "str + len - l"                *
 *****************************************************************************/

    idx = f AND 1 ? (len-l)            /* Right justification                */
                  : 0                ; /* Left justification                 */
                                       /*                                    */
    Strcpy(str + idx ,                 /* Copy "buf" to "str" (left) ou to   */
           buf        )              ; /* "str + len - l" (right)            */

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("encode")                ; /* Message                            */
#endif                                 /* End   conditional compilation      */

  END_PROC                             /* End of the procedure               */

/*  Procedure print_str -----------------------------------------------------*/
/*
    Purpose : Print one string at its location

    Author : JYA                                 Last modification : 07/17/2023

...............................................................................

    Calling example : print_str(locidx, str)

    Input     : locidx   INT        Index in the "start_loc[]" table
                str      CHAR*      String to be printed

    Output    : 

    Notes : 

...............................................................................
*/

static void   PROCEDURE print_str(INT locidx, CHAR *str)

  BEGIN                                /* Begin of this procedure            */
 
/*  Declaration of local variables of this procedure ........................*/

    const Start_loc *ploc            ; /* Address of "start_loc[locidx]"     */
    UCHAR           *snd             ; /* Buffer to be sent address          */
    CHAR            *pt              ; /* Write pointer                      */
    INT             lg               ; /* Number of bytes to be sent         */
    INT             len              ; /* Current item length in bytes       */

/*  Beginning of the code ...................................................*/

#ifdef TEST                            /* Begin conditional compilation      */
    procdeb("print_str")             ; /* Message                            */
#endif                                 /* End   conditional compilation      */

/*****************************************************************************
 * Step 1 : If the function is called but the current page does not contains *
 * -------  the "@xyz" keyword, if "*(ploc->ligadr)" is the 0 empty value,   *
 *          we do not display "str", we simply exit                          *
 *****************************************************************************/

    ploc = start_loc + locidx        ; /* "ploc" is "start_loc[locidx]" addr */
                                       /*                                    */
    if ( *(ploc->ligadr) EQ 0)         /* If current menu page does not      */
      goto exit                      ; /* features @xyz, exit immediately    */


/*****************************************************************************
 * Step 3 : Put in the transmit buffer the characters that have to be sent   *
 * ------   in order to print the "str" string. We are using "lg" as the     *
 *           current number of bytes inside the transmit buffer. We:         *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will save current ANSI terminal pointer position. Then we      *
 *            increment the "lg" number of bytes and "pt" write pointer.     *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will move the console cursor the location where the first      *
 *            character of the date will be printed. Then we increment the   *
 *            "lg" number of bytes and the "pt" write vpointer.              *
 *          - Copy into the transmit buffer the "str" C string and then      *
 *            increment the "lg" byte count and the "pt" write pointer.      *
 *          - Copy into the transmit buffer the ANSI escape sequence that    *
 *            will restore terminal pointer position. Then we increment the  *
 *            "lg" number of bytes.                                          *
 *          ---------------------------------------------------------------- *
 *          The cursor "normal" position is on the right of the last line    *
 *          of the current menu page "Enter command". But the line number    *
 *          where that prompt lays is not the same one for all the menu      *
 *          pages. So whatever the current menu page is, we request the      *
 *          terminal to save the cursor position and to restore it after     *
 *          current time has been displayed.                                 *
 *****************************************************************************/

    snd = ui_sndbuf                  ; /* Buffer to be sent address          */
    pt  = (CHAR*) snd                ; /* Init write pointer = beg of buffer */
    lg  = 0                          ; /* Init number of written bytes       */
                                       /*                                    */
    Strcpy(pt, loc_save)             ; /* Copy into the transmit buffer      */
                                       /* the ANSI escape sequence           */
    len = Strlen(pt)                 ; /* Count of bytes, without the 0      */
    pt += len                        ; /* Increment the write pointer        */
    lg += len                        ; /* Increment the number of bytes      */
                                       /*                                    */
    hsprintf(pt              ,         /* We directly build into "snd" the   */
             loc_set         ,         /* ANSI ESC sequence which moves the  */
             *(ploc->ligadr) ,         /* cursor to line "*ploc->ligadr" and */
             *(ploc->coladr)   )     ; /* column "*ploc->coladr"             */
                                       /*                                    */
    len = Strlen(pt)                 ; /* Count of bytes, without the 0      */
    pt += len                        ; /* Increment the write pointer        */
    lg += len                        ; /* Increment the number of bytes      */
                                       /*                                    */
    Strcpy(pt, str)                  ; /*                                    */
    len = Strlen(pt)                 ; /* Count of bytes, without the 0      */
    pt += len                        ; /* Increment the write pointer        */
    lg += len                        ; /* Increment the number of bytes      */
                                       /*                                    */
    Strcpy(pt, loc_restore)          ; /* Copy into the transmit buffer      */
                                       /* the ANSI escape sequence           */
    len = Strlen(pt)                 ; /* Count of bytes, without the 0      */
    pt += len                        ; /* Increment the number of bytes      */
    lg += len                        ; /* Increment the number of bytes      */


/*****************************************************************************
 * Step 3 : Send using serial ASY, so IOD "ui_ttyiod", the "lg" characters   *
 * ------   that are in the buffer pointed by "ui_sndbuf". We call the       *
 *          "io_write" procedure that sends a REQ_WRITE request event to     *
 *          the AUT_ASY VMIO automaton. Then we unschedule until the         *
 *          RESP_WRITE response event is received.                           *
 *****************************************************************************/

    io_write (ui_ttyiod   ,            /* I/O descriptor for ASY\ASY1        */
              snd         ,            /* Address of buffer                  */
              lg          ,            /* Number of bytes to be sent         */
              0             )        ; /* Flags, unused here                 */

    exit                             : /* Procedure exit label               */
                                     ;

#ifdef TEST                            /* Begin conditional compilation      */
    procfin("print_str")             ; /* Message                            */
#endif                                 /* End   conditional compilation      */

  END_PROC                             /* End of the procedure               */
