/*
**  lora.c - Sample code for HyperpanelOS =================================  **
**                                                                           **
**  This simple code is located into the application container, it is run    **
**  by the VMK sub-operating system. On the other hand, the I/O container    **
**  runs all the drivers that are VMIO finite state machines.                **
**                                                                           **
**  The goal of this app is to use the RAD driver for LoRa modem.            **
**                                                                           **
**  =======================================================================  **
*/


/* Documentation ...................-----------------------------------------

   The NUCLEO-WL55JC1 board is running a STM32WL55JC microcontroller. It can
   configure and use a LoRa modem.                                            

   Following informations from: STM RM0453 Reference manual

   Sub-GHz radio (SUBGHZ) (page 109)

   How to configure the Dragino LoRa Gateway?
     - Defines a new profile with:
       - Enable device supports OTAA
       - Set the MAC version to "LoRaWAN 1.0.4"
       - Disable device supports Class-B
       - Disable device supports Class-C
     - Add a new device with:
       - DevEUI = 2cf7f120420072df
       - JoinEUI = 8000000000000006
       - Application Key = 49fc9e07e85f7a8aa8654b551860a02b
     - To restart the Join procedure, we MUST flush OTAA device nonces on
       the "Applications/camping/Devices/your_device_name/OTAA Keys" tab.

*/
/* Include files and external reference -------------------------------------*/

#include <hypos.h>                     // Hyperpanel OS basic interfaces.    */
#include <drv_asy.h>                   // Prototype of "asy_write()".        */
#include <drv_rad.h>                   // Prototype of RAD driver interface  */

#define _JOIN_REQ_

extern void rad_show_params(unsigned int*, unsigned int*) ; // Prototype

/* Internal defines of this module ------------------------------------------*/

#define TICK                  1000     // Code for tick event


/*  Internal global variables of this module --------------------------------*/

    static unsigned int idto         ; // Timer identifier
    static          int cnt          ; // Counter of messages
    static          int rad_iod      ; // IOD of devive RAD\DEV0
    static          int rad_modd     ; // Module identifier of Radio driver

#ifdef _JOIN_REQ_
    static unsigned char join_req[] =  // Join-request frame
      {0x00,                           // MHDR
       0x06,0x00,0x00,0x00,0x00,0x00,  // JoinEUI  = 80:00:00:00:00:00:00:06
       0x00,0x80,                      //
       0xdf,0x72,0x00,0x42,0x20,0xf1,  // DevEUI   = 2C:F7:F1:20:42:00:72:DF
       0xf7,0x2c,                      //
       0x00,0x00,                      // DevNonce = 0000
       0xa2,0x0f,0x89,0xb9          }; // MIC      = A20F89B9
#endif

#ifdef _JOIN_ACC_
    static unsigned char join_acc[] =  // Join-accept frame
      {0x20,                           // MHDR
       0x96,0xcb,0x15,0x9b,0xf6,0x1a,  // Encrypted Join-accept frame
       0x0f,0x97,0xdb,0xac,0xed,0x37,  //
       0xf9,0xd4,0xbf,0xef,0xee,0x14,  //
       0xd7,0x25,0x83,0x51,0x4a,0xca,  //
       0x22,0xfd,0x36,0xfa,0x63,0xcb,  //
       0xcc,0xe9                    }; //
#endif


/*  Prototypes --------------------------------------------------------------*/

    static int  wait_evt(void)                  ; // Prototype
    static void write_log(char*)                ; // Prototype
    static void hexDump(char*,void*,int)        ; // Prototype
    static void val2str(unsigned char*,int,int) ; // Prototype
    static void show_date(void)                 ; // Prototype
    static void open_lora(void)                 ; // Prototype
    static void close_lora(void)                ; // Prototype
    static void write_lora(unsigned char*,int)  ; // Prototype
    static void read_lora(int,int)              ; // Prototype
    static void resp_write(void)                ; // Prototype
    static void resp_read(void)                 ; // Prototype


/*  Procedure loop_app_tsk --------------------------------------------------

    Purpose : This is our task main loop.
*/

int loop_app_tsk (void *param)
  {
    int             ev = TICK        ; // Our event
    int             nb               ; // Loop counter
    int             lock             ; // Lock for sending
    int             len              ; // Number of bytes to send
    unsigned char  *buf              ; // Address of the buffer to send
    char            mess[128]        ; // Message to be sent on ASY0


/* Start the LoRa tests application message .................................*/

    cnt      =  1                    ; // Init log message counter
    nb       =  0                    ; // Init loop counter

#if defined(_JOIN_REQ_)
    lock     =  0                    ; // Init write lock
    buf      =  join_req             ; // Address of the Join-request frame
    len      =  sizeof(join_req)     ; // Number of bytes to send

#elif defined(_JOIN_ACC_)
    lock     =  0                    ; // Init write lock
    buf      =  join_acc             ; // Address of the Join-accept frame
    len      =  sizeof(join_acc)     ; // Number of bytes to send 

#else
    lock     =  1                    ; // Init write lock
    buf      =  HNULL                ; // Unused
    len      =  0                    ; // Unused

#endif

    rad_modd = -1                    ; // Reset value
    rad_iod  = -1                    ; // Reset value

    set_vhalt_att(0,1)               ; // Enter in Debug mode

    strcpy(mess,"\r\nStart lora application\r\n"); // Start message.
    write_log(mess)                  ; // Write on ASY 

    show_date()                      ; // Display the current date


/* Open the LoRa modem device ...............................................*/

    open_lora()                      ; // Open the LoRa modem device
    read_lora(3, 128)                ; // Put 3 receive buffers of 128-bytes


/* Start a timer that will send an event every 2000 ms ......................*/

    wait_ev :                          // Beginning of loop label

    set_uto(TIMER      ,               // Timer mode: clock
            1000       ,               // Duration in milliseconds
            TICK , 0   ,               // Event code and reserve field
            &idto     )              ; // Timer identifier
    ev = wait_evt()                  ; // Unschedule until an event is received


/* Main loop ................................................................*/

    if (ev == TICK)                    // If the event is the tick event
      {                                //
        nb ++                        ; // One more loop
        show_date()                  ; // Display the current date
                                       //
        if ( rad_iod NE -1 &&          // If the LoRa modem device is oponed
             nb      GE 15    )        // Loop counter value is > 15 
          {                            //
//          close_lora()             ; // closes the LoRa modem device
          }                            //
                                       //
        if ( rad_iod    NE -1 &&       // LoRa modem device is oponed
             lock       EQ  0 &&       // Next write is enabled
             (nb  % 60) EQ 0     )     // Loop counter value every 60 sec
          {                            //
            lock = 1                 ; // Next write is locked
            write_lora(buf, len)     ; // Send the requested buffer
          }                            //
      }                                //
                                       //
    else                               // Telecoms Events
      {                                //
        clear_uto(idto)              ; // Clears the Timer
                                       //
        if ( ev == RESP_WRITE )        // If the event is the RESP_RPC event
          {                            //
            resp_write()             ; // Process the RESP_RPC event
            read_lora(0, 0)          ; // Enters RX mode
            lock = 0                 ; // Next write is unlocked
          }                            //

        if ( ev == RESP_READ )         // If the event is the RESP_RPC event
          {                            //
            clear_uto(idto)          ; // Clears the Timer
            resp_read()              ; // Process the RESP_RPC event
            read_lora(1, 128)        ; // Put 1 receive buffer of 128-bytes
          }                            //
      }                                //

    goto wait_ev                     ; // Wait for the next event

    return  0                        ; // Return code of the procedure 
  }



/*  Procedure wait_evt ------------------------------------------------------*/
/*
    Purpose : Unschedule until the next event is received, whatever it is.
*/

static int wait_evt (void)
  {
    unsigned int waitlist[1][3]      ; // Parameter of "waitevt_task"

/*****************************************************************************
 * Step 1 : Build a list with one WAIT_CODEINT entry that will accept all    *
 * ------   the event codes ranging from 0 to 20000. Then call               *
 *          "waitevt_task", we will be unscheduled until the next event will *
 *          be received                                                      *
 *****************************************************************************/

    waitlist[0][0]  = WAIT_CODEINT   ; // All events with
    waitlist[0][1]  = 0              ; // a code between 0
    waitlist[0][2]  = 20000          ; // and 20000

    waitevt_task(waitlist ,            // Address of waiting list
                 1        ,            // Size of "waitlist[]"
                 0        ,            // maximum waiting time = no
                 0        )          ; // Do not purge previous events

/*****************************************************************************
 * Step 2 : Here we are scheduled again. The VMK has written into its        *
 * ------   global variable "task_evt" a copy of the event that has          *
 *          scheduled us again.                                              *
 *****************************************************************************/

    return task_evt.code             ; // Return event code
  }

/*  Procedure write_log -------------------------------------------------------

    Purpose : 

*/

static void write_log(char *mess)
{
    asy_write(0,(unsigned char*)mess,strlen(mess));
}


/*  Procedure hexDump ---------------------------------------------------------

    Purpose : 

*/

static void hexDump(char *desc, void *addr, int len)
{
    int i                                      ;
    unsigned char *p   = (unsigned char*)addr-1;
    unsigned char buf[76]                      ;

    if (desc != HNULL)                     // Output description if given.
      {                                    // id order to be able
        hsprintf((char*)buf,"%s:\r\n", desc); // to make a difference
        write_log((char*)buf)               ; // Write message
      }

    memset(buf, 0x20, 76)                ; // Fill to blanck
    buf[ 0] = '\0'                       ; // Empty string
    buf[73] = '\r'                       ; // 
    buf[74] = '\n'                       ; //
    buf[75] = '\0'                       ; // End of string

    for ( i=0; i <= len; i++,p++ )
      {
        if ( (i%16) )
          {
            val2str(buf+3*(i%16)+2, *p, 2)    ;
            buf[(i%16)+56] = (*p<0x20) | (*p>0x7e)
                           ? '.' : *p         ;
          }

        else
          {
            if ( *buf ) 
              {
                val2str(buf+3*16+2, *p, 2)    ;
                buf[16+56] = (*p<0x20) | (*p>0x7e)
                           ? '.' : *p         ;
                write_log((char*)buf)         ;
                memset(buf, 0x20, 73)         ; // Fill to blanck
                buf[ 0] = '\0'                ; // Empty string
              }
            if ( i != len )
              val2str(buf+(i%16), i, 4)       ;
          }
      }
    if ( *buf ) write_log((char*)buf)         ;
}


/*  Procedure val2str ---------------------------------------------------------

    Purpose : 

*/

static void val2str(unsigned char *buf, int val, int nb)
{
    int i, dig;

    for ( i=0; i < nb; i++ )
      {
        dig = (val >> (4*(nb-1-i))) & 0xF;
        *(buf+i) = dig > 9 ? dig + 0x61 - 0x0A : dig + 0x30;
      }
}


/*  Procedure show_date -------------------------------------------------------

    Purpose : Display the current date/time

*/

static void show_date(void)
  {
    int             d,m,y,h,mi,s,ms  ; // Date and time
    char            mess[128]        ; // Message to be sent on ASY0

    tim_get(&d,&m,&y,&h,&mi,&s,&ms)  ; // Get date and time

    hsprintf(mess                ,     // We build in "mess" the message
           "%6d  %02d/%02d/%4d - %02d:%02d:%02d\r\n",
           cnt++,d,m,y,h,mi,s      ) ; //
    write_log(mess)                  ; // We send on ASY 

  }


/*  Procedure open_lora -------------------------------------------------------

    Purpose : Initialises, allocates and opens a LoRa modem device

*/

static void open_lora(void)
  {

    int            r                 ; // Returned code
    char           mess[128]         ; // Message to be sent on ASY

/*****************************************************************************
 * Step 1 : Initialize the Radio driver devices RAD.                         *
 * -------  We call the procedure "drv_init_driver()". This procedure returns*
 *          the Radio module identifier "rad_modd".                          *
 *****************************************************************************/

    r = drv_init_driver("RAD"   ,      // Name of the driver module
                        0       ,      // Node number = local (0)
                        1000    ,      // Reqid
                        &rad_modd)   ; // Module identifier MODD

    hsprintf(mess,"%6d drv_init_driver [%08X] %d\r\n",cnt++,rad_modd,r);
    write_log(mess);
    if ( r ) goto end                ; // On error,


/*****************************************************************************
 * Step 2 : Allocate the Radio driver device RAD\RAD1.                       *
 * -------  We call the procedure "drv_alloc_device()" with the flags        *
 *          NBLOCKING_IO set. This procedure returns the IOD "rad_iod" of the*
 *          device.                                                          *
 *****************************************************************************/

    r = drv_alloc_device(rad_modd  ,   // Module Descriptor Id
                       "RAD\\RAD1" ,   // Device name = name
                       ""          ,   // Tagini  = empty
                       ""          ,   // Taglist = empty
                       NBLOCKING_IO,   // Flags   = 0 or NBLOCKING_IO
                       1001        ,   // Reqid
                       0           ,   // Route ID   = empty
                       0           ,   // Route ID 2 = empty
                       &rad_iod     ); // Returned I/O descriptor

    hsprintf(mess,"%6d drv_alloc_device [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess);
    if ( r ) goto end                ; // On error,


/*****************************************************************************
 * Step 3 : Open the allocated Radio device to use it.                       *
 * -------  We call the procedure "drv_open_device()" with the tag list      *
 *          "MODEM=LORA". This tag list defines only tag MODEM which         *
 *          defines the radio modem to select.                               *
 *****************************************************************************/

    r = drv_open_device(rad_iod     ,  // I/O descriptor
                        "MODEM=LORA",  // Select the LoRa Modem       
                        1002        ); // Reqid

    hsprintf(mess,"%6d drv_open_device [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess);


    end : if ( r ) vhalt (r)         ; // Debug

  }


/*  Procedure close_lora ------------------------------------------------------

    Purpose : Closes, frees and terminates a LoRa modem device

*/

static void close_lora(void)
  {

    int            r                 ; // Returned code
    char           mess[128]         ; // Message to be sent on ASY

/*****************************************************************************
 * Step 1 : Closes the allocated Radio device.                               *
 * -------  We call the procedure "drv_close_device()" with IOD value of the *
 *          radio device.                                                    *
 *****************************************************************************/

    r = drv_close_device(rad_iod ,      // I/O descriptor
                         1000     )   ; // Reqid

    hsprintf(mess,"%6d drv_close_device [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess);
    if ( r ) goto end                ; // On error,


/*****************************************************************************
 * Step 2 : Free the Radio driver device RAD\RAD1.                           *
 * -------  We call the procedure "drv_free()" with the IOD value of the     *
 *          radio device.                                                    *
 *****************************************************************************/

    r = drv_free(rad_iod  ,            // Module Descriptor Id
                 1001      )         ; // Reqid

    hsprintf(mess,"%6d drv_free [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess);
    if ( r ) goto end                ; // On error,

    rad_iod = -1                     ; // Reset value


/*****************************************************************************
 * Step 1 : Terminates the Radio driver devices RAD.                         *
 * -------  We call the procedure "drv_end()" with the Radio module          *
 *          identifier "rad_modd".                                           *
 *****************************************************************************/

    r = drv_end(rad_modd ,             // Module identifier MODD
                1002      )          ; // Reqid

    hsprintf(mess,"%6d drv_end [%08X] %d\r\n",cnt++,rad_modd,r);
    write_log(mess);

    rad_modd = -1                    ; // Reset value

    end : if ( r ) vhalt (r)         ; // Debug

  }


/*  Procedure write_lora ------------------------------------------------------

    Purpose : Write a PHYPayload.

*/

static void write_lora(unsigned char *buf, int len)
  {

    int            r                 ; // Returned code
    char           mess[128]         ; // Message to be sent on ASY

/*****************************************************************************
 * Step 1 : Configure radio TX parameters                                    *
 * -------  We call the procedure "drv_setval()" with IOD value of the radio *
 *          device.                                                          *
 *            - MODE       = STANDBY                                         *
 *            - FREQ       = 868300000                                       *
 *            - SF         = 12                                              *
 *            - BM         = 4                                               *
 *            - CR         = 1                                               *
 *            - PREAMBLEN  = 8                                               *
 *            - HEADTYP    = EXP                                             *
 *            - SYNCWRDLEN = 52                                              *
 *            - PAYLOADLEN = 23 (Number of bytes to send)                    *
 *            - CRCEN      = ON                                              *
 *            - INVIQ      = OFF                                             *
 *            - TXTIMEOUT  = 0                                               *
 *            - CMD        = TXCONF                                          *
 *****************************************************************************/

    r = drv_setval(rad_iod        ,    // I/O descriptor
                   "MODE=STANDBY\n"    //
                   "FREQ=868300000\n"  //
                   "SF=12\n"           //
                   "BW=4\n"            //
                   "CR=1\n"            //
                   "PREAMBLEN=8\n"     //
                   "HEADTYP=EXP\n"     //
                   "SYNCWRDLEN=52\n"   //
#ifdef _JOIN_REQ_
                   "PAYLOADLEN=23\n"   //
#else
                   "PAYLOADLEN=33\n"   //
#endif
                   "CRCEN=ON\n"        //
                   "INVIQ=OFF\n"       //
                   "POWER=16\n"        //
                   "TXTIMEOUT=0\n"     //
                   "CMD=TXCONF"   ,    //
                   1000            ) ; // Reqid

    hsprintf(mess,"%6d drv_setval [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess)                  ;
    if ( r < 0 ) goto end            ; // On failure


/*****************************************************************************
 * Step 2 : Write a PHYPayload.                                              *
 * -------  We call the procedure "drv_write()" with IOD value of the radio  *
 *          device.                                                          *
 *****************************************************************************/

    if ( len GT 0 )                    // Display data to send
      {                                //
        hexDump("send [PHYPayload]",   //
                buf,len            ) ; //
      }                                //

     r = drv_write(rad_iod   ,         // I/O descriptor
                   buf       ,         // Buffer address
                   0         ,         // Offset of the 1st byte to send
                   len       ,         // Number of bytes to send
                   USER_ALLOC,         // User provided
                   1001       )      ; // Reqid

     hsprintf(mess,"%6d drv_write [%08X] %d\r\n",cnt++,rad_iod,r);
     write_log(mess);

     end : if ( r ) vhalt (r)        ; // Debug

  }


/*  Procedure read_lora -------------------------------------------------------

    Purpose : Put received buffers.

*/

static void read_lora(INT nb, INT len)
{
    int             r                ; // Error code
    int             i                ; // Loop counter
    int             of7, l           ; // Firts byte in buffer (unused)
    int             lg               ; // Count of bytes received / Error code
    unsigned char  *adr              ; // Buffer address 
    char            mess[128]        ; // Message to be sent on ASY2
   
/*****************************************************************************
 * Step 1 : Put "nb" received buffer of "len" bytes.                         *
 * -------                                                                   *
 *****************************************************************************/

    for ( i = 0; i < nb; i++ )          // Allocates "nb" buffers
      {
        adr = HNULL                   ; // No buffer address, it will be
                                       // allocated by "drv_read"
        r   = drv_read(rad_iod ,        //
                       len     ,        // Maximum number of bytes / buffer
                       0       ,        // Flags
                       &adr    ,        // Buffer address
                       &l      ,        // Allocated buffer address
                       &of7    ,        // Offset (unused)
                       &lg     ,        // Number of bytes read (unused)
                       1000     )     ; // Reqid

        if ( r )                        // On failure
          { 
            hsprintf(mess,"%6d drv_read [%08X] %d\r\n",cnt++,rad_iod,r);
            write_log(mess);
            goto end                  ; //
          }
      }


/*****************************************************************************
 * Step 2 : Configure radio RX parameters                                    *
 * -------  We call the procedure "drv_setval()" with IOD value of the radio *
 *          device.                                                          *
 *          Setting the mandatories LoRa modem parameters:                   *
 *            - MODE       = STANDBY                                         *
 *            - FREQ       = 868300000                                       *
 *            - SF         = 12                                              *
 *            - BM         = 4                                               *
 *            - CR         = 1                                               *
 *            - PREAMBLEN  = 8                                               *
 *            - HEADTYP    = EXP                                             *
 *            - PAYLOADLEN = 63 (Max payload size for SF12/125 + overHead)   *
 *            - CRCEN      = OFF                                             *
 *            - INVIQ      = ON                                              *
 *            - RXTIMEOUT  = 0                                               *
 *            - SYMBNUM    = 0                                               *
 *            - CMD        = RXCONF                                          *
 *****************************************************************************/

    r = drv_setval(rad_iod        ,    // I/O descriptor
                   "MODE=STANDBY\n"    //
//                 "FREQ=868300000\n"  //
                   "FREQ=869525000\n"  //
                   "SF=12\n"           //
                   "BW=4\n"            //
                   "CR=1\n"            //
                   "PREAMBLEN=8\n"     //
                   "HEADTYP=EXP\n"     //
                   "PAYLOADLEN=63\n"   // DR_0 LoRa: SF12, 125 kHz + OverHead
                   "CRCEN=OFF\n"       //
                   "INVIQ=ON\n"        //
                   "RXTIMEOUT=0\n"     //
                   "SYMBNUM=0\n"       //
                   "CMD=RXCONF"   ,    //
                   1000            ) ; // Reqid

    hsprintf(mess,"%6d drv_setval [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess)                  ; //
    if ( r ) goto end                ; // On failure


/*****************************************************************************
 * Step 3 : Start the continious radio RX mode.                              *
 * -------  We call the procedure "drv_setval()" with IOD value of the radio *
 *          device.                                                          *
 *          Setting the mandatory LoRa modem parameter:                      *
 *            - MODE = RX                                                    *
 *****************************************************************************/

    r = drv_setval(rad_iod     ,       // I/O descriptor
                   "MODE=RX"   ,       // Start the RX radio mode
                   1001         )    ; // Reqid

    hsprintf(mess,"%6d drv_setval [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess)                  ;

    end : if ( r ) vhalt (r)         ; // Debug
}


/*  Procedure resp_write ------------------------------------------------------

    Purpose : End of sending buffer

*/

static void resp_write(void)
  {

    int            r                 ; // Returned code
    char           mess[128]         ; // Message to be sent on ASY

/*****************************************************************************
 * Step 1 : Check the returned error code.                                   *
 * -------                                                                   *
 *****************************************************************************/

    r = task_evt.length              ; // Error code

    hsprintf(mess,"%6d RESP_WRITE event [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess)                  ; // Display message

    if ( r < 0 ) goto end            ; // Debug


/*****************************************************************************
 * Step 2 : Force the sub-GHz radio in Sleep mode.                           *
 * -------                                                                   *
 *****************************************************************************/

    r = drv_setval(rad_iod       ,    // I/O descriptor
                   "MODE=SLEEP"  ,    //
                   1000           ) ; // Reqid

    hsprintf(mess,"%6d drv_setval [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess)                  ; //

    end : if ( r < 0 ) vhalt (r)     ; // Debug

  }


/*  Procedure resp_read -------------------------------------------------------

    Purpose : Receive a buffer

*/

static void resp_read(void)
  {

    int             r                ; // Returned code
    unsigned char  *buf              ; // Point to received buffer address
    Buf_head       *hd               ; // Header address of buffer
    char            mess[128]        ; // Message to be sent on ASY
    S_evt           evt              ; // Event received

/*****************************************************************************
 * Step 1 : Check the returned error code.                                   *
 * -------                                                                   *
 *****************************************************************************/

    evt = task_evt                   ; // Copy the current event
    buf = evt.address                ; // Address of received buffer
    r   = evt.length                 ; // Error code

    hsprintf(mess,"%6d RESP_READ event [%08X] %d\r\n",cnt++,rad_iod,r);
    write_log(mess)                  ; // Display message

    if ( r < 0 ) goto end            ; // Debug
 

/*****************************************************************************
 * Step 2  Display data buffer received in RESP_READ Event                   *
 * -------                                                                   *
 *****************************************************************************/

    hd  = (Buf_head*)buf - 1         ; // Point to buffer header

    hexDump("Received bytes",
            buf+hd->pos, evt.length) ;  // Display received data

    end :

    if ( buf ) free_buf(buf, 0 )     ; // Free receive buffer
//  if ( r < 0 ) vhalt (r)           ; // Debug

  }

